/* callbacks.c - controls GTK callback functions (excluding network) */

#include <unistd.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <signal.h>

#include <ctype.h>

#include "xtext.h"
#include "napster.h"
#include "gnapster.h"
#include "callbacks.h"
#include "ui.h"
#include "upload.h"
#include "network.h"
#include "commands.h"
#include "queue.h"
#include "download.h"
#include "servers.h"
#include "chan.h"
#include "xtext.h"
#include "text.h"
#include "utils.h"
#include "themes.h"
//#include "menus.h"

#include "debug.h"

#include <gdk/gdkx.h>

#ifndef HAVE_GNOME
#include <gdk/gdkkeysyms.h>
#endif /* !HAVE_GNOME */

extern ConnInfo conn_info;
extern UserInfo user_info;
extern GnapsterMain gnapster_main;
extern SearchTab search_tab;
extern BrowseTab browse_tab;
extern DownloadTab download_tab;
extern UploadTab upload_tab;
extern ConsoleTab console_tab;
extern MOTDTab motd_tab;
extern PropDlg prop_dlg;
extern BuildDlg build_dlg;
extern BrowseDlg browse_dlg;
extern AddDlg add_dlg;
extern EditDlg edit_dlg;
extern ListChanDlg list_chan_dlg;
extern ResumeDlg resume_dlg;
extern RefreshDlg refresh_dlg;

extern int building;
extern GList *total_list;
extern GList *ignore_list;
extern GList *hooks;

extern int na_err;

int nick_curr = 0, nick_end = 0;
GList *nick_history = NULL;

int loc_curr = 0, loc_end = 0;
GList *console_history = NULL;

#ifndef HAVE_GNOME
/* i really dont like how this works, oh well */
void *realfunc[] = {
   search_download_cb, browse_user_menu_cb, whois_user_menu_cb,
   browse_download_cb, retry_download_cb, cancel_download_cb,
   cancel_remove_cb, cancel_upload_cb, force_queue_cb, cancel_queue_cb,
   cancel_movequeue_cb
};

void thunk(gpointer data, guint function, GtkWidget *item) {
   void *data_pass;
   
   data_pass = gtk_item_factory_popup_data_from_widget(item);
   GTK_SIGNAL_FUNC(realfunc[function])(item, data_pass);
}
#endif /* !HAVE_GNOME */

void browse_clear(GtkWidget *w) {
   gtk_clist_clear(GTK_CLIST(browse_tab.ctree));
}

/* hooray for xchat */
int property_notify(GtkWidget *w, GdkEventProperty *event, void *data) {
   static Atom prop = None;
   
   if (prop == None)
     prop = XInternAtom(GDK_DISPLAY(), "_XROOTPMAP_ID", True);
   
   if (event->atom == prop)
     update_xtext(NULL);
   
   return FALSE;
}

void gnapster_exit_cb() {
   int w = -1, h = -1;

   if (connected()) {
      CLOSE(conn_info.sock);
      CLOSE(conn_info.bind_sock);
      
      INPUT_REMOVE(conn_info.sock_input);
      INPUT_REMOVE(conn_info.bind_sock_input);
   }

   gdk_window_get_size(gnapster_main.window->window, &w, &h);
   j_config_set_int("/gnapster/Geometry/w", w);
   j_config_set_int("/gnapster/Geometry/h", h);
   j_config_sync();
   
   gtk_main_quit();
}

void safe_exit_check() {
   if (!safe_exit())
     return;
   
   if (!GTK_CLIST(upload_tab.clist)->rows &&
       !GTK_CLIST(download_tab.clist)->rows &&
       !GTK_CLIST(download_tab.queue_clist)->rows)
     gnapster_exit_cb();
}

void safe_exit_cb() {
   disconnect_cb();
   
   gnapster_update_status("Safe exit mode activated...finishing all pending transfers", 0);
   
   conn_info.state |= SAFE_EXIT_MASK;
   
   safe_exit_check();
}

void set_trans(GtkWidget *text) {
   gtk_xtext_set_background(GTK_XTEXT(text), NULL, user_info.conf[TINT],
			    user_info.conf[TINT]);
   
   GTK_XTEXT(text)->tint_red = user_info.conf[TINT_RED];
   GTK_XTEXT(text)->tint_green = user_info.conf[TINT_GREEN];
   GTK_XTEXT(text)->tint_blue = user_info.conf[TINT_BLUE];

   if (GTK_XTEXT(text)->transparent) {
      gtk_xtext_free_trans(GTK_XTEXT(text));
      gtk_xtext_load_trans(GTK_XTEXT(text));
   }
   
   gtk_xtext_render_page(GTK_XTEXT(text), GTK_XTEXT(text)->adj->value);
}

void update_xtext(GtkWidget *w) {
   GList *ptr;
   
   set_trans(console_tab.text);
   
   /* update all other widgets */
   for(ptr=console_tab.channel_list; ptr; ptr=ptr->next)
     set_trans(((ChannelInfo *)ptr->data)->text);
}

void browse_font_ok_cb(GtkWidget *w, GtkWidget *dlg) {
   char *font;
   
   font = gtk_font_selection_dialog_get_font_name(GTK_FONT_SELECTION_DIALOG(dlg));
   
   if (prop_dlg.font_entry && font && strlen(font))
     gtk_entry_set_text(GTK_ENTRY(prop_dlg.font_entry), font);
   
   gtk_widget_destroy(dlg);
}

void browse_font_cancel_cb(GtkWidget *w, GtkWidget *dlg) {
   gtk_widget_destroy(dlg);
}

void browse_font_cb() {
   GtkWidget *dlg;
   
   dlg = gtk_font_selection_dialog_new(_("Select Font"));
   
   gtk_signal_connect(GTK_OBJECT(GTK_FONT_SELECTION_DIALOG(dlg)->ok_button), "clicked",
		      GTK_SIGNAL_FUNC(browse_font_ok_cb), dlg);
   gtk_signal_connect(GTK_OBJECT(GTK_FONT_SELECTION_DIALOG(dlg)->cancel_button), "clicked",
		      GTK_SIGNAL_FUNC(browse_font_cancel_cb), dlg);
   
   gtk_font_selection_dialog_set_font_name(GTK_FONT_SELECTION_DIALOG(dlg),
					   gtk_entry_get_text(GTK_ENTRY(prop_dlg.font_entry)));
   
   gtk_widget_show(dlg);
}

void update_shares_cb() {
   shared_list_handle(user_info.upload_dir, UPDATE_LIST);   
}

void connect_last_cb() {
   if (connected() || connecting())
     already_connected(NULL);
   else
     connect_cb();
}

void already_connected_cb(gint reply, void *data) {
   int treply;
   Server *nsrv;
   
   if (!data)
     return;
   
#ifdef HAVE_GNOME
   treply = reply;
#else
   treply = data ? DIALOG_YES : DIALOG_NO;
#endif /* HAVE_GNOME */
   
   nsrv = (Server *)data;
   if (!nsrv)
     return;
   
   if (treply == DIALOG_YES) {
      disconnect_cb();
      if (strcmp(nsrv->network, "connect_cb_regular")) {
	 Server *nsrv;
	 
	 nsrv = (Server *)data;
	 browse_dlg_connect_cb(NULL, nsrv);
      } else {
	 connect_cb();
	 server_free(nsrv);
      }
   } else if (treply != DIALOG_NO) {
      g_assert_not_reached();
   }
}

void already_connected(Server *nsrv) {
   GtkWidget *dialog;
   
   if (!nsrv) {
      Server *srv;
      
      srv = d_new(N_SERVER);
      srv->network = d_strdup("connect_cb_regular");
      
      nsrv = srv;
   }
   
   dialog = j_question_dialog(_("You're already connected to a server. Disconnect?"),
			      already_connected_cb,
			      nsrv,
			      GTK_WINDOW(gnapster_main.window));
}

void connect_official_cb() {
   j_config_set_string("/gnapster/Options/server", SERVER_NAPSTER_COM);
   j_config_set_int("/gnapster/Options/server_port", 8875);
   j_config_sync();
   if (connected() || connecting()) {
      already_connected(NULL);
      return;
   }
   connect_cb();
}

void switch_page_cb(GtkWidget *widget, GtkNotebookPage *page, int page_num,
		    gpointer data) {
   GtkWidget *nth_page, *label;
   GtkStyle *old_style;
   
   if (page_num == 4 || widget == console_tab.notebook)
     gtk_widget_grab_focus(console_tab.entry);
   
   nth_page = gtk_notebook_get_nth_page(GTK_NOTEBOOK(widget), page_num);
   label = gtk_notebook_get_tab_label(GTK_NOTEBOOK(widget), nth_page);
   old_style = gtk_widget_get_style(label);
   if (old_style == gnapster_main.highlight)
     gtk_widget_set_style(label, gnapster_main.normal);
}

void build_list_cb() {
   if (building) {
      building = 0;
      return;
   }

   shared_list_handle(gtk_entry_get_text(GTK_ENTRY(j_file_entry_gtk_entry(prop_dlg.upload_dir))),
		      BUILD_LIST);
}

void update_list_cb() {
   shared_list_handle(gtk_entry_get_text(GTK_ENTRY(j_file_entry_gtk_entry(prop_dlg.upload_dir))),
		      UPDATE_LIST);
}

void query_user_cb(GtkWidget *w, gpointer data) {
   gtk_notebook_set_page(GTK_NOTEBOOK(gnapster_main.notebook), 4);
   gtk_entry_set_text(GTK_ENTRY(console_tab.entry), "/query ");
   gtk_widget_grab_focus(console_tab.entry);
}

void unquery_user_cb(GtkWidget *w, gpointer data) {
   if (!remove_console_tab(NULL, 1))
     j_error_dialog("You must click on a user query tab before attempting to close it");
}

void menu_popup_cb(GtkWidget *widget, GdkEventButton *event, gpointer data) {
   GtkWidget *temp_menu;
   int row = -1, col;

   gtk_clist_get_selection_info(GTK_CLIST(widget), event->x, event->y,
				&row, &col);

   if (event->type == GDK_2BUTTON_PRESS && event->button == 1 &&
       widget == search_tab.clist) {
      Search *temp;
      Transfer *download;
      
      temp = gtk_clist_get_row_data(GTK_CLIST(widget), row);
      if (!temp || !temp->user) 
	return;

      download = search_dup(temp, TRANSFER);
      handle_download(download, DOWNLOAD_NORMAL);
   } else if (event->type == GDK_2BUTTON_PRESS && event->button == 1 &&
	      widget == browse_tab.ctree) {
//      browse_download_cb(browse_tab.ctree, NULL);
   } else if (event->button == 3) {
#ifdef HAVE_GNOME
      temp_menu = gnome_popup_menu_new(data);
      gnome_popup_menu_do_popup_modal(temp_menu, NULL, NULL, event, widget);
      gtk_widget_destroy(temp_menu);
#else
      UNUSED(temp_menu);
      gtk_item_factory_popup_with_data(GTK_ITEM_FACTORY(data), widget, NULL, 
				       (guint)event->x_root, (guint)event->y_root,
				       event->button, event->time);
#endif /* HAVE_GNOME */
   }
}

void console_activate_cb() {
   char *console_text = NULL, *command, *message;
   
   console_text = gtk_entry_get_text(GTK_ENTRY(console_tab.entry));
   if (!console_text || !connected()) 
     return;
   
   if (!strlen(console_text)) 
     return;
   
   if (loc_end > MAX_HISTORY) {
      GList *link;
      
      link = g_list_nth(console_history, 0);
      console_history = g_list_remove_link(console_history, link);
      d_free(link->data);
      loc_end--;
   }
   console_history = g_list_append(console_history, d_strdup(console_text));
   loc_end++;
   loc_curr = loc_end;

   if (console_text[0] != '/') {
      do_sendtext(NULL, d_strdup(console_text));
      gtk_entry_set_text(GTK_ENTRY(console_tab.entry), "");
      return;
   }
   
   command = next_arg(console_text, &console_text);
   if (!command) {
      gtk_entry_set_text(GTK_ENTRY(console_tab.entry), "");
      return;
   }
   message = last_arg(console_text, &console_text);
   
   NA_RESET();

   if (!handle_command(command, message))
     hook_text_insert(CURR, SYSTEM, "unknown_cmd", "%s",
		      command);
   
   gtk_entry_set_text(GTK_ENTRY(console_tab.entry), "");
}

GList *add_nick_history(GList *nh, char *user) {
   if (nick_end > MAX_HISTORY) {
      GList *link;

      link = g_list_nth(nh, 0);
      nh = g_list_remove_link(nh, link);

      d_free(link->data);

      nick_end--;
   }

   nh = remove_nick(nh, user);

   nh = g_list_append(nh, d_strdup(user));

   nick_end = g_list_length(nh);
   nick_curr = nick_end;
   
   return nh;
}

int console_keypress_cb(GtkWidget *w, GdkEventKey *key) {
   GList *ptr;
   
   if (key->keyval != GDK_Tab)
     nick_curr = nick_end;

   if (key->keyval >= GDK_1 && key->keyval <= GDK_9 && 
       key->state & GDK_MOD1_MASK) {
      gtk_notebook_set_page(GTK_NOTEBOOK(console_tab.notebook),
			    (key->keyval - 49));
      return 1;
   }
   
   if (key->keyval == GDK_Tab) {
      int page_num, x;
      ChannelInfo *chan;
      GList *nicks;
      GCompletion *comp;
      char *text, *lastword, *cnick, *strbuf, *npfx;
      
      gtk_signal_emit_stop_by_name(GTK_OBJECT(w), "key_press_event");
      
      text = gtk_entry_get_text(GTK_ENTRY(console_tab.entry));
      if (!strlen(text) || nick_curr != nick_end) {
	 char *last_nick, *buf;
	 
	 nick_curr--;
	 
	 if (nick_curr < 0)
	   nick_curr = (nick_end - 1);
	 
	 last_nick = g_list_nth_data(nick_history, nick_curr);
	 
	 if (last_nick) {
	    buf = d_strdup_printf(DEBUG "/m %s ",
				  last_nick);
	    gtk_entry_set_text(GTK_ENTRY(console_tab.entry), buf);
	    d_free(buf);
	 }

	 return 1;
      }
      
      page_num = gnapster_get_pagenum();
      
      if (page_num <= 1) 
	return 1;
      
      chan = g_list_nth_data(console_tab.channel_list,
			     page_num - 2);
      if (chan->user)
	return 1;
      
      lastword = strrchr(text, ' ');
      if (!lastword)
	lastword = text;
      else
	lastword++;
      
      if (lastword)
	lastword = d_strdup(all_lower(lastword));
      
      if (!lastword)
	return 1;
      
      comp = g_completion_new(nick_comp_func);
      g_completion_add_items(comp, GTK_CLIST(chan->clist)->row_list);
      
      nicks = g_completion_complete(comp, lastword, &npfx);
      if (npfx)
	d_hunk_add(npfx, strlen(npfx));
      x = strlen(lastword);
      d_free(lastword);
      if (!nicks || !npfx) {
	 g_completion_free(comp);
	 return 1;
      }
      
      if (g_list_length(nicks) > 1) {
	 GList *ptr;
	 GtkCListRow *row;
	 User *user;
	 int c;
	 
	 c = 0;
	 
	 for(ptr=nicks; ptr; ptr=ptr->next) {
	    row = ptr->data;
	    if (!row)
	      continue;
	    
	    user = row->data;
	    if (!user)
	      continue;
	    
	    hook_text_insert(CURR, (c == 0) ? SYSTEM : MESSAGE, "user_list",
			     "%-20s", user->user);
	    
	    c++;
	    if (c > 1 || !ptr->next) {
	       gnapster_text_insert(CURR, MESSAGE, "\n");
	       c = 0;
	    }
	 }
	 
	 d_free(npfx);
	 g_completion_free(comp);
	 
	 return 1;
      }
      
      cnick = ((User *)((GtkCListRow *)g_list_nth_data(nicks, 0))->data)->user;
      if (!cnick) {
	 d_free(npfx);
	 g_completion_free(comp);
	 return 1;
      }
      
      text[strlen(text) - x] = 0;
      
      strbuf = d_strdup_printf(DEBUG "%s%s ", text, cnick);
      gtk_entry_set_text(GTK_ENTRY(console_tab.entry), strbuf);
      d_free(strbuf);
      
      g_completion_free(comp);
      d_free(npfx);
      
      return 1;
   }
   
   if (key->keyval != GDK_Up && key->keyval != GDK_Down)
     return 0;
   
   switch(key->keyval) {
    case GDK_Up:
      loc_curr--;
      if (loc_curr < 0) loc_curr = 0;
      break;
    case GDK_Down:
      loc_curr++;
      if (loc_curr > loc_end) loc_curr = loc_end;
      break;
   }
   ptr = g_list_nth(console_history, loc_curr);
   if (ptr) {
      char *text;
      
      text = ptr->data;
      if (text)
	gtk_entry_set_text(GTK_ENTRY(w), text);
   } else if (loc_curr == loc_end) {
      gtk_entry_set_text(GTK_ENTRY(w), "");
   }
	
   gtk_signal_emit_stop_by_name(GTK_OBJECT(w), "key_press_event");
   
   return 1;
}

Search *get_data(const void *row) {
   return ((GtkCListRow *)row)->data;
}

User *get_chan_data(const void *row) {
   return ((GtkCListRow *)row)->data;
}

int compare_chan_user(GtkCList *clist, const void *row1, const void *row2) {
   User *s1 = get_chan_data(row1), *s2 = get_chan_data(row2);
   if (!s1 || !s2)
     return 0;
   
   return g_strcasecmp(s1->user, s2->user);
}

int compare_chan_conn(GtkCList *clist, const void *row1, const void *row2) {
   User *s1 = get_chan_data(row1), *s2 = get_chan_data(row2);
   if (!s1 || !s2)
     return 0;
   
   if (s1->conn <= s2->conn) return 1;
   if (s1->conn > s2->conn) return -1;
   
   return 0;
}

int compare_chan_shares(GtkCList *clist, const void *row1, const void *row2) {
   User *s1 = get_chan_data(row1), *s2 = get_chan_data(row2);
   if (!s1 || !s2) 
     return 0;
   
   if (s1->shares <= s2->shares) return 1;
   if (s1->shares > s2->shares) return -1;
   
   return 0;
}

void channel_sort(GtkCList *clist, int column) {
   void *sortfunc[3] = {
      compare_chan_user, compare_chan_shares, compare_chan_conn
   };
   
   if (column >= 3 || column < 0)
     return;
   
   gtk_clist_set_compare_func(clist, sortfunc[column]);
   
   if (clist->sort_column == column) {
      clist->sort_type = (clist->sort_type == GTK_SORT_ASCENDING) ? 
			 GTK_SORT_DESCENDING : GTK_SORT_ASCENDING;
   }
   
   gtk_clist_set_sort_column(clist, column);
   gtk_clist_sort(clist);
}

int compare_file(GtkCList *clist, const void *row1, const void *row2) {
   Search *s1 = get_data(row1), *s2 = get_data(row2);
   
   if (!s1 || !s2)
     return 0;
   
   return g_strcasecmp(s1->trunc_file, s2->trunc_file);
}

int compare_user(GtkCList *clist, const void *row1, const void *row2) {
   Search *s1 = get_data(row1), *s2 = get_data(row2);
   
   if (!s1 || !s2)
     return 0;
   
   return g_strcasecmp(s1->user, s2->user);
}

int compare_size(GtkCList *clist, const void *row1, const void *row2) {
   Search *s1 = get_data(row1), *s2 = get_data(row2);
   
   if (!s1 || !s2)
     return 0;
   
   if (s1->size <= s2->size) return 1;
   if (s1->size > s2->size) return -1;
   
   return 1;
}

int compare_length(GtkCList *clist, const void *row1, const void *row2) {
   Search *s1 = get_data(row1), *s2 = get_data(row2);
   int seconds1, seconds2;
   
   if (!s1 || !s2)
     return 0;
   
   seconds1 = (s1->size / ((s1->kbps * 1000) / 8));
   seconds2 = (s2->size / ((s2->kbps * 1000) / 8));
   
   if (seconds1 <= seconds2) return 1;
   if (seconds1 > seconds2) return -1;
   
   return 1;
}

int compare_bitrate(GtkCList *clist, const void *row1, const void *row2) {
   Search *s1 = get_data(row1), *s2 = get_data(row2);
   
   if (!s1 || !s2) 
     return 0;
   
   if (s1->kbps <= s2->kbps) return 1;
   if (s1->kbps > s2->kbps) return -1;
   
   return 0;
}

int compare_connection(GtkCList *clist, const void *row1, const void *row2) {
   Search *s1 = get_data(row1), *s2 = get_data(row2);
   
   if (!s1 || !s2)
     return 0;
   
   if (s1->connection <= s2->connection) return 1;
   if (s1->connection > s2->connection) return -1;
   
   return 0;
}

void search_sort(GtkCList *clist, int column) {
   void *sortfunc[6] = {
      compare_file, compare_connection, compare_user, compare_size, 
      compare_length, compare_bitrate
   };
   
   if (column >= 6 || column < 0)
     return;

   gtk_clist_set_compare_func(clist, sortfunc[column]);

   if (clist->sort_column == column) {
      clist->sort_type = (clist->sort_type == GTK_SORT_ASCENDING) ?
		    	 GTK_SORT_DESCENDING : GTK_SORT_ASCENDING;
   }
   
   gtk_clist_set_sort_column(clist, column);
   
   gtk_clist_sort(clist);
}

void end_search(char *msg) {
   char *str = NULL;
   
   if (msg)
     str = d_strdup_printf(DEBUG msg,
			   gtk_entry_get_text(GTK_ENTRY(search_tab.gtk_entry)));

   gnapster_update_status(conn_info.status, 0);

   gtk_entry_set_text(GTK_ENTRY(search_tab.gtk_entry), "");
   gtk_entry_set_text(GTK_ENTRY(search_tab.gtk_not_entry), "");

   gtk_entry_set_editable(GTK_ENTRY(search_tab.gtk_entry), 1);

   gtk_clist_sort(GTK_CLIST(search_tab.clist));
   gtk_clist_thaw(GTK_CLIST(search_tab.clist));
   
   if (msg) {
      if (!g_list_length(GTK_CLIST(search_tab.clist)->row_list))
	gnapster_clist_append(search_tab.clist, search_tab.vbox, NULL,
			      str, "", "", "", "", "", NULL);

      d_free(str);
   }
   
   conn_info.state &= ~SEARCHING_MASK;
}

void cancel_search_cb() {
   if (!searching())
     return;
   
   cancels_inc(1);
   
   end_search(NULL);
}

void search_activate_cb() {
   char *not_string, *string;
   char extra_optns[1024], temp[1024];
   char *bps_table[] = {
      "96", "112", "128", "160", "192", NULL
   };
   
   if (!connected() || searching())
     return;
   
   string = gtk_entry_get_text(GTK_ENTRY(search_tab.gtk_entry));
   if (!string || !(*string))
     return;
   
   string = d_strdup(string);
   
   not_string = gtk_entry_get_text(GTK_ENTRY(search_tab.gtk_not_entry));
   if (!not_string)
     return;
   
   not_string = d_strdup(not_string);

   memset(extra_optns, 0, sizeof(extra_optns));
   
   gnapster_update_status(_("Searching..."), 1);
   
   conn_info.state |= SEARCHING_MASK;
   
   if (user_info.conf[LINE_SPEED_SWITCH]) {
      char *cmp, *ptr;
      
      cmp = d_strdup(listint_to_str(user_info.conf[LINE_SPEED_CMP], 0));
      ptr = cmp;
      while(*ptr) {
	 *ptr = toupper(*ptr);
	 ptr++;
      }
      
      g_snprintf(temp, sizeof(temp), " LINESPEED \"%s\" %i", cmp, user_info.conf[LINE_SPEED]);
      strcat(extra_optns, temp);
      d_free(cmp);
   }
   
   if (user_info.conf[BITRATE_SWITCH]) {
      char *cmp, *ptr, *bit;
      
      cmp = d_strdup(listint_to_str(user_info.conf[BITRATE_CMP], 0));
      
      for(ptr=cmp; *ptr; ptr++)
	*ptr = toupper(*ptr);

      bit = bps_table[user_info.conf[BITRATE]];
      
      g_snprintf(temp, sizeof(temp), " BITRATE \"%s\" \"%s\"", cmp, bit);
      
      strcat(extra_optns, temp);
      
      d_free(cmp);
   }
   
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(search_tab.local_only)))
     strcat(extra_optns, " LOCAL_ONLY");
   
   if (*not_string) {
      char buf[1024];
      
      g_snprintf(buf, sizeof(buf), " FILENAME EXCLUDES \"%s\"", not_string);
      strcat(extra_optns, buf);
   }

   g_strdown(string);
   napster_send(conn_info.sock, NAPSTER_SEARCH, "FILENAME CONTAINS \"%s\" MAX_RESULTS 100%s",
		string, extra_optns);
   
   d_free(string);
   d_free(not_string);

   gtk_clist_clear(GTK_CLIST(search_tab.clist));
   gtk_clist_freeze(GTK_CLIST(search_tab.clist));
   gtk_entry_set_editable(GTK_ENTRY(search_tab.gtk_entry), 0);
}

void browse_user_menu(GtkWidget *clist) {
   void *temp;
   Type *n;
   int row = 0;
   char user[1024];
   
   if (!connected() || browsing())
     return;

   temp = gnapster_get_selection_data(clist, &row);
   if (!temp) 
     return;
   
   n = (Type *)temp;
   
   if (n->type == N_SEARCH )
     g_snprintf(user, sizeof(user), ((Search *)temp)->user);
   else if (n->type == TRANSFER)
     g_snprintf(user, sizeof(user), ((Transfer *)temp)->user);
   else if (n->type == N_USER)
     g_snprintf(user, sizeof(user), ((User *)temp)->user);
   else 
     return;
   
   gnapster_update_status(_("Retrieving mp3 list..."), 1);
   conn_info.state |= BROWSING_MASK;
   
   if (opennap_version(0, 34))
     napster_send(conn_info.sock, NAPSTER_OPENNAP_BROWSE, user);
   else
     napster_send(conn_info.sock, NAPSTER_BROWSE, user);
         
   gtk_clist_clear(GTK_CLIST(browse_tab.ctree));
   gtk_clist_freeze(GTK_CLIST(browse_tab.ctree));
   gtk_entry_set_editable(GTK_ENTRY(browse_tab.entry), 0);
}

void browse_user_menu_cb(GtkWidget *w, gpointer data) {
   GtkWidget *data_widget;

   if (!data) return;

   data_widget = (GtkWidget *)data;
   browse_user_menu(data_widget);
}

void browse_user_cb() {
   char *string;
   
   if (GTK_CLIST(browse_tab.ctree)->freeze_count || !connected() ||
       browsing())
     return;
   
   string = gtk_entry_get_text(GTK_ENTRY(browse_tab.entry));
   if (!strlen(string)) 
     return;
   
   gnapster_update_status(_("Retrieving mp3 list..."), 1);
   conn_info.state |= BROWSING_MASK;
   
   if (opennap_version(0, 34))
     napster_send(conn_info.sock, NAPSTER_OPENNAP_BROWSE, string);
   else
     napster_send(conn_info.sock, NAPSTER_BROWSE, string);
   
   gtk_clist_clear(GTK_CLIST(browse_tab.ctree));
   gtk_clist_freeze(GTK_CLIST(browse_tab.ctree));
   gtk_entry_set_editable(GTK_ENTRY(browse_tab.entry), 0);
}

void whois_user_menu_cb(GtkWidget *w, gpointer data) {
   GtkWidget *clist;
   int row = 0;
   Search *temp;
   Type *n;
   char *user;
   
   if (!data) 
     return;
   
   clist = (GtkWidget *)data;
   if (!clist) 
     return;
   
   temp = gnapster_get_selection_data(clist, &row);
   if (!temp) 
     return;
   
   n = (Type *)temp;
   
   if (n->type == N_SEARCH)
     user = temp->user;
   else if (n->type == N_USER) 
     user = ((User *)temp)->user;
   else if (n->type == TRANSFER)
     user = ((Transfer *)temp)->user;
   else
     return;

   hook_text_insert(CURR, SYSTEM, "whois_ack", "%s",
		    user);
   napster_send(conn_info.sock, NAPSTER_WHOIS_REQUEST, user);
}

void connect_cb() {
   if (connected() || connecting()) 
     return;
   
   if (!user_info.user) {
      gnapster_update_status(_("Please edit your preferences before attempting to connect!"), 0);
      return;
   }
   
   CLOSE(conn_info.sock);
   INPUT_REMOVE(conn_info.sock_input);
   
   conn_info.state &= ~CONNECTED_MASK;
   conn_info.state |= CONNECTING_MASK;
   
   d_free(user_info.server);
   user_info.server = d_config_get_string("/gnapster/Options/server");
   user_info.server_port = j_config_get_int("/gnapster/Options/server_port");
   
   /* Napster servers changed their IP...this compensates... */
   if (!user_info.server || !strcmp(user_info.server, "208.184.216.222")) {
      j_config_set_string("/gnapster/Options/server", SERVER_NAPSTER_COM);
      j_config_set_int("/gnapster/Options/server_port", 8875);
      j_config_sync();
      user_info.server = d_config_get_string("/gnapster/Options/server");
      user_info.server_port = j_config_get_int("/gnapster/Options/server_port");
   }
   
   if (!user_info.server_port) {
      j_config_set_int("/gnapster/Options/server_port", 8875);
      j_config_sync();
      user_info.server_port = j_config_get_int("/gnapster/Options/server_port");
   }
   
   if (user_info.server_port == 8875) {
      /* Query server... */
      gnapster_update_status(_("Finding best host..."), 1);
      napster_get_best_host();
   } else {
      if (conn_info.ip)
	d_free(conn_info.ip);
      
      conn_info.ip = d_strdup(user_info.server);
      conn_info.port = user_info.server_port;
      
      gnapster_update_status(_("Connecting to server..."), 1);
      napster_connect(conn_info.ip, conn_info.port);
   }
}

int reconnect_cb() {
   connect_cb();
   
   gtk_timeout_remove(conn_info.reconnect_timeout);
   
   return 0;
}

void disconnect_cb() {
   if (!connected() && !connecting())
     return;
   
   /* we can assume these are set because the above statement failed */
   CLOSE(conn_info.sock);
   INPUT_REMOVE(conn_info.sock_input);
   
   napster_unbind();

   INPUT_REMOVE(conn_info.upload_input);
   
   nb_destroy_all();
   
   gtk_label_set_text(GTK_LABEL(upload_tab.label), "");
   gtk_entry_set_editable(GTK_ENTRY(search_tab.gtk_entry), 1);

   /* opennap only feature */
   gtk_widget_hide(search_tab.not_label);
   gtk_widget_hide(search_tab.not_entry);

   gnapster_update_status(_("Disconnected"), 0);
   
   mp3_list_free();
   
   gtk_xtext_remove_lines(GTK_XTEXT(motd_tab.text), -1, TRUE);
   
   channel_list_clear();
   
   if (!safe_exit())
     conn_info.reconnect_timeout = gtk_timeout_add(60000 * 2, reconnect_cb,
						   NULL);

   /* this holds things like opennap version, connected || connecting,
    * searching, browsing, etc etc */
   conn_info.state = 0;
}

void search_download_cb(GtkWidget *w, gpointer data) {
   download_true_cb(search_tab.clist, data);
}

void browse_download_cb(GtkWidget *w, gpointer data) {
   download_true_cb(browse_tab.ctree, data);
}

void cancel_upload_cb() {
   Transfer *temp;
   GtkWidget *clist = (GtkWidget *)upload_tab.clist;
   
   temp = gnapster_get_selection_data(clist, NULL);
   if (!temp)
     return;
   
   upload_close(temp, TRANSFER_CANCEL);
}

char *listint_to_str(int i, int type) {
   char *cmp[] = { 
      "At least", "Equal to", "At best", NULL
   }, *conn[] = {
      "Unknown", "14.4", "28.8", "33.6", "56k", "64k ISDN", "128k ISDN", "Cable", "DSL",
      "T1", "T3+", NULL
   };
   
   switch(type) {
    case 0:
      return cmp[i];
    case 1:
      return conn[i];
    default:
      return NULL;
   }
}

int liststr_to_int(char *s) {
   char *conn[] = {
      "Unknown", "14.4", "28.8", "33.6", "56k", "64k ISDN", "128k ISDN", "Cable", "DSL",
      "T1", "T3+", NULL
   };
   char *bitrate[] = {
      "96", "112", "128", "160", "192", NULL
   };
   int i = 0;

   if (!strcmp(s, "At least"))
     return 0;
   else if (!strcmp(s, "Equal to"))
     return 1;
   else if (!strcmp(s, "At best"))
     return 2;
   
   while(conn[i]) {
      if (!strcmp(s, conn[i])) return i;
      i++;
   }
   i = 0;
   while(bitrate[i]) {
      if (!strcmp(s, bitrate[i])) return i;
      i++;
   }

   return 0;
}

void combo_changed(GtkWidget *widget, gpointer data) {
   WidgetProperty *prop;
   int val;
   
   prop = gtk_object_get_data(GTK_OBJECT(widget), "prop");
   if (!prop)
     return;
   
   if (GTK_IS_SPIN_BUTTON(widget))
     val = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
   else
     val = liststr_to_int(gtk_entry_get_text(GTK_ENTRY(widget)));
   
   user_info.conf[prop->key] = val;

   j_config_set_int(prop->conf, val);
   j_config_sync();
}

void preferences_changed_cb() {
   j_property_box_changed(prop_dlg.window);
}

#ifdef HAVE_GNOME
void preferences_button_cb(GtkWidget *widget, gint arg1, gpointer data) {
#else
void preferences_button_cb(GtkWidget *widget, gpointer data) {
#endif /* HAVE_GNOME */
   char *user, *pass, *nap_port, *conn, *download_dir, *upload_dir;
   char *font, *theme, *socks_server, *socks_port;
   char *socks5_username, *socks5_password;

   if (prop_dlg.calling++) {
      prop_dlg.calling = 0;
      return;
   }
   
   user = gtk_entry_get_text(GTK_ENTRY(prop_dlg.user_entry));
   pass = gtk_entry_get_text(GTK_ENTRY(prop_dlg.pass_entry));
   nap_port = gtk_entry_get_text(GTK_ENTRY(prop_dlg.nap_port));
   conn = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(prop_dlg.connection)->entry));
   download_dir = gtk_entry_get_text(GTK_ENTRY(j_file_entry_gtk_entry(prop_dlg.download_dir)));
   upload_dir = gtk_entry_get_text(GTK_ENTRY(j_file_entry_gtk_entry(prop_dlg.upload_dir)));
   
   socks_server = gtk_entry_get_text(GTK_ENTRY(prop_dlg.socks_server));
   socks_port = gtk_entry_get_text(GTK_ENTRY(prop_dlg.socks_port));
   
   socks5_username = gtk_entry_get_text(GTK_ENTRY(prop_dlg.socks5_username));
   socks5_password = gtk_entry_get_text(GTK_ENTRY(prop_dlg.socks5_password));
   
   font = gtk_entry_get_text(GTK_ENTRY(prop_dlg.font_entry));
   
   theme = gtk_entry_get_text(GTK_ENTRY(prop_dlg.curr_theme));
   
   if (!strlen(user) || !strlen(pass) || !strlen(conn) || 
       !strlen(download_dir) || !strlen(nap_port) || !strlen(font) || 
       !strlen(theme)) {
      gnapster_update_status(_("Form not complete!  Make sure you fill in everything!"), 0);
      return;
   }
   
   if (user_info.conf[SOCKS_FIREWALL]) {
      if (!strlen(socks_server) || !strlen(socks_port))
	user_info.conf[SOCKS_FIREWALL] = 0;
      else {
	 j_config_set_string("/gnapster/Connection/socks_server", socks_server);
	 j_config_set_string("/gnapster/Connection/socks_port", socks_port);
      }
      
      if (user_info.conf[SOCKS5]) {
	 j_config_set_string("/gnapster/Connection/socks5_username", 
			     socks5_username);
	 j_config_set_string("/gnapster/Connection/socks5_password", 
			     socks5_password);
      }
   }

   user_info.connection = napster_conn_as_int(conn);
   
   handle_rebind(my_atoi(nap_port));

   j_config_set_string("/gnapster/User/user", user);
   j_config_set_string("/gnapster/User/pass", pass);
   
   j_config_set_int("/gnapster/User/connection", user_info.connection);
   j_config_set_string("/gnapster/User/dl_dir", download_dir);
   
   if (strlen(upload_dir)) 
     j_config_set_string("/gnapster/User/ul_dir", upload_dir);
   
   j_config_set_string("/gnapster/xtext/font", font);
   
   j_config_set_string("/gnapster/Display/theme", theme);

   gnapster_config_set_int(0);

   if (!user_info.theme || strcmp(user_info.theme, theme)) {
      destroy_theme_hook(hooks);
      load_theme();
   }
   
   d_free(user_info.user);
   d_free(user_info.pass);
   
   d_free(user_info.download_dir);
   d_free(user_info.upload_dir);
   
   user_info.user = d_config_get_string("/gnapster/User/user");
   user_info.pass = d_config_get_string("/gnapster/User/pass");
   
   user_info.download_dir = d_config_get_string("/gnapster/User/dl_dir");   
   user_info.upload_dir = d_config_get_string("/gnapster/User/ul_dir");
   
#ifndef HAVE_GNOME
   if (!strcmp(data, "ok"))
     j_dialog_close(prop_dlg.window);
#endif /* !HAVE_GNOME */
}

void preferences_destroy_cb(GtkWidget *widget, gint arg1, gpointer data) {
#ifndef HAVE_GNOME
   j_dialog_close(prop_dlg.window);
#endif /* !HAVE_GNOME */
   
   prop_dlg.window = NULL;
}

void clicked_cb(GtkWidget *widget, int *data) {
   GtkToggleButton *button;
   WidgetProperty *prop;
   GList *ptr;
   
   button = GTK_TOGGLE_BUTTON(widget);
   
   *data = button->active ? 1 : 0;
   
//   *data = !*data;
   
   prop = gtk_object_get_data(GTK_OBJECT(widget), "prop");
   if (!prop)
     return;
   
   j_config_set_int(prop->conf, *data);
   ptr = prop->sensitive;
   while(ptr) {
      gtk_widget_set_sensitive(GTK_WIDGET(ptr->data), *data);
      ptr = ptr->next;
   }
   
   j_config_sync();
}

void preferences_clicked_cb(GtkWidget *widget, int *data) {
   clicked_cb(widget, data);
   preferences_changed_cb();
}

void preferences_cb() {
   if (dialog_open(prop_dlg.window)) 
     return;
   
   create_prop_dlg(preferences_changed_cb,
		   preferences_button_cb,
		   preferences_destroy_cb,
		   preferences_clicked_cb,
		   &(user_info.new_user));
   gtk_widget_show_all(prop_dlg.window);
}

void build_dlg_cancel_cb() {
   building = 0;
   j_dialog_close(build_dlg.window);
   gdk_input_remove(build_dlg.pipe_input);
   if (build_dlg.cpid)
     kill(build_dlg.cpid, SIGUSR1);
}

void about_cb() {
   GtkWidget *about_box;
   const char *authors[] = {
      "Josh Guilfoyle (jasta) <jasta@gotlinux.org> - Main developer / project maintainer",
/*      "Aaron Lehmann <aaronl@vitelus.com> - anti-gnome bitch ;)",*/
/*      "Jeff Hughes (valaxer) <valaxer@nwinet.com> - gnapster testing bitch",*/
      NULL,
   };
   
   about_box = j_about_new(_("Gnapster"), VERSION,
			   _("Copyright (C) 1999-2000 Josh Guilfoyle"),
			   authors,
			   _("Napster (online mp3 community) client written for GTK+/GNOME"),
			   NULL);
   
   gtk_widget_show_all(about_box);
}
