#include <unistd.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <ctype.h>

#include "napster.h"
#include "gnapster.h"
#include "callbacks.h"
#include "ui.h"
#include "upload.h"
#include "network.h"
#include "commands.h"
#include "download.h"
#include "queue.h"
#include "utils.h"

#include "chan.h"
#include "text.h"

#include "debug.h"

extern ListChanDlg list_chan_dlg;
extern ConnInfo conn_info;
extern GnapsterMain gnapster_main;
extern ConsoleTab console_tab;
extern PropDlg prop_dlg;
extern UserInfo user_info;

void hide_list_cb(GtkWidget *w, gpointer data) {
   GtkWidget *win;
   
   win = data;
   if (!win)
     return;
   
   if (GTK_WIDGET_VISIBLE(win)) {
      gtk_widget_hide(win);
      gtk_label_set_text(GTK_LABEL(GTK_BIN(w)->child), "<");
   } else {
      gtk_widget_show(win);
      gtk_label_set_text(GTK_LABEL(GTK_BIN(w)->child), ">");
   }
}

GList *nick_completion_fill_nicks(GCompletion *comp, GtkCList *clist) {
   User *dptr;
   GList *nick_list = NULL, *ptr;
   
   for(ptr=clist->row_list; ptr; ptr=ptr->next) {
      dptr = GTK_CLIST_ROW(ptr)->data;
      if (!dptr)
	break;
      
      if (dptr->user && strcmp(dptr->user, user_info.user))
	nick_list = g_list_append(nick_list, d_strdup(all_lower(dptr->user)));
   }
   
   g_completion_add_items(comp, nick_list);
   
   return nick_list;
}

void nick_list_free(GList *list) {
   GList *ptr;
   
   for(ptr=list; ptr; ptr=ptr->next)
     d_free(ptr->data);
}

char *nick_comp_func(void *data) {
   GtkCListRow *row;
   User *nuser;
   
   row = data;
   nuser = row->data;
   
   return all_lower(nuser->user);
}

char *nick_completion(char *ptr, GtkCList *clist) {
   char *nick, *cnick, *text, *new_buf, *npfx;
   GCompletion *comp;
   GList *nicks;
   
   if (!ptr || !clist)
     return NULL;
   
   new_buf = d_strdup(ptr);
   
   if (!strchr(ptr, ':'))
     return new_buf;
   
   nick = d_strdup(all_lower(next_arg_full(ptr, &ptr, ':')));
   text = ptr;
   
   /* Not the first word */
   if (!nick)
     return new_buf;
   
   if (strchr(nick, ' ') || strlen(nick) <= 1) {
      d_free(nick);
      return new_buf;
   }
   
   comp = g_completion_new(nick_comp_func);
   g_completion_add_items(comp, clist->row_list);
   
   nicks = g_completion_complete(comp, nick, &npfx);
   d_free(nick);
   if (npfx)
     d_hunk_add(npfx, strlen(npfx));
   if (!nicks || !npfx) {
      if (npfx)
	d_free(npfx);
      g_completion_free(comp);
      return new_buf;
   }
   
   cnick = ((User *)((GtkCListRow *)g_list_nth_data(nicks, 0))->data)->user;
   if (!strcmp(cnick, user_info.user) && g_list_length(nicks) > 1)
     cnick = ((User *)((GtkCListRow *)g_list_nth_data(nicks, 1))->data)->user;
   
   if (!cnick) {
      d_free(npfx);
      g_completion_free(comp);
      return new_buf;
   }
   
   /* completion succeeded, create a new buffer to hold the new string */
   d_free(new_buf);
   new_buf = d_strdup_printf(DEBUG "%s:%s", cnick, text);
   
   d_free(npfx);
   g_completion_free(comp);
   
   return new_buf;
}

void autojoin_handle() {
   FILE *f;
   char *chan, *dptr;
   GList *ptr, *felem;
   FileEntry *fent;
   
   f = open_autojoin("r");
   if (!f)
     return;
   
   felem = read_file(f);
   
   fclose(f);
   
   if (felem)
     hook_text_insert(CONSOLE, SYSTEM, "general_message", "%s",
		      "auto-joining channels");

   for(ptr=felem; ptr; ptr=ptr->next) {
      fent = ptr->data;
      if (!fent)
	continue;
      
      dptr = fent->lptr;
      
      chan = next_arg(dptr, &dptr);
      if (!chan)
	continue;
      
      napster_send(conn_info.sock, NAPSTER_JOIN_CHAN, chan);
   }
   
   read_file_free(felem);
}

FILE *open_autojoin(char *flags) {
   FILE *f;
   char *conf;
   
   conf = local_path("autojoin", NULL);
   
   f = fopen(conf, flags);
   
   if (!f)
     j_error("fopen", conf, NULL);
   
   d_free(conf);
   
   return f;
}

void append_autojoin_entry(char *chan) {
   FILE *f;
   char *dptr;
   GList *ptr, *felem;
   FileEntry *fent;
   
   if (!chan)
     return;
   
   f = open_autojoin("r");
   if (f) {
      felem = read_file(f);

      fclose(f);

      for(ptr=felem; ptr; ptr=ptr->next) {
	 fent = ptr->data;
	 if (!fent)
	   continue;
	 
	 dptr = fent->lptr;

	 if (!strncmp(dptr, chan, strlen(chan))) {
	    d_free(dptr);
	    break;
	 }
      }
      
      read_file_free(felem);
   }
   
   f = open_autojoin("a");
   if (!f)
     return;

   fprintf(f, "%s\n", chan);
   
   fclose(f);
}

void remove_autojoin_entry(char *chan) {
   FILE *f;
   char *dptr;
   GList *ptr, *felem;
   FileEntry *fent;
   
   if (!chan)
     return;
   
   f = open_autojoin("r");
   if (!f)
     return;
   
   felem = read_file(f);
   
   fclose(f);
   
   f = open_autojoin("w");
   
   for(ptr=felem; ptr; ptr=ptr->next) {
      fent = ptr->data;
      if (!fent)
	continue;
      
      dptr = fent->lptr;
      
      if (strncmp(dptr, chan, strlen(chan)))
	fprintf(f, "%s", dptr);
   }
   
   read_file_free(felem);
   
   fclose(f);
}

void autojoin_add() {
   char *chan;
   
   chan = gtk_entry_get_text(GTK_ENTRY(prop_dlg.aj_entry));
   if (!strlen(chan))
     return;
   
   append_autojoin_entry(chan);
   
   gtk_entry_set_text(GTK_ENTRY(prop_dlg.aj_entry), "");
   
   populate_autojoin_clist();
}

void autojoin_remove() {
   char *chan;
   int row = -1;
   
   gnapster_get_selection_data(prop_dlg.aj_clist, &row);
   if (row == -1)
     return;
   
   gtk_clist_get_text(GTK_CLIST(prop_dlg.aj_clist), row, 0, &chan);
   
   remove_autojoin_entry(chan);
   
   populate_autojoin_clist();
}

void populate_autojoin_clist() {
   FILE *f;
   char *dptr;
   GList *ptr, *felem;
   FileEntry *fent;
   
   if (!prop_dlg.aj_clist)
     return;
   
   f = open_autojoin("r");
   if (!f)
     return;

   if (prop_dlg.aj_clist) {
      gtk_clist_freeze(GTK_CLIST(prop_dlg.aj_clist));
      gtk_clist_clear(GTK_CLIST(prop_dlg.aj_clist));
   }
   
   felem = read_file(f);
   
   fclose(f);
   
   for(ptr=felem; ptr; ptr=ptr->next) {
      fent = ptr->data;
      if (!fent)
	continue;
      
      dptr = fent->lptr;
      
      gnapster_clist_append(prop_dlg.aj_clist, NULL, NULL, dptr, NULL);
   }

   read_file_free(felem);
   
   gtk_clist_thaw(GTK_CLIST(prop_dlg.aj_clist));
}

void list_chan_dlg_clist_cb(GtkWidget *widget, GdkEventButton *event, gpointer data) {
   int row = -1, col;
   
   gtk_clist_get_selection_info(GTK_CLIST(widget), event->x, event->y,
				&row, &col);
   
   if (event->type == GDK_2BUTTON_PRESS && event->button == 1)
     list_chan_dlg_cb(NULL, "join");
}

void list_chan_dlg_cb(GtkWidget *w, void *data) {
   if (!data)
     return;
   
   if (!strcmp(data, "join")) {
      Channel *nchan;
      
      nchan = gnapster_get_selection_data(list_chan_dlg.clist, NULL);
      if (!nchan)
	return;
      
      napster_send(conn_info.sock, NAPSTER_JOIN_CHAN, nchan->chan);
   }
   
   j_dialog_close(list_chan_dlg.window);
   list_chan_dlg.window = NULL;
}

void list_chan_cb(GtkWidget *w, gpointer data) {
   if (!connected() || connecting())
     return;
   
   if (dialog_open(list_chan_dlg.window))
     return;
   
   create_list_chan_dlg();
   
   gtk_clist_freeze(GTK_CLIST(list_chan_dlg.clist));
   
   napster_send(conn_info.sock, NAPSTER_CHAN_LIST, NULL);
}

void join_chan_cb(GtkWidget *w, gpointer data) {
   gtk_notebook_set_page(GTK_NOTEBOOK(gnapster_main.notebook), 4);
   gtk_entry_set_text(GTK_ENTRY(console_tab.entry), "/join");
   gtk_widget_grab_focus(console_tab.entry);
}

void part_chan_cb(GtkWidget *w, gpointer data) {
   if (!remove_console_tab(NULL, 0))
     j_error_dialog("You must click on a channel tab before attempting to remove it");
}

void channel_list_clear() {
   GList *ptr;
   ChannelInfo *chinfo;
   
   for(ptr=console_tab.channel_list; ptr; ptr=ptr->next) {
      chinfo = ptr->data;
      if (!chinfo)
	continue;
      
      if (chinfo->clist)
	gtk_clist_clear(GTK_CLIST(chinfo->clist));
      
      /* if we remove the 1st one everytime, we will get
       * rid of everything except Messages */
      gtk_notebook_remove_page(GTK_NOTEBOOK(console_tab.notebook),
			       1);
      
      d_free(chinfo->channel_name);
      d_free(chinfo);
   }
   
   g_list_free(console_tab.channel_list);
   console_tab.channel_list = NULL;
}

int chan_lookup(char *chan) {
   GList *ptr;
   int i = 0;
   
   for(ptr=console_tab.channel_list; ptr; ptr=ptr->next) {
      if (!g_strcasecmp(((ChannelInfo *)ptr->data)->channel_name, chan) &&
	  !((ChannelInfo *)ptr->data)->user)
	return (i + 2);
      
      i++;
   }
   
   return -1;
}

GtkCList *chan_lookup_clist(char *chan) {
   GList *ptr;
   
   for(ptr=console_tab.channel_list; ptr; ptr=ptr->next) {
      if (!g_strcasecmp(((ChannelInfo *)ptr->data)->channel_name, chan) &&
	  !((ChannelInfo *)ptr->data)->user)
	return GTK_CLIST(((ChannelInfo *)ptr->data)->clist);
   }
   
   return NULL;
}

char *chan_lookup_chan(int index) {
   ChannelInfo *chan;
   
   chan = g_list_nth_data(console_tab.channel_list, index);
   if (!chan)
     return NULL;
   
   if (!chan->user)
     return chan->channel_name;
   
   return NULL;
}

char *get_current_chan() {
   int page_num;
   
   page_num = gnapster_get_pagenum();
   if (page_num <= 1)
     return NULL;
   
   return chan_lookup_chan(page_num - 2);
}
