#include <unistd.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <ctype.h>

#include "napster.h"
#include "gnapster.h"
#include "callbacks.h"
#include "ui.h"
#include "upload.h"
#include "network.h"
#include "commands.h"
#include "queue.h"
#include "download.h"
#include "utils.h"

#include "debug.h"

extern DownloadTab download_tab;
extern UserInfo user_info;
extern ConnInfo conn_info;
extern BrowseTab browse_tab;

void retry_download_cb() {
   GtkWidget *clist = download_tab.clist;
   Transfer *download;
   GList *selection;
   
   selection = GTK_CLIST(clist)->selection;
   if (!selection)
     return;

   download = gtk_clist_get_row_data(GTK_CLIST(clist), INT(selection->data));
   if (!download)
     return;
   
   if (download->timeout_tag)
     remote_queue_check(download);
}

int peruser_downloads(char *user) {
   GList *ptr;
   GtkCList *clist;
   GtkCListRow *clist_row;
   Transfer *data;
   int downloads = 0;
   
   clist = GTK_CLIST(download_tab.clist);
   ptr = clist->row_list;
   if (!ptr)
     return -1;
   for(;ptr;ptr=ptr->next) {
      clist_row = ptr->data;
      data = clist_row->data;
      if (!data)
	break;
      if (!strcmp(data->user, user))
        downloads++;
   }
   
   return downloads;
}

void handle_download(Transfer *data, unsigned int flags) {
   int downloads;
   
   if (!data)
     return;
   
   downloads = g_list_length(GTK_CLIST(download_tab.clist)->row_list);
   
   if (!(flags & DOWNLOAD_FORCE) &&
       user_info.conf[MAX_DOWNLOADS_SWITCH] &&
       downloads >= user_info.conf[MAX_DOWNLOADS]) {
      add_queue(data);
      return;   
   }
   
   if (!(flags & DOWNLOAD_FORCE) &&
       user_info.conf[MAX_PERUSER_DOWNLOADS_SWITCH] &&
       peruser_downloads(data->user) >= user_info.conf[MAX_PERUSER_DOWNLOADS]) {
      add_queue(data);
      return;
   }
   
   show_download(data);
   
   napster_send(conn_info.sock, NAPSTER_GET, "%s \"%s\"",
		data->user, data->file);
}

void download_true_cb(GtkWidget *w, void *data) {
   void *temp;
   GList *sel, *selptr = NULL;
   Transfer *download;
   
   for(sel=GTK_CLIST(w)->selection; sel; sel=sel->next) {
      temp = (w == browse_tab.ctree) ?
	gtk_ctree_node_get_row_data(GTK_CTREE(w), sel->data) :
        gtk_clist_get_row_data(GTK_CLIST(w), INT(sel->data));
      
      if (!temp)
	continue;
      
      selptr = g_list_append(selptr, temp);
   }

   for(sel=selptr; sel; sel=sel->next) {
      temp = sel->data;
      if (!temp)
	continue;
      
      if (w == download_tab.queue_clist) {
	 download = download_dup((Transfer *)temp);
	 handle_download(download, DOWNLOAD_FORCE);
	 
	 gtk_clist_remove(GTK_CLIST(w),
			  gtk_clist_find_row_from_data(GTK_CLIST(w), (Transfer *)temp));
      } else {
	 download = search_dup((Search *)temp, TRANSFER);
	 
	 /* if we're browsing, reconstruct the data */
	 if (w == browse_tab.ctree)
	   construct_path(w, (Search *)temp, download);
	 
	 handle_download(download, DOWNLOAD_NORMAL);
      }
   }
}

void cancel_download_cb() {
   Transfer *temp;   
   GtkWidget *clist = (GtkWidget *)download_tab.clist;
   GList *selection, *selptr = NULL;

   selection = GTK_CLIST(clist)->selection;
   if (!selection) 
     return;

   while(selection) {
      temp = gtk_clist_get_row_data(GTK_CLIST(clist), INT(selection->data));

      if (temp)
	selptr = g_list_append(selptr, temp);

      selection = selection->next;
   }

   selection = selptr;
   while(selection) {
      temp = selection->data;
      download_close(temp, TRANSFER_CANCEL);
      selection = selection->next;
   }
}

void cancel_remove_cb() {
   Transfer *temp;   
   GtkWidget *clist = (GtkWidget *)download_tab.clist;
   GList *selection, *selptr = NULL;

   selection = GTK_CLIST(clist)->selection;
   if (!selection) 
     return;

   while(selection) {
      temp = gtk_clist_get_row_data(GTK_CLIST(clist), INT(selection->data));

      if (temp)
	selptr = g_list_append(selptr, temp);

      selection = selection->next;
   }

   selection = selptr;
   while(selection) {
      temp = selection->data;
      download_close(temp, TRANSFER_CANCEL | TRANSFER_FILE_REMOVE);
      selection = selection->next;
   }
}

