/* gnapster.c - controls main loop and various run-time GUI changes */

#include <stdio.h>

#include <sys/stat.h>
#include <unistd.h>
#include <netinet/in.h>

#include <signal.h>

#include <sys/utsname.h>
#include <ctype.h>

#include "napster.h"
#include "gnapster.h"
#include "ui.h"
#include "callbacks.h"

#include "upload.h"
#include "network.h"
#include "chan.h"
#include "resume.h"
#include "text.h"
#include "xtext.h"
#include "utils.h"
#include "themes.h"
#include "servers.h"

#include "timestamp.h"

#include "debug.h"

#include "../pixmaps/speedgreen.xpm"
#include "../pixmaps/speedyellow.xpm"
#include "../pixmaps/speedred.xpm"
#include "../pixmaps/speedgray.xpm"

GnapsterMain gnapster_main;
SearchTab search_tab;
BrowseTab browse_tab;
DownloadTab download_tab;
UploadTab upload_tab;
ConsoleTab console_tab;
MOTDTab motd_tab;

PropDlg prop_dlg;
BuildDlg build_dlg;
BrowseDlg browse_dlg;
AddDlg add_dlg;
EditDlg edit_dlg;
ListChanDlg list_chan_dlg;
ResumeDlg resume_dlg;
RefreshDlg refresh_dlg;

ConnInfo conn_info;
UserInfo user_info;

int na_err = 0;

int building;
GList *total_list = NULL;
GList *ignore_list = NULL;
GList *signore_list = NULL;
GList *hooks = NULL;
GdkFont *xtext_font = NULL;

GdkPixmap *speedgreen = NULL, *speedyellow = NULL, *speedred = NULL, *speedgray = NULL;
GdkBitmap *speedgreenb = NULL, *speedyellowb = NULL, *speedredb = NULL, *speedgrayb = NULL;

ConfigTable conf_table[] = {
  /* Key                        Gnapster key		Default Value */
   { "name_fix", 		NAME_FIX,		1 },
   { "line_speed_switch", 	LINE_SPEED_SWITCH,	0 },
   { "line_speed_cmp", 		LINE_SPEED_CMP,		0 },
   { "line_speed", 		LINE_SPEED,		0 },
   { "bitrate_switch", 		BITRATE_SWITCH,		0 },
   { "bitrate_cmp", 		BITRATE_CMP,		0 },
   { "bitrate", 		BITRATE,		0 },
   { "max_uploads_switch", 	MAX_UPLOADS_SWITCH,	1 },
   { "max_uploads", 		MAX_UPLOADS,		5 },
   { "reject_ul", 		REJECT_UL,		0 },
   { "firewall", 		FIREWALL,		0 },
   { "max_peruser_uploads_switch", MAX_PERUSER_UPLOADS_SWITCH,	0 },
   { "max_peruser_uploads",	MAX_PERUSER_UPLOADS,	2 },
   { "autoquery",		AUTOQUERY,		1 },
   { "max_downloads_switch",	MAX_DOWNLOADS_SWITCH,	0 },
   { "max_downloads",		MAX_DOWNLOADS,		4 },
   { "max_peruser_downloads_switch", MAX_PERUSER_DOWNLOADS_SWITCH, 1 },
   { "max_peruser_downloads",	MAX_PERUSER_DOWNLOADS,	1 },
   { "hide_users",		HIDE_USERS,		0 },
   { "logging",                 LOGGING,                1 },
   { "hide_joinpart",           HIDE_JOINPART,          0 },
   { "socks_firewall",          SOCKS_FIREWALL,         0 },
   { "socks4",                  SOCKS4,                 0 },
   { "socks5",                  SOCKS5,                 1 },
   { "tint",                    TINT,                   0 },
   { "tint_red",                TINT_RED,               180 },
   { "tint_green",              TINT_GREEN,             180 },
   { "tint_blue",               TINT_BLUE,              180 },
   { "autocancel",              AUTOCANCEL,             0 },
   { "search_local",            SEARCH_LOCAL,           0 },
   { "list_refresh",            LIST_REFRESH,           1 },
   { NULL, 		       -1,		       -1 },
};

Transfer *gnapster_match_real_download(char *user, char *file, int *row) {
   Transfer *ret;
   GList *ptr;
   int i = 0;
   
   ptr = GTK_CLIST(download_tab.clist)->row_list;
   if (!ptr)
     return NULL;
   while(ptr) {
      ret = gtk_clist_get_row_data(GTK_CLIST(download_tab.clist), i);
      if (!strcmp(ret->file, file) &&
	  !strcmp(ret->user, user)) {
	 if (row)
	   *row = i;
	 return ret;
      }
      ptr = ptr->next, i++;
   }
   
   return NULL;
}


char *gnapster_sv() {
   static char ver[256];
   struct utsname uts;
   
   if (uname(&uts) < 0)
     return NAP_VERSION;
   
   g_snprintf(ver, sizeof(ver), "%s [ %s ] / %s %s %s",
	      NAP_VERSION, TIMESTAMP, uts.nodename, uts.sysname, uts.release);
   
   return ver;
}

int gnapster_key_lookup(char *s) {
   int i = 0;
   
   if (!s) return -1;
   
   while(conf_table[i].conf_str) {
      if (!strcmp(conf_table[i].conf_str, s))
	return i;
      i++;
   }
   
   return -1;
}

void gnapster_config_combo_widget(GtkWidget *w, char *key) {
   WidgetProperty *prop;
   char *full_conf;
   GtkWidget *widget;
   int conf;
   
   widget = GTK_IS_SPIN_BUTTON(w) ? w : GTK_COMBO(w)->entry;
   
   conf = gnapster_key_lookup(key);
   if (conf < 0)
     fprintf(stderr, "Warning, key '%s' not found!\n", key);
   
   if (GTK_IS_SPIN_BUTTON(w))
       gtk_spin_button_set_value(GTK_SPIN_BUTTON(w), user_info.conf[conf]);

   prop = d_new(W_PROP);
   
   full_conf = d_strdup_printf(DEBUG "/gnapster/Options/%s", key);
   prop->key = conf;
   prop->conf = full_conf;
   gtk_object_set_data_full(GTK_OBJECT(widget), "prop", prop, gnapster_destroy);
   gtk_signal_connect(GTK_OBJECT(widget), "changed",
		      GTK_SIGNAL_FUNC(combo_changed), NULL);
}

void gnapster_config_toggle_widget(GtkWidget *w, char *key, ...) {
   WidgetProperty *prop;
   GList *sensitive = NULL;
   char *full_conf;
   GtkWidget *widget;
   int conf;
   va_list args;
   
   conf = gnapster_key_lookup(key);
   if (conf < 0)
     fprintf(stderr, "Warning, key '%s' not found!\n", key);
   
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), user_info.conf[conf]);
   
   prop = d_new(W_PROP);
   
   full_conf = d_strdup_printf(DEBUG "/gnapster/Options/%s", key);
   prop->key = conf;
   prop->conf = full_conf;
   
   va_start(args, key);
   
   for(;;) {
      widget = va_arg(args, GtkWidget *);
      if (!widget) 
	break;
      
      gtk_widget_set_sensitive(GTK_WIDGET(widget), user_info.conf[conf]);
      
      sensitive = g_list_append(sensitive, widget);
   }
   
   va_end(args);
   
   prop->sensitive = sensitive;
   
   gtk_object_set_data_full(GTK_OBJECT(w), "prop", prop, gnapster_destroy);
   
   gtk_signal_connect(GTK_OBJECT(w), "clicked",
		      GTK_SIGNAL_FUNC(clicked_cb), &user_info.conf[conf]);
}

int upload_accepted(char *user, char *file, char *tf) {
   int uploads;
   
   uploads = g_list_length(GTK_CLIST(upload_tab.clist)->row_list);
   
   if (user_info.conf[MAX_UPLOADS_SWITCH] &&
       uploads >= user_info.conf[MAX_UPLOADS]) {
      hook_text_insert(CONSOLE, SYSTEM_N, "reject_upload", "%s\4%s",
		       user, tf);
      napster_send(conn_info.sock, NAPSTER_SENDLIMIT,
		   "%s \"%s\" %i", user, file,
		   user_info.conf[MAX_UPLOADS]);
      
      return 0;
   }
   
   if (user_info.conf[REJECT_UL] &&
       g_list_length(GTK_CLIST(download_tab.clist)->row_list)) {
      hook_text_insert(CONSOLE, SYSTEM, "reject_upload", "%s\4%s",
		       user, tf);
      napster_send(conn_info.sock, NAPSTER_SENDLIMIT,
		   "%s \"%s\" 1", user, file);
      
      return 0;
   }
   
   if (user_info.conf[MAX_PERUSER_UPLOADS_SWITCH] &&
       napster_user_uploads(user) >= user_info.conf[MAX_PERUSER_UPLOADS]) {
      hook_text_insert(CONSOLE, SYSTEM, "reject_upload", "%s\4%s",
		       user, tf);
      napster_send(conn_info.sock, NAPSTER_SENDLIMIT,
		   "%s \"%s\" %i", user, file,
		   user_info.conf[MAX_PERUSER_UPLOADS]);
      
      return 0;
   }
   
   return 1;
}

int verify_file_shared(char *sfile) {
   FILE *f;
   GList *felem, *ptr;
   FileEntry *fent;
   char *fptr, *mtime, *file;
   char *dptr;
   int x = 0;
   
   if (!sfile)
     return 0;
   
   f = open_shared("r");
   if (!f)
     return 0;

   felem = read_file(f);
   
   fclose(f);
   
   for(ptr=felem; ptr; ptr=ptr->next) {
      fent = ptr->data;
      if (!fent)
	continue;
      
      dptr = fent->lptr;
      
      mtime = next_arg_first(dptr, &dptr, &fptr);
      file = next_arg(dptr, &dptr);
      
      NA_ERR_HANDLE_CONTINUE();
      
      if (!strcmp(file, sfile))
	x = 1;
      
      if (x)
	break;
   }
   
   read_file_free(felem);
   
   return x;
}

FILE *open_file(Transfer *download, char *file) {
   FILE *f;
   char *fn = NULL, *basefile, *ext;
   int i = 0;
   
   basefile = d_strdup(file);
   
   ext = strrchr(basefile, '.');
   if (!ext)
     return NULL;
   
   ext[0] = 0;
   ext++;

   fn = d_strdup_printf(DEBUG "%s/%s", user_info.download_dir, file);
   conv_file(fn);
   
   if (file_exists(fn))
     for(i=1; i <= 20; i++) {
	fn = d_strdup_printf(DEBUG "%s/%s.%i.%s", user_info.download_dir, basefile, i, ext);
	conv_file(fn);

	if (!file_exists(fn))
	  break;

	d_free(fn);
     }
   
   d_free(basefile);
   
   f = fopen(fn, "w");
   if (!f)
     return NULL;
   
   download->fpath = fn;
   if (i)
     hook_text_insert(CONSOLE, SYSTEM, "download_newfile", "%s\4%s\4%s",
		      download->user, download->trunc_file, fn);
   
   return f;
}

void *download_dup(Transfer *download) {
   Transfer *temp;
   
   if (!download)
     return NULL;

   temp = d_new(TRANSFER);
   
//   temp->trunc_file = d_strdup(download->trunc_file);
   temp->file = d_strdup(download->file);
   temp->trunc_file = trunc_file(temp->file);
   temp->user = d_strdup(download->user);
   temp->id = true_id(download->id);
   temp->num = download->num;
   temp->kbps = download->kbps;
   temp->freq = download->freq;
   temp->size = download->size;
   temp->total = download->total;
   temp->connection = download->connection;
   if (download->resume_path)
     temp->resume_path = d_strdup(download->resume_path);
   
   return temp;
}

void *search_dup(Search *search, int type) {
   if (!search) 
     return NULL;
   
   if (type == N_SEARCH) {
      Search *temp;
      
      temp = d_new(N_SEARCH);
      
      temp->file = d_strdup(search->file);
      temp->trunc_file = trunc_file(temp->file);
      temp->user = d_strdup(search->user);
      temp->id = true_id(search->id);
      temp->num = search->num;
      temp->size = search->size;
      temp->kbps = search->kbps;
      temp->freq = search->freq;
      temp->connection = search->connection;
      temp->ip = search->ip;
      return temp;
   } else if (type == TRANSFER) {
      Transfer *temp;
      
      temp = d_new(TRANSFER);

      temp->file = d_strdup(search->file);
      temp->trunc_file = trunc_file(temp->file);
      temp->user = d_strdup(search->user);
      temp->id = true_id(search->id);
      temp->num = search->num;
      temp->size = search->size;
      temp->kbps = search->kbps;
      temp->freq = search->freq;
      temp->connection = search->connection;
//      temp->ip = search->ip;
      if (search->resume_path)
	temp->resume_path = d_strdup(search->resume_path);
      return temp;
   }
   return NULL;
}

void construct_path(GtkWidget *w, Search *orig, Transfer *out) {
   GtkCTreeNode *node, *node_ptr;
   GList *paths, *ptr;
   char *buf, *dir, *path;

   /* we use the original to locate the node */
   node = gtk_ctree_find_by_row_data(GTK_CTREE(w), NULL, orig);
   
   paths = NULL;
   path = NULL;
   
   /* now, we need to work backwards to build the directory path */
   for(node_ptr=node; node_ptr; node_ptr=GTK_CTREE_ROW(node_ptr)->parent) {
      dir = GTK_CELL_PIXTEXT(GTK_CTREE_ROW(node_ptr)->row.cell[0])->text;

      paths = g_list_prepend(paths, dir);
   }
   
   if (!paths)
     return;
   
   /* username */
   out->user = d_strdup(paths->data);
   
   /* the first entry *SHOULD* be the username, 2 and on are the path */
   for(ptr=paths->next; ptr; ptr=ptr->next) {
      dir = ptr->data;
      if (!dir)
	continue;
      
      buf = d_strdup_printf(DEBUG "%s\\%s", path ? path : "", dir);
      d_free(path);
      
      path = buf;
   }
   
   g_list_free(paths);
   
   if (!path)
     return;
   
   if (out->file)
     out->file = d_str_expand(DEBUG out->file, "%s", path + 1);
   else
     out->file = d_strdup(path);
   
   d_free(path);
   
   out->trunc_file = trunc_file(out->file);
}

Transfer *gnapster_parse_download(char *s, int alt) {
   char *file, *user, *id;
   int port, connection;
   unsigned long int ip = 0;
   Transfer *temp;
   
   user = next_arg(s, &s);
   convert(next_arg(s, &s), "%lu", &ip);
   convert(next_arg(s, &s), "%i", &port);
   file = next_arg(s, &s);
   id = next_arg(s, &s);
   convert(next_arg(s, &s), "%i", &connection);
   
   NA_ERR_HANDLE(NULL);
   
   if (strchr(id, '-'))
     id = next_arg_full(id, &id, '-');
   
   NA_RESET();
   
   if (port == 0 && user_info.conf[FIREWALL] && !alt) {
      Transfer *real;
      
      hook_text_insert(CONSOLE, SYSTEM, "general_message", "unable to receive from %s, %s",
		       user, "user cannot accept connections");
      
      real = gnapster_match_real_download(user, file, NULL);
      if (real)
	download_close(real, TRANSFER_ERROR);
      
      return NULL;
   }
   
   temp = d_new(TRANSFER);
   
   temp->file = d_strdup(file);
   temp->trunc_file = trunc_file(temp->file);
   temp->user = d_strdup(user);
   temp->id = true_id(id);
   temp->fpath = NULL;
   temp->connection = connection;
   temp->port = port;
   temp->ip = BSWAP32(ip);
   
   return temp;
}

Search *gnapster_parse_search(char *s) {
   char *file, *user, *id;
   int num, size, kbps, freq, connection;
   unsigned long int ip = 0;
   Search *temp;

   file = next_arg(s, &s);
   id = next_arg(s, &s);
   size = my_atoi(next_arg(s, &s));
   kbps = my_atoi(next_arg(s, &s));
   freq = my_atoi(next_arg(s, &s));
   num = my_atoi(next_arg(s, &s));
   user = next_arg(s, &s);
   convert(next_arg(s, &s), "%lu", &ip);
   connection = my_atoi(next_arg(s, &s));
   
   NA_ERR_HANDLE(NULL);

/*   trunc_file = strrchr(file, '\\');
   if (!trunc_file) 
     return NULL;
   
   trunc_file++;*/
   
   temp = d_new(N_SEARCH);

   temp->file = d_strdup(file);
   temp->trunc_file = trunc_file(temp->file);
   temp->user = d_strdup(user);
   temp->id = true_id(id);
   temp->num = num;
   temp->size = size;
   temp->kbps = kbps;
   temp->freq = freq;
   temp->connection = connection;
   temp->ip = ip;
   
   return temp;
}

Search *gnapster_parse_resume(char *s) {
   char *file, *user, *id, *port;
   int size, connection;
   unsigned long int ip = 0;
   Search *temp;

   user = next_arg(s, &s);
   convert(next_arg(s, &s), "%lu", &ip);
   port = next_arg(s, &s);
   file = next_arg(s, &s);
   id = next_arg(s, &s);
   size = my_atoi(next_arg(s, &s));
   connection = my_atoi(next_arg(s, &s));
   
   NA_ERR_HANDLE(NULL);
   
/*   trunc_file = strrchr(file, '\\');
   if (!trunc_file) 
     return NULL;
   
   trunc_file++;*/
   
   temp = d_new(N_SEARCH);

   temp->file = d_strdup(file);
   temp->trunc_file = trunc_file(temp->file);
   temp->user = d_strdup(user);
   temp->id = true_id(id);
   temp->size = size;
   temp->connection = connection;
   temp->ip = ip;
   
   return temp;
}

void check_resume(Transfer *t) {
   FILE *f;
   GList *ptr, *felem;
   FileEntry *fent;
   char *dptr, *file, *checksum;
   size_t size;
   
   if (!t || t->resume_path)
     return;
   
   f = open_resume("r");
   if (!f)
     return;
   
   felem = read_file(f);
   
   fclose(f);
   
   for(ptr=felem; ptr; ptr=ptr->next) {
      fent = ptr->data;
      if (!fent)
	continue;
      
      dptr = fent->lptr;
      
      file = next_arg(dptr, &dptr);
      next_arg(dptr, &dptr);
      checksum = next_arg_full(dptr, &dptr, '-');
      convert(next_arg(dptr, &dptr), "%lu", &size);
      
      NA_ERR_HANDLE_CONTINUE();
      
      if (t->size == size || match_checksum(t->id, checksum))
	t->resume_path = d_strdup(file);
      else
	continue;
      
      break;
   }
   
   if (t->resume_path) {
      if (!file_exists(t->resume_path)) {
	 d_free(t->resume_path);
	 t->resume_path = NULL;
      }
   }
   
   read_file_free(felem);
}

/*void gnapster_true_fill_resume_path(void *data, int type) {
   FILE *f;
   GList *ptr, *felem;
   FileEntry *fent;
   char *dptr;
   char *file, *size, *checksum, *md5;
   int rsize;
   
   if (!data)
     return;   
   
   f = open_resume("r");
   if (!f)
     return;
   
   felem = read_file(f);
   
   fclose(f);
   
   for(ptr=felem; ptr; ptr=ptr->next) {
      fent = ptr->data;
      if (!fent)
	continue;
      
      dptr = fent->lptr;
      
      file = next_arg(dptr, &dptr);
      size = next_arg(dptr, &dptr);
      checksum = next_arg(dptr, &dptr);
      
      NA_ERR_HANDLE_CONTINUE();
      
      md5 = next_arg_full(checksum, &checksum, '-');
      rsize = my_atoi(next_arg(checksum, &checksum));
      
      NA_RESET();
      	 
      if (type == N_DOWNLOAD) {
	 Transfer *download = (Transfer *)data;

	 if (md5 && !strcmp(download->id, md5) && download->size == rsize) {
	    if (!download->resume_path)
	      download->resume_path = d_strdup(file);
	    break;
	 }
      } else {
	 Search *resume = (Search *)data;
	 
	 if (md5 && !strcmp(resume->id, md5) && resume->size == rsize) {
	    if (!resume->resume_path)
	      resume->resume_path = d_strdup(file);
	    break;
	 }
      }
   }
   
   read_file_free(felem);
}

void gnapster_fill_dl_resume_path(Transfer *download) {
   gnapster_true_fill_resume_path(download, N_DOWNLOAD);
}

void gnapster_fill_resume_path(Search *resume) {
   gnapster_true_fill_resume_path(resume, N_SEARCH);
}*/

Search *gnapster_parse_browse(char *s, char **next, int opennap) {
   char *user = NULL, *file, *id;
   int size, kbps, freq, num;
   Search *temp;

   if (!opennap)
     user = next_arg(s, &s);
   
   file = next_arg(s, &s);
   id = next_arg(s, &s);
   size = my_atoi(next_arg(s, &s));
   kbps = my_atoi(next_arg(s, &s));
   freq = my_atoi(next_arg(s, &s));
   num = my_atoi(next_arg(s, &s));
   
   NA_ERR_HANDLE(NULL);
   
   if (next)
     *next = s;

/*   if (!opennap) {
      trunc_file = strrchr(file, '\\');
      if (!trunc_file) 
	return NULL;
      
      trunc_file++;
   }*/
   
   temp = d_new(N_SEARCH);
   
   temp->file = d_strdup(file);
   temp->trunc_file = (opennap) ? 
     temp->file : trunc_file(temp->file);
   temp->user = user ? d_strdup(user) : NULL;
   temp->id = true_id(id);
   temp->num = num;
   temp->size = size;
   temp->kbps = kbps;
   temp->freq = freq;
   
   return temp;
}

int gnapster_update_appbar() {
   float val;
#ifdef HAVE_GNOME
   gnapster_main.progress = gnome_appbar_get_progress(GNOME_APPBAR(gnapster_main.appbar));
#endif /* HAVE_GNOME */
   val = gtk_progress_get_value(GTK_PROGRESS(gnapster_main.progress));
   val += 0.5;
   if (val > 100) val = 0;
   gtk_progress_set_value(GTK_PROGRESS(gnapster_main.progress), val);
   
   return 1;
}

void gnapster_update_status(char *s, int i) {
   if (s) {
#ifdef HAVE_GNOME
      gnome_appbar_pop(GNOME_APPBAR(gnapster_main.appbar));
      gnome_appbar_push(GNOME_APPBAR(gnapster_main.appbar), s);
#else
      gtk_statusbar_pop(GTK_STATUSBAR(gnapster_main.appbar), 1);
      gtk_statusbar_push(GTK_STATUSBAR(gnapster_main.appbar), 1, s);
#endif /* HAVE_GNOME */
   }
   if (i == 0) {
      gtk_timeout_remove(gnapster_main.appbar_timeout);
      gtk_progress_set_activity_mode(GTK_PROGRESS(gnapster_main.progress), 0);
      gtk_progress_set_value(GTK_PROGRESS(gnapster_main.progress), 0);
   } else if (i == 1) {
      if (gnapster_main.appbar_timeout) gtk_timeout_remove(gnapster_main.appbar_timeout);
      gnapster_main.appbar_timeout = gtk_timeout_add(50, gnapster_update_appbar,
						     NULL);
      gtk_progress_set_activity_mode(GTK_PROGRESS(gnapster_main.progress), 1);
      gtk_progress_set_value(GTK_PROGRESS(gnapster_main.progress), 0);
   }
}

void gnapster_highlight(GtkWidget *notebook, GtkWidget *vbox) {
   GtkWidget *label, *notebook_page;
   int curr_page;
   
   label = gtk_notebook_get_tab_label(GTK_NOTEBOOK(notebook),
				      vbox);
   curr_page = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
   notebook_page = gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook), 
					     curr_page);
   if (notebook_page != GTK_WIDGET(vbox))
     gtk_widget_set_style(label, gnapster_main.highlight);   
}

int user_lookup(char *user) {
   GList *ptr;
   int i = 0;
   
   ptr = console_tab.channel_list;
   while(ptr) {
      if (!g_strcasecmp(((ChannelInfo *)ptr->data)->channel_name, user) &&
	  ((ChannelInfo *)ptr->data)->user)
	return (i + 2);
      i++;
      ptr = ptr->next;
   }
   
   return -1;
}

int gnapster_get_pagenum() {
   int page_num, retval;
   
   page_num = 
     gtk_notebook_get_current_page(GTK_NOTEBOOK(console_tab.notebook));
   
   if (page_num < 0) return -1;
   retval = (page_num == 0) ? 0 : (page_num + 1);
   
   return retval;
}

int gnapster_clist_append(GtkWidget *clist, GtkWidget *vbox, void *data, ...) {
   va_list args;
   GList *ap = NULL;
   char *s;
   int ll, row = -1;

   va_start(args, data);
   while(1) {
      s = va_arg(args, char *);
      if (!s)
	break;
      ap = g_list_append(ap, s);
   }
   va_end(args);
   
   ll = g_list_length(ap);
   
     {
	char *add_id[ll];
	GList *ptr;
	int x = 0;
	
	ptr = ap;
	while(ptr) {
	   add_id[x++] = (ptr->data) ? ptr->data : "???";
	   ptr = ptr->next;
	}

	row = gtk_clist_append(GTK_CLIST(clist), add_id);
   
	gtk_clist_set_row_data_full(GTK_CLIST(clist), row, data,
				    (GtkDestroyNotify)gnapster_destroy);
     }
   

   if (vbox)
     gnapster_highlight(gnapster_main.notebook, vbox);
   
   return row;
}

void *gnapster_get_selection_data(GtkWidget *clist, int *row) {
   void *temp;
   GList *selection;
   
   selection = GTK_CLIST(clist)->selection;
   if (!selection) return NULL;

   temp = gtk_clist_get_row_data(GTK_CLIST(clist), INT(selection->data));
   if (row)
     *row = INT(selection->data);
   
   return temp;
}

void *gnapster_get_ctree_selection_data(GtkWidget *ctree, int *row) {
   void *temp;
   GList *selection;
   
   selection = GTK_CLIST(ctree)->selection;
   if (!selection)
     return NULL;
   
   temp = gtk_ctree_node_get_row_data(GTK_CTREE(ctree), selection->data);
   
   return temp;
}

void gnapster_get_styles(GtkWidget *widget) {
   gnapster_main.normal = gtk_widget_get_style(widget);
   gnapster_main.highlight = gtk_style_new();
   gnapster_main.highlight->font = gnapster_main.normal->font;
   if (same_color(gnapster_main.normal->fg[GTK_STATE_NORMAL],
		  gnapster_main.normal->fg[GTK_STATE_SELECTED])) {
      gnapster_main.highlight->fg[GTK_STATE_NORMAL].pixel = 
	gnapster_main.normal->fg[GTK_STATE_SELECTED].pixel;
      gnapster_main.highlight->fg[GTK_STATE_NORMAL].red = 65535;
      gnapster_main.highlight->fg[GTK_STATE_NORMAL].green = 0;
      gnapster_main.highlight->fg[GTK_STATE_NORMAL].blue = 0;
   } else
     gnapster_main.highlight->fg[GTK_STATE_NORMAL] = gnapster_main.normal->fg[GTK_STATE_SELECTED];
}

void gnapster_config_set_int(int defaults) {
   char temp_str[1024];
   int i = 0;

   while(conf_table[i].conf_str) {
      g_snprintf(temp_str, sizeof(temp_str), "/gnapster/Options/%s",
		 conf_table[i].conf_str);
      j_config_set_int(temp_str,
		       (defaults == 1) ? conf_table[i].conf_val : user_info.conf[i]);
      i++;
   }
   j_config_sync();
}

void gnapster_config_get_int() {
   char temp_str[1024];
   int i = 0;
   
   if (!j_config_has_section("/gnapster/Options")) 
     return;
   
   while(conf_table[i].conf_str) {
      g_snprintf(temp_str, sizeof(temp_str), "/gnapster/Options/%s=%i",
		 conf_table[i].conf_str, conf_table[i].conf_val);
      user_info.conf[i] = j_config_get_int(temp_str);
      i++;
   }
}

int verify_shared() {
   FILE *f;
   char *ptr, *str, buf[256];
   int args = 0;

   f = open_shared("r");
   /* Return 1 to just ignore it */
   if (!f) 
     return 1;
   
   /* Read the first line only */
   fgets(buf, sizeof(buf) - 1, f);
   fclose(f);
   str = buf;
   do {
      ptr = next_arg(str, &str);
      NA_RESET();
      
      args++;
   } while(ptr);
   
   if (args <= 7) 
     return 0;
   
   return 1;
}

void load_signore() {
   char *signore;
   char *ptr = NULL, *freeptr = NULL;
   int i = 0;
   
   signore = d_config_get_string("/gnapster/Options/signore");
   if (!signore)
     return;
   
   while((ptr = next_arg_full(signore, &signore, '\4'))) {
      if (i == 0)
	freeptr = ptr;
      
      signore_list = g_list_append(signore_list, d_strdup(ptr));
      
      i++;
   }
   
   NA_RESET();
   
   d_free(freeptr);
}

int idle_startup_cb(gpointer data) {
   if (!strcmp(data, "connect"))
     connect_cb();
   else if (!strcmp(data, "props"))
     preferences_cb();
   return 0;
}

void load_pixmaps() {
   GtkStyle *style;
   
   if (speedgreen != NULL)
     return;
   
   style = gtk_widget_get_style(gnapster_main.window);
   speedgreen = gdk_pixmap_create_from_xpm_d(gnapster_main.window->window,
					     &speedgreenb, &style->bg[GTK_STATE_NORMAL],
					     speedgreen_xpm);
   speedyellow = gdk_pixmap_create_from_xpm_d(gnapster_main.window->window,
					     &speedyellowb, &style->bg[GTK_STATE_NORMAL],
					     speedyellow_xpm);
   speedred = gdk_pixmap_create_from_xpm_d(gnapster_main.window->window,
					     &speedredb, &style->bg[GTK_STATE_NORMAL],
					     speedred_xpm);
   speedgray = gdk_pixmap_create_from_xpm_d(gnapster_main.window->window,
					     &speedgrayb, &style->bg[GTK_STATE_NORMAL],
					     speedgray_xpm);   
}

void set_window_geometry() {
   int w, h;
   
   w = j_config_get_int("/gnapster/Geometry/w=690");
   h = j_config_get_int("/gnapster/Geometry/h=355");
   
   gtk_widget_set_usize(GTK_WIDGET(gnapster_main.window), w, h);
}

int main(int argc, char **argv) {
//   bindtextdomain(PACKAGE, GNOMELOCALEDIR);
//   textdomain(PACKAGE);

#ifdef HAVE_GNOME
   gnome_init(PACKAGE, VERSION, argc, argv);
#else
   gtk_init(&argc, &argv);
#endif /* HAVE_GNOME */
   
   log_add_hooks(NULL, "general_error", "join_ack", "server_msg",
		 "global", "global_self", "privmsg", "privmsg_out",
		 "cancel_download", "timeout_download", "download_error",
		 "download_resume", "download_complete", "accept_upload", 
		 "reject_upload", "cancel_upload", "timeout_upload", 
		 "upload_complete", 
		 NULL);

   memset(&user_info, 0, sizeof(UserInfo));
   user_info.conf[TERMINATION] = -1;
   
   memset(&conn_info, 0, sizeof(ConnInfo));

   create_priv_dir();
   gnapster_config_get_int();
   
   unlink_log();
   
   user_info.default_font = d_config_get_string("/gnapster/xtext/font");
   if (!user_info.default_font) {
      user_info.default_font =
	d_strdup("-b&h-lucidatypewriter-medium-r-normal-*-*-120-*-*-m-*-*-*");
      j_config_set_string("/gnapster/xtext/font", user_info.default_font);
   }

   xtext_font = load_font();
   
   create_gnapster_main();
   generate_colors(gnapster_main.window);
   set_window_geometry();
   
   create_search_tab();
   create_browse_tab();
   create_download_tab();
   create_upload_tab();
   create_console_tab();
   create_motd_tab();

   gtk_widget_show_all(gnapster_main.window);
   gtk_widget_hide(search_tab.not_label);
   gtk_widget_hide(search_tab.not_entry);
   gnapster_get_styles(gnapster_main.window);

//   generate_colors(gnapster_main.window);
   load_pixmaps();

   building = 0;
   
   browse_dlg.window = NULL;
   list_chan_dlg.window = NULL;
   resume_dlg.window = NULL;
   
   load_signore();
   
   /* Handle the default theme */
   load_theme();
   
   user_info.user = d_config_get_string("/gnapster/User/user");
   if (!user_info.user) {
      gnapster_update_status(_("Please fill out the preferences form before trying to connect."), -1);
      gtk_idle_add(idle_startup_cb, "props");
   } else {
      gnapster_update_status(_("Auto-connecting..."), -1);
      user_info.pass = d_config_get_string("/gnapster/User/pass");
      user_info.download_dir = d_config_get_string("/gnapster/User/dl_dir");
      user_info.upload_dir = d_config_get_string("/gnapster/User/ul_dir");
      user_info.connection = j_config_get_int("/gnapster/User/connection");
      user_info.server = d_config_get_string("/gnapster/Options/server");
      if (!j_config_has_section("/gnapster/Options")) {
	 gnapster_text_insert(CONSOLE, SYSTEM,
			      "Options category not set, assuming defaults.  Please check the preferences dialog for the new preference changes\n");
	 gnapster_config_set_int(1);
      }
      gnapster_config_get_int();
      gtk_idle_add(idle_startup_cb, "connect");
   }

   conn_info.nap_port = (user_info.conf[FIREWALL]) ?
     0 : j_config_get_int("/gnapster/Options/port=6699");
   
   conn_info.bind_sock_input = -1;
   
   signal(SIGPIPE, SIG_IGN);

   if (!verify_shared())
     j_error_dialog("Your ~/.gnapster/shared file is outdated, please use the preferences menu to rebuild your list");

   gtk_main();
   
   d_summary();
   
   return 0;
}
