#include <stdio.h>
#include <stdlib.h>

#include <string.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include <sys/stat.h>
#include <fcntl.h>

#include <unistd.h>
#include <ctype.h>

#include "napster.h"
#include "gnapster.h"
#include "network.h"
#include "ui.h"
#include "upload.h"
#include "callbacks.h"
#include "queue.h"
#include "servers.h"
#include "resume.h"
#include "chan.h"
#include "text.h"
#include "utils.h"
#include "connect.h"

#include "debug.h"

#ifndef MSG_WAITALL
#define MSG_WAITALL 0x100
#endif /* MSG_WAITALL */

/* 1 second */
#define CLIST_UPDATE_SPEED 1000

/* for nb_read */
#define MAIN 0
#define NAPIGATOR 1

#define NB_INC 64
GList *nb_rd = NULL;

#define RW_BUFFER 2048

Packet *curr_packet = NULL;

#define CONNECT 0
#define ACCEPT 1

extern DownloadTab download_tab;
extern UploadTab upload_tab;
extern ConnInfo conn_info;
extern UserInfo user_info;

extern int na_err;
extern GList *total_list;

int n_send(int sock, char *fmt, ...) {
   char *data;
   va_list args;
   
   if (sock < 0)
     return -1;
   
   if (fmt) {
      va_start(args, fmt);
      data = d_strdup_vprintf(fmt, args);
      va_end(args);
   }
   
   if (data)
     send(sock, data, strlen(data), 0);
   
   d_free(data);
   
   return 1;
}

NBRead *nb_active(int type) {
   GList *ptr;
   NBRead *nb;

   for(ptr=nb_rd; ptr; ptr=ptr->next) {
      nb = ptr->data;
      if (!nb)
	continue;
      
      if (nb->nb_type == type)
	return nb;
   }
   
   /* no active read found, let's create one */
   nb = d_malloc(sizeof(NBRead));
   memset(nb, 0, sizeof(NBRead));
   
   nb->nb_type = type;
   
   nb_rd = g_list_prepend(nb_rd, nb);
   
   return nb;
}

void nb_terminate(NBRead *nb) {
   if (!nb)
     return;
   
   nb->data[nb->len] = 0;
   
//   nb->data = realloc(nb->data, nb->len + 1);
   
   nb->term = 1;
}

void nb_add(NBRead *nb, char c) {
   if (nb->term) {
      nb->len = nb->term = 0;
      
      d_free(nb->data);
      nb->data = NULL;
   }
   
   /* check to make sure we need more mem */
   if (!nb->data || (nb->len % NB_INC) == 0)
     nb->data = d_realloc(nb->data, nb->len + NB_INC);
   
   nb->data[nb->len++] = c;
}

int nb_read(NBRead *nb, int source, size_t count) {
   char buf[RW_BUFFER], *ptr;
   int n;
   
   if (nb->term)
     nb->len = 0;
   
   n = recv(source, buf, (count - nb->len), 0);
   if (n <= 0)
     return 0;
   
   for(ptr=buf; n>0; ptr++, n--)
     nb_add(nb, *ptr);
   
   return 1;
}

void nb_free(NBRead *nb) {
   if (!nb)
     return;
   
   d_free(nb->data);
   d_free(nb);
}

void nb_finish(NBRead *nb) {
   nb_rd = g_list_remove(nb_rd, nb);
   
   nb_free(nb);
}

void nb_destroy_all() {
   GList *ptr;
   NBRead *nb;
   
   if (!nb_rd)
     return;
   
   for(ptr=nb_rd; ptr; ptr=ptr->next) {
      nb = ptr->data;
      if (!nb)
	continue;
      
      nb_free(nb);
   }
   
   g_list_free(nb_rd);
   
   nb_rd = NULL;
}

void napigator_finish(int s) {
   /* repopulate */
   populate_browse_dlg_clist();
   
   /* close it */
   refresh_dlg_cb(NULL, NULL);
   
   gdk_input_remove(conn_info.napigator_input);
   close(s);
}

int napigator_process(char *data) {
   char *dptr, *dup;
   char *ip, *port, *network, *title;
   int ret = 1;
   
   if (!data)
     return 0;
   
   dup = d_strdup(data);
   dptr = dup;
   
   ip = next_arg(dptr, &dptr);
   port = next_arg(dptr, &dptr);
   network = next_arg(dptr, &dptr);
   next_arg(dptr, &dptr), next_arg(dptr, &dptr), next_arg(dptr, &dptr);
   title = next_arg(dptr, &dptr);
   
   if (na_err)
     d_free(dup);
   
   NA_ERR_HANDLE(ret);
   
   if (!strcmp(network, "Napster"))
     ret = 0;
   else if (!check_server_match(ip, port))
     append_server_entry(ip, port, title, network);
   
   d_free(dup);
   
   return ret;
}

void napigator_read(void *data, int source, GdkInputCondition cond) {
   int n;
   char buf[1024], *ptr;
   NBRead *nb;
   
   n = recv(source, buf, sizeof(buf) - 1, 0);
   if (n <= 0) {
      napigator_finish(source);
      return;
   }
   
   buf[n] = 0;
   
   nb = nb_active(NAPIGATOR);
   
   for(ptr=buf; n>0; ptr++, n--) {
      if (*ptr == '\n') {
	 nb_terminate(nb);

	 hack_chr(nb->data, '\r');

	 /* HTTP header */
	 if (!strcmp(nb->data, "Content-Type: text/html"))
	   nb->flag++;
	 else if (nb->flag) {
	    if (!napigator_process(nb->data)) {
	       napigator_finish(source);
	       
	       nb_finish(nb);
	       
	       return;
	    }
	 }
      } else
	nb_add(nb, *ptr);
   }
}

int sock_err(int source) {
   int err, len, x;
   
   len = sizeof(err);
   
   x = getsockopt(source, SOL_SOCKET, SO_ERROR, &err, &len);
   
   if (x < 0 || err)
     return 1;
   
   return 0;
}

void napigator_conn(void *data, int source, GdkInputCondition cond) {
   gdk_input_remove(conn_info.napigator_input);
   
   if (sock_err(source)) {
      j_error_dialog("Could not create connection to www.napigator.com");
      return;
   }
   
   fcntl(source, F_SETFL, 0);
   
   n_send(source, "GET /servers.php?version=107&client=gnapster HTTP/1.0\n\n");
   
   conn_info.napigator_input = gdk_input_add(source, GDK_INPUT_READ,
					     napigator_read, NULL);
}

void napigator_connect() {
   /* gah, quit hardcoding stuff! */
   dispatch_connect(inet_addr("209.25.178.23"), htons(80),
		    &(conn_info.napigator_sock), &(conn_info.napigator_input),
		    GDK_INPUT_WRITE,
		    napigator_conn, NULL);
}


/* TRANSFERS */


void download_close(Transfer *download, TransferInfo info) {
   int incomplete = 0, row;

   if (!download)
     return;
   
   if (download->start_timeout)
     gtk_timeout_remove(download->start_timeout);
   if (download->sock_input)
     gdk_input_remove(download->sock_input);
   if (download->update_tag)
     gtk_timeout_remove(download->update_tag);
   if (download->timeout_tag)
     gtk_timeout_remove(download->timeout_tag);
   if (download->fdesc && download->status == 2)
     fclose(download->fdesc);
   
   if (info & TRANSFER_FILE_REMOVE && download->fpath && download->status == 2) {
      unlink(download->fpath);
      remove_resume_entry(download);
   }
   
   if (download->sock)
     close(download->sock);
   
   if (download->status == 2)
     napster_send(conn_info.sock, NAPSTER_DOWNLOAD_COMPLETE, NULL);
   
   if (info & (TRANSFER_CLOSED | TRANSFER_ERROR))
     hook_text_insert(CONSOLE, SYSTEM_N, "cancel_download", "%s\4%s\4%s",
		      download->user, download->trunc_file,
		      simplify_size(download->total - download->size));
   else if (info & TRANSFER_TIMEOUT)
     hook_text_insert(CONSOLE, SYSTEM_N, "timeout_download", "%s\4%s\4%s",
		      download->user, download->trunc_file,
		      simplify_size(download->total = download->size));
   
   if (download->status == 2) {
      if (info & TRANSFER_FINISHED) {	 
	 hook_text_insert(CONSOLE, SYSTEM_P, "download_complete", "%s\4%s",
			  download->user, download->trunc_file);
	 
	 remove_resume_entry(download);
	 
	 if (download->resume_path) {
	    char *path;
	    path = d_strdup_printf(DEBUG "%s/%s", user_info.download_dir,
				   download->trunc_file);
	    conv_file(path);
	    rename(download->fpath, path);
	 }
      }
      
      incomplete = (info & (TRANSFER_CLOSED | TRANSFER_TIMEOUT));

      if (incomplete && download->fpath && !download->resume_path) {
	 char *path, *file;
	 char *new_path;

	 path = d_strdup(download->fpath);
	 file = strrchr(path, '/');
	 if (file) {
	    int x = -1;
	    struct stat st;

	    file[0] = 0;
	    file++;
	    new_path = d_strdup_printf(DEBUG "%s/incomplete", user_info.download_dir);
	    if (stat(new_path, &st) < 0)
	      x = mkdir(new_path, 0755);
	    if (S_ISDIR(st.st_mode) || x >= 0) {
	       d_free(new_path);
	       new_path = d_strdup_printf(DEBUG "%s/incomplete/%s", 
					  user_info.download_dir, file);
	       rename(download->fpath, new_path);
	       modify_resume_entry(new_path, download);
	    }
	    d_free(new_path);
	 }
	 d_free(path);
      }
   }
   
   row = gtk_clist_find_row_from_data(GTK_CLIST(download_tab.clist), download);
   if (row >= 0)
     gtk_clist_remove(GTK_CLIST(download_tab.clist), row);
   else
     transfer_free(download);

   safe_exit_check();
   queue_check();
}

void upload_close(Transfer *upload, TransferInfo info) {
   int row;
   
   if (upload->sock_input)
     gdk_input_remove(upload->sock_input);

   if (upload->update_tag)
     gtk_timeout_remove(upload->update_tag);
   
   if (upload->accept_timeout)
     gtk_timeout_remove(upload->accept_timeout);
   
   if (upload->start_timeout)
     gtk_timeout_remove(upload->start_timeout);
   
   napster_send(conn_info.sock, NAPSTER_UPLOAD_COMPLETE, NULL);

   if (info & TRANSFER_CLOSED)
     hook_text_insert(CONSOLE, SYSTEM_N, "cancel_upload", "%s\4%s",
		      upload->user, upload->trunc_file);
   else if (info & TRANSFER_FINISHED)
     hook_text_insert(CONSOLE, SYSTEM_P, "upload_complete", "%s\4%s",
		      upload->user, upload->trunc_file);
   else if (info & TRANSFER_TIMEOUT)
     hook_text_insert(CONSOLE, SYSTEM_N, "timeout_upload", "%s\4%s",
			                         upload->user, upload->trunc_file);
   
   if (upload->sock >= 0)
     close(upload->sock);
   
   if (upload->fdesc)
     fclose(upload->fdesc);
   
   row = gtk_clist_find_row_from_data(GTK_CLIST(upload_tab.clist), upload);
   if (row >= 0)
     gtk_clist_remove(GTK_CLIST(upload_tab.clist), row);
   else
     transfer_free(upload);
   
   safe_exit_check();
}

void transfer_close(Transfer *t, TransferInfo info) {
   if (t->ttype == DOWNLOAD)
     download_close(t, info);
   else
     upload_close(t, info);
}

int start_timeout(void *data) {
   Transfer *t;
   
   if (!data)
     return 0;
   
   t = data;
   
   hook_text_insert(CURR, SYSTEM_N, "general_error", "%s",
		    "download start timed out");
   
   if (t->status < 2)
     download_close(t, TRANSFER_ERROR);
   
   return 0;
}

int accept_timeout(void *data) {
   Transfer *t;
   
   if (!data)
     return 0;
   
   t = data;
   
   gdk_input_remove(t->sock_input);
   
   close(t->sock);
   
   transfer_free(t);
   
   return 0;
}

void show_download(Transfer *download) {
   char *kbps;
   
   if (!download)
     return;
   
   /* can this file be resumed? if so, setup resume_path */
   check_resume(download);

   /* this is for the updating of the transfer once it's started */
   download->calls = 5;
   
   kbps = d_strdup_printf(DEBUG "%i", download->kbps);
   gnapster_clist_append(download_tab.clist, download_tab.vbox,
			 download, download->trunc_file,
			 napster_conn_as_str(download->connection),
			 download->user, kbps, "Requesting...", NULL);
   
   download->ttype = DOWNLOAD;
   
   /* if we dont start in 5 minutes, cancel it */
   download->start_timeout = gtk_timeout_add(60000 * 5, start_timeout,
					     download);
}

FILE *open_path(Transfer *t) {
   FILE *fdesc;
   
   if (!t)
     return NULL;
   
   fdesc = NULL;
   
   if (!t->resume_path && t->ttype == DOWNLOAD)
     fdesc = open_file(t, t->trunc_file);
   
   if (t->ttype == UPLOAD) {
      /* uploading */
      if (!upload_accepted(t->user, t->file, t->trunc_file))
	return NULL;
      
      if (!verify_file_shared(t->file)) {
	 n_send(t->sock, "FILE NOT SHARED");	 
	 return NULL;
      }
      
      fdesc = fopen(conv_unix(t->file), "r");
   } else if (t->resume_path) {
      /* downloading + resuming */
      
      fdesc = fopen(t->resume_path, "a");
      t->fpath = d_strdup(t->resume_path);
   }
   
   return fdesc;
}

void h_download(void *data, int source, GdkInputCondition cond) {
   Transfer *download;
   char buf[RW_BUFFER];
   int n;
   
   if (!data)
     return;
   
   download = data;
   
   n = recv(source, buf, sizeof(buf), 0);
   
   if (n <= 0) {
      if (download->size == download->total ||
	  (download->total - download->size) <= 2048)
	download_close(download, TRANSFER_FINISHED);
      else
	download_close(download, TRANSFER_CLOSED);
      
      return;
   }
   
   if (!download->fdesc ||
       fwrite(buf, n, sizeof(char), download->fdesc) <= 0) {
      hook_text_insert(CONSOLE, SYSTEM_N, "general_error", "%s",
		       "error writing to disk");
      
      download_close(download, TRANSFER_CANCEL);
      
      return;
   }
   
   download->size += n;
   download->temp_size += n;
}

void h_upload(void *data, int source, GdkInputCondition cond) {
   Transfer *upload;
   char buf[RW_BUFFER];
   int n, sent;
   
   if (!data)
     return;
   
   upload = data;
   
   if (upload->size >= upload->total) {
      upload_close(upload, TRANSFER_FINISHED);
      return;
   }
   
   lseek(fileno(upload->fdesc), upload->size, SEEK_SET);

   n = read(fileno(upload->fdesc), buf, sizeof(buf));
   if (n <= 0) {
      upload_close(upload, TRANSFER_CLOSED);
      return;
   }

   sent = send(upload->sock, buf, n, 0);
   if (sent <= 0) {
      upload_close(upload, TRANSFER_CLOSED);
      return;
   }

   upload->size += sent;
   upload->temp_size += sent;
}

int update_transfer(void *data) {
   Transfer *t;
   GtkCList *clist;
   char *str;
   
   if (!data)
     return 0;
   
   t = data;
   
   t->calls++;
   
   if (t->calls >= 3) {
      if (t->temp_size == 0)
	t->timeout_calls++;
      else
	t->timeout_calls = 0;
      
      t->kps = 
	(((float)t->temp_size / (float)1024)) / 
	(float)(CLIST_UPDATE_SPEED / 1000);
      
      t->kps /= t->calls;
      
      t->temp_size = t->calls = 0;
      
      if (t->timeout_calls == ((5 * 60) / 3)) {
	 transfer_close(t, TRANSFER_TIMEOUT);
	 return 0;
      }
   }
   
   str = d_strdup_printf(DEBUG "%.02fM [%i%%] @ %.01fk/s",
			 ((float)t->size / 1024) / 1024,
			 (int)((float)t->size / (float)t->total * 100),
			 (float)t->kps);
   
   clist = (t->ttype == DOWNLOAD) ? 
     GTK_CLIST(download_tab.clist) : GTK_CLIST(upload_tab.clist);
   
   gtk_clist_set_text(clist, gtk_clist_find_row_from_data(clist, t),
		      (t->ttype == DOWNLOAD) ? 4 : 3, str);
   
   d_free(str);
   
//   t->temp_size = 0;
   
   return 1;
}

/* Peer-To-Peer Protocol:
 * 
 *  *NOTE* the client that accept's the connection must immediately send
 *  a '1' to the other peer, dont ask me why, you just do :)
 * 
 *  * accept():
 * 
 *    * Downloading
 * 
 *      <REMOTE> SENDuser "file" total_size
 *       Their client sends this to show you that the user 'user' is going
 *       to send you the file 'file', which it's total size is 'total_size'.
 * 
 *      <LOCAL> local_size
 *       You send back the size of the file that you currently have.  Most
 *       often this will be 0, as you do not have the file yet, but if you do,
 *       and you are going to resume it, send the current size of the file
 *       that you have.
 * 
 *    * Uploading
 * 
 *      <REMOTE> GETuser "file" remote_size
 *       The remote client sends this to ask that you send the file to them
 *       on this TCP connection.  remote_size is the size that their file is,
 *       usually 0, but if it is > 0, you must start sending the file at
 *       that offset
 * 
 *      <LOCAL> total_size
 *       You then respond with the total size of the file on your machine
 * 
 *  * connect():
 * 
 *    * Downloading
 *      
 *      <LOCAL> GETmynick "file" local_size
 *       user should be your nickname here, so their client knows who
 *       is downloading from them.  local_size is the size as you have it
 *       completed on your drive, 0 if it's a new file
 * 
 *      <REMOTE> total_size
 *       The remote client will tell you the total size
 * 
 *    * Uploading
 * 
 *      <LOCAL> SENDmynick "file" total_size
 *       Instruct their client that you will be sending the file with
 *       the given size to them
 * 
 *      <REMOTE> remote_size
 *       They will tell you at what offset to resume at
 */

int connect_complete(Transfer *t) {
   int n;
   char *ptr, *nptr, buf[RW_BUFFER], new_buf[RW_BUFFER];
   unsigned long int size;
   
   n = recv(t->sock, buf, sizeof(buf) - 1, MSG_PEEK);
   if (n <= 0)
     return 0;
   
   buf[n] = 0;
   
   if (!isdigit(buf[0])) /* we got an error */
     return 0;
   
   for(ptr=buf, nptr=new_buf; *ptr && isdigit(*ptr); ptr++)
     *nptr++ = *ptr;
   
   *nptr++ = 0;
   
   /* skip the appropriate number of bytes now */
   recv(t->sock, buf, strlen(new_buf), 0);
   
   convert(new_buf, "%lu", &size);
   
   /* ok, this data is either the remote offset, or the total size */
   if (t->ttype == DOWNLOAD)
     t->total = size;
   else
     t->size = size;
   
   return 1;
}

int setup_transfer(Transfer *t) {
   struct stat st;
   
   t->fdesc = open_path(t);
   if (!t->fdesc) {
      if (t->ttype == DOWNLOAD)
	j_warning_dialog("Unable to open your configured download path for write!");
      
      return 0;
   }
   
   conv_win(t->file);
   
   if (t->ttype == UPLOAD) {
      fstat(fileno(t->fdesc), &st);
      
      t->total = st.st_size;
      
      /* let the other end know the size */
      if (t->conn_type == ACCEPT)
	n_send(t->sock, "%lu", t->total);
      else
	napster_send_send(t->sock, user_info.user, t->file,
			  t->total);
   }
   
   if (t->ttype == DOWNLOAD && t->resume_path) {
      fstat(fileno(t->fdesc), &st);

      t->size = st.st_size;
      
      hook_text_insert(CURR, SYSTEM_P, "download_resume", "%s\4%s\4%s",
		       t->user, t->trunc_file, 
		       simplify_size(t->size));
   }
   
   if (t->ttype == DOWNLOAD) {
      if (t->conn_type == ACCEPT)
	n_send(t->sock, "%li", t->size);
      else
	napster_send_get(t->sock, user_info.user, t->file,
			 t->size);
   }

   /* we need to get the rest of the data now */
   if (t->conn_type == CONNECT)
     if (!connect_complete(t))
       return 0;

   return 1;
}

void transfer_callbacks(Transfer *t) {
   GdkInputCondition cond;
   void *func;
   
   cond = (t->ttype == DOWNLOAD) ?
     GDK_INPUT_READ : GDK_INPUT_WRITE;
   
   func = (t->ttype == DOWNLOAD) ?
     h_download : h_upload;
   
   t->sock_input = gdk_input_add(t->sock, cond, func, t);
   
   t->update_tag = gtk_timeout_add(CLIST_UPDATE_SPEED,
				   update_transfer, t);
}

/* The start_transfer functions will properly setup and then start
 * a transfer */

void start_download(Transfer *download) {
   if (!download)
     return;
   
   download->status++;
   
   /* status 2 means it is ready for start_download */
   if (download->status != 2) {
      download_close(download, TRANSFER_ERROR);
      return;
   }
   
   /* if we arent resuming this file, then add it to the list of
    * downloads that are able to resume */
   gtk_timeout_remove(download->start_timeout);
   
   if (!setup_transfer(download)) {
      download_close(download, TRANSFER_ERROR);
      
      return;
   }

   napster_send(conn_info.sock, NAPSTER_DOWNLOAD_START, NULL);   

   if (!download->resume_path)
     append_resume_entry(download);
   
   transfer_callbacks(download);
}

void start_upload(Transfer *upload) {
   if (!upload)
     return;
   
   if (upload->accept_timeout)
     gtk_timeout_remove(upload->accept_timeout);
   
   if (!setup_transfer(upload)) {
      upload_close(upload, TRANSFER_ERROR);
      
      return;
   }
   
   napster_send(conn_info.sock, NAPSTER_UPLOAD_START, NULL);
   
   hook_text_insert(CONSOLE, SYSTEM_P, "accept_upload", "%s\4%s",
		    upload->user, upload->trunc_file);
   
   gnapster_clist_append(upload_tab.clist, upload_tab.vbox,
			 upload, upload->trunc_file,
			 napster_conn_as_str(upload->connection),
			 upload->user, "0.00M [%0] @ 0.0k/s", NULL);
   
   upload->calls = 5;
   
   upload->ttype = UPLOAD;
   
   transfer_callbacks(upload);
}

void start_transfer(Transfer *t) {
   if (t->ttype == DOWNLOAD)
     start_download(t);
   else
     start_upload(t);
}


/* LIST SUBMITTION */


void finish_submit() {
   if (connected())
     autojoin_handle();
   
   if (total_list)
     mp3_list_free();
   
   gdk_input_remove(conn_info.upload_input);
}

GList *find_pending_dir(GList **list) {
   GList *ptr, *child, *cptr, *tmp;

   for(ptr=(*list); ptr; ) {
      child = ptr->data;
      if (!child)
	break;
      
      for(cptr=child->next; cptr; cptr=cptr->next) {
	 if (cptr->data)
	   return child;
      }
      
      /* the above child had nothing valuable in it, lets
       * destroy it */
      
      tmp = ptr->next;
      
      *list = g_list_remove_link(*list, ptr);
      
      /* get rid of the dir name */
      d_free(child->data);
      
      /* destroy the g_list */
      g_list_free(ptr->data);
      
      g_list_free_1(ptr);
      
      ptr = tmp;
   }
   
   return NULL;
}

char *construct_packet(GList **list) {
   GList *child, *ptr;
   MP3Stats *mp3;
   char *fn, *p, *buf, *pbuf;
   int psize, bsize;
   
   if (!list || !(*list))
     return NULL;

   /* this will pop inactive dirs as well */
   child = find_pending_dir(list);
   if (!child)
     return NULL;
   
   /* prefix the packet with the directory */
   p = d_strdup_printf(DEBUG "\"%s\"",
		       child->data);
   
   psize = strlen(p);
   
   for(ptr=child->next; ptr; ptr=ptr->next) {
      mp3 = ptr->data;
      if (!mp3)
	continue;
      
      fn = strrchr(mp3->filename, '\\') + 1;
      
      buf = d_strdup_printf(DEBUG "\"%s\" %s %lu %li %lu %lu",
			    fn, mp3->checksum, mp3->filesize,
			    mp3->bitrate, mp3->frequency, mp3->time);
      
//      d_free(fn);
      
      bsize = strlen(buf);
      
      if (psize + bsize <= 1500) {
	 /* add to the packet */
	 pbuf = d_strdup_printf(DEBUG "%s %s", p, buf);
	 d_free(p);
	 
	 p = pbuf;
	 psize += (bsize + 1); /* +1 for the ' ' */
	 
	 /* pop it */
	 mp3_free(mp3);
	 ptr->data = NULL;
      }
      
      d_free(buf);
      
      if (psize + bsize > 1500)
	return p;
   }
   
   return p;
}

void handle_submit(void *data, int source, GdkInputCondition cond) {
   char *packet;
   
   packet = construct_packet(&total_list);
   
   if (!packet) {
      /* no packet was found, that means we've sent it all */
      finish_submit();
      return;
   }
   
   napster_send(conn_info.sock, NAPSTER_SUBMIT_DIRECTORY,
		"%s", packet);
   
   d_free(packet);
}


/* INCOMING CONNECTIONS */


void push_transfer_info(Transfer *t, char *nick, char *file, unsigned long int *size) {
   if (!t)
     return;
   
   t->user = d_strdup(nick);
   t->file = d_strdup(file);
   
   t->trunc_file = trunc_file(t->file);
   
/*   t->trunc_file = strrchr(file, '\\');
   if (!t->trunc_file) {
      close(t->sock);
      transfer_free(t);
      
      return;
   }
   
   t->trunc_file = d_strdup(t->trunc_file + 1);*/
   
   if (size) {
      if (t->ttype == UPLOAD) /* resume offset */
	t->size = *size;
      else /* total size */
	t->total = *size;
   }
}

int accept_complete(Transfer *t) {
   Transfer *real;
   char *dup, *ptr, *nick, *file;
   unsigned long int size;
   
   if (!t)
     return 0;
   
   dup = d_strdup(t->header);
   ptr = dup;
   
   nick = next_arg(ptr, &ptr);
   file = next_arg(ptr, &ptr);
   convert(next_arg(ptr, &ptr), "%lu", &size);
   
   NA_ERR_HANDLE(0);
   
   if (t->ttype == DOWNLOAD) {
      /* connect in to the download attached to the clist */
      
      real = gnapster_match_real_download(nick, file, NULL);
      if (!real) /* they're probably trying to exploit you ;) */
	return 0;
      
      real->sock = t->sock;
      real->ttype = t->ttype;
      real->conn_type = t->conn_type;
      
      gtk_timeout_remove(t->accept_timeout);
      
      /* just destroy the old transfer, we really only needed it
       * as a scratch pad */      
      transfer_free(t);
      
      t = real;
   }
   
   t->size = 0;
   
   d_free(t->user);
   d_free(t->file);
//   d_free(t->trunc_file);
   
//   gtk_timeout_remove(t->accept_timeout);
      
   push_transfer_info(t, nick, file, &size);
   
   d_free(dup);
   
   start_transfer(t);
   
   return 1;
}

void close_accept(Transfer *t) {
   close(t->sock);
   gdk_input_remove(t->sock_input);
   
   gtk_timeout_remove(t->accept_timeout);
   transfer_free(t);
}

/* this function is called twice, once to get the first 3 chars off the
 * header (either GET or SEN[D])...then, it calls itself again, and gathers
 * t->header with the rest of the data, then sends it off to accept_complete */
void handle_accept_header(void *data, int source, GdkInputCondition cond) {
   Transfer *t;
   char buf[1024];
   int n;
   
   if (!data)
     return;
   
   t = data;

   n = (t->header) ?
     recv(source, buf, sizeof(buf) - 1, 0) :
     recv(source, buf, 3, MSG_WAITALL);
   
   if (n <= (!t->header) ? 2 : 0) {
      close_accept(t);
      return;
   }
   
   buf[n] = 0;
   
   if (!t->header) {
      if (!strncmp(buf, "GET", 3))
	t->ttype = UPLOAD;
      else if (!strncmp(buf, "SEN", 3))
	t->ttype = DOWNLOAD;
      else {
	 close_accept(t);
	 return;
      }
      
      /* push the D */
      if (t->ttype == DOWNLOAD)
	recv(source, buf, 1, 0);
      
      /* this is just used to set it to != NULL */
      t->header = d_strdup(buf);
   } else {
      d_free(t->header);
      t->header = d_strdup(buf);
      
      gdk_input_remove(t->sock_input);
      
      accept_complete(t);
   }
}

void handle_connection(void *data, int source, GdkInputCondition cond) {
   Transfer *t;
   struct sockaddr_in from;
   int len;
   
   t = d_new(TRANSFER);
   
   len = sizeof(from);
   
   t->sock =
     accept(conn_info.bind_sock, (struct sockaddr *)&from, &len);
   
   if (t->sock < 0) {
      j_error("accept", NULL);
      transfer_free(t);
      
      return;
   }
   
   /* if we do not get data in 30 seconds, close the connection */
   t->accept_timeout =
     gtk_timeout_add(30000, accept_timeout, t);
   
   /* stupid napster protocol thing */
   n_send(t->sock, "1");
   
   t->conn_type = ACCEPT;

   t->sock_input = gdk_input_add(t->sock, GDK_INPUT_READ,
				 handle_accept_header, t);
}

void conn_outgoing_header(void *data, int source, GdkInputCondition cond) {
   Transfer *t;
   int n;
   char c;
   
   if (!data)
     return;
   
   t = data;
   
   gdk_input_remove(t->sock_input);
   
   n = recv(source, &c, 1, 0);
   if (n != 1 || c != '1') {
      download_close(t, TRANSFER_ERROR);
      
      return;
   }

   /* this handles errors as well */
   start_transfer(t);
}

void conn_outgoing(void *data, int source, GdkInputCondition cond) {
   Transfer *t;
   
   if (!data)
     return;
   
   t = data;
   
   gdk_input_remove(t->sock_input);
   
   if (sock_err(source)) {
      napster_send(conn_info.sock, NAPSTER_PORT_ERROR, "%s",
		   t->user);
      
      hook_text_insert(CONSOLE, SYSTEM_N, "general_error", "%s %s",
		       "error connecting to user", t->user);
      
      transfer_close(t, TRANSFER_ERROR);
      
      return;
   }
   
   fcntl(source, F_SETFL, 0);
   
   t->conn_type = CONNECT;
   
   /* this will further setup the transfer.... */
   t->sock_input = gdk_input_add(source, GDK_INPUT_READ,
				 conn_outgoing_header, t);
}


/* MAIN READ FUNCTION */


void network_read_cb(void *data, int source, GdkInputCondition cond) {
   int n;
   NBRead *nb;
   Packet *p;
   
   nb = nb_active(MAIN);
   
   if (!nb->flag)
     nb->flag++;
   
   if (nb->flag == 1) {
      /* read the 4 byte header */
      n = nb_read(nb, source, 4);
      if (n <= 0) {
	 disconnect_cb();
	 return;
      }
      
      if (nb->len < 4)
	return;

      nb_terminate(nb);
      
      p = d_malloc(sizeof(Packet));
      
      memcpy(p, nb->data, 4);
      p->len = BSWAP16(p->len);
      p->cmd = BSWAP16(p->cmd);
      
      /* one packet will always be held in memory, the last one seen */
      d_free(curr_packet);
      curr_packet = p;
      
      nb->flag++;
      if (p->len == 0)
	nb->flag++;
   } else if (nb->flag == 2) {
      n = nb_read(nb, source, curr_packet->len);
      if (n <= 0) {
	 disconnect_cb();
	 return;
      }
      
      if (nb->len >= curr_packet->len)
	nb->flag++;
   }
   
   if (nb->flag == 3) {
      char buf[nb->len + 1];
      
      nb_terminate(nb);
      
      if (!connected()) {
	 conn_info.state &= ~CONNECTING_MASK;
	 conn_info.state |= CONNECTED_MASK;
	 
	 napster_bind(conn_info.nap_port);
      }
      
      /* it is possible that napster_handle_data may want to
       * destroy all NB hooks with disconnect_cb... so we cant assume anything
       * after this point */
      snprintf(buf, sizeof(buf), "%s", nb->data);
      
      nb_finish(nb);
      
      napster_handle_data(curr_packet->cmd, buf);
   }
}


/* MAIN NETWORK CONNECTION */


void read_best_host(void *data, int source, GdkInputCondition cond) {
   int n;
   char buf[RW_BUFFER], *ptr, *ip;
   unsigned short port;
   
   n = recv(source, buf, sizeof(buf) - 1, 0);
   if (n <= 0) {
      disconnect_cb();
      return;
   }
   
   buf[n] = 0;
   ptr = buf;
   
   ip = next_arg_full(ptr, &ptr, ':');
   convert(next_arg(ptr, &ptr), "%hu", &port);
   
   if (na_err) {
      disconnect_cb();
      return;
   }
   
   NA_RESET();
   
   d_free(conn_info.ip);
   conn_info.ip = d_strdup(ip);
   conn_info.port = port;
   
   gdk_input_remove(conn_info.sock_input);
   close(source);

   /* napster's stupid way of telling you the servers are full */
   if (!strcmp(conn_info.ip, "127.0.0.1")) {
      hook_text_insert(CONSOLE, SYSTEM, "general_error", "%s",
		       "Napster query server reported all servers were busy");
      
      disconnect_cb();
      
      return;
   }
   
   gnapster_update_status(_("Best host found, connecting..."), -1);
   napster_connect(conn_info.ip, conn_info.port);
}

void conn_main(void *data, int source, GdkInputCondition cond) {
   gdk_input_remove(conn_info.sock_input);
   
   if (sock_err(source)) {
      hook_text_insert(CONSOLE, SYSTEM, "general_error", "%s"
		       "failure connecting to napster server\n");
      
      disconnect_cb();
      
      return;
   }
   
   fcntl(source, F_SETFL, 0);
   
   conn_info.sock_input = gdk_input_add(source, GDK_INPUT_READ,
					read_best_host, NULL);
}

void login(int source) {
   char *str;
   
   if (user_info.new_user) {
      napster_send(source, NAPSTER_NEW_USER, user_info.user);
      user_info.new_user = 0;
   } else {
      napster_send(source, NAPSTER_LOGIN, "%s %s %i \"%s\" %i",
		   user_info.user, user_info.pass, conn_info.nap_port,
		   NAP_VERSION, user_info.connection);
      
      str = d_strdup_printf(DEBUG "Connected (%s:%i)...awaiting login reply...",
			    conn_info.ip, conn_info.port);
      
      gnapster_update_status(str, -1);
      
      d_free(str);
   }
}

void conn_final(void *data, int source, GdkInputCondition cond) {
   gdk_input_remove(conn_info.sock_input);
   
   if (sock_err(source)) {
      hook_text_insert(CONSOLE, SYSTEM_N, "general_error", "%s",
		       "failure connecting to napster server");
      
      disconnect_cb();
      
      return;
   }
   
   fcntl(source, F_SETFL, 0);

   login(source);

   /* start reading packets */
   conn_info.sock_input = gdk_input_add(source, GDK_INPUT_READ,
					network_read_cb, NULL);
}
