#ifndef __NETWORK_H
#define __NETWORK_H

#if G_BYTE_ORDER == G_LITTLE_ENDIAN
#define BSWAP16(x) (x)
#define BSWAP32(x) (x)
#elif G_BYTE_ORDER == G_BIG_ENDIAN
#define BSWAP16(x) (((x & 0xff) << 8) | ((x >> 8) & 0xff))
#define BSWAP32(x) (((x & 0xff) << 24) | ((x & 0xff00) << 8) | ((x & 0xff0000) >> 8) | (x >> 24))
#else
#error EEP! Unknown byte order!?
#endif

/* I'll add threading in the next release so we'll gethostbyname() this */
#define SERVER_NAPSTER_COM "64.124.41.17"

#define DOWNLOAD 0
#define UPLOAD   1

typedef struct {
   int type;
   int nb_type;

   char *data;
   int len;
   
   int term;
   int flag;
} NBRead;

typedef struct _packet {
   unsigned short len;
   unsigned short cmd;
} Packet;

typedef enum {
   TRANSFER_FINISHED = 1 << 0,
   TRANSFER_CLOSED = 1 << 1,
   TRANSFER_FILE_REMOVE = 1 << 2,
   TRANSFER_CANCEL = 1 << 3,
   TRANSFER_TIMEOUT = 1 << 4,
   TRANSFER_ERROR = 1 << 5,
} TransferInfo;

int n_send(int sock, char *fmt, ...);
NBRead *nb_active(int type);
void nb_terminate(NBRead *nb);
void nb_add(NBRead *nb, char c);
int nb_read(NBRead *nb, int source, size_t count);
void nb_free(NBRead *nb);
void nb_finish(NBRead *nb);
void nb_destroy_all();
void napigator_finish(int s);
int napigator_process(char *data);
void napigator_read(void *data, int source, GdkInputCondition cond);
int sock_err(int source);
void napigator_conn(void *data, int source, GdkInputCondition cond);
void napigator_connect();
void download_close(Transfer *download, TransferInfo info);
void upload_close(Transfer *upload, TransferInfo info);
void transfer_close(Transfer *t, TransferInfo info);
int start_timeout(void *);
int accept_timeout(void *);
void show_download(Transfer *download);
FILE *open_path(Transfer *t);
void h_download(void *data, int source, GdkInputCondition cond);
void h_upload(void *data, int source, GdkInputCondition cond);
int update_transfer(void *data);
int connect_complete(Transfer *t);
int setup_transfer(Transfer *t);
void transfer_callbacks(Transfer *t);
void start_download(Transfer *download);
void start_upload(Transfer *upload);
void finish_submit();
GList *find_pending_dir(GList **list);
char *construct_packet(GList **list);
void handle_submit(void *data, int source, GdkInputCondition cond);
void push_transfer_info(Transfer *t, char *nick, char *file, unsigned long int *size);
int accept_complete(Transfer *t);
void handle_accept_header(void *data, int source, GdkInputCondition cond);
void handle_connection(void *data, int source, GdkInputCondition cond);
void conn_outgoing_header(void *data, int source, GdkInputCondition cond);
void conn_outgoing(void *data, int source, GdkInputCondition cond);
void network_read_cb(void *data, int source, GdkInputCondition cond);
void conn_main(void *data, int source, GdkInputCondition cond);
void login();
void conn_final(void *data, int source, GdkInputCondition cond);

#endif /* __NETWORK_H */
