#include <unistd.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <ctype.h>

#include "napster.h"
#include "gnapster.h"
#include "callbacks.h"
#include "ui.h"
#include "upload.h"
#include "network.h"
#include "commands.h"
#include "download.h"
#include "utils.h"

#include "queue.h"

#include "debug.h"

extern ConnInfo conn_info;
extern DownloadTab download_tab;
extern UserInfo user_info;

int remote_queue_check(void *data) {
   Transfer *download;
   
   if (!data)
     return 0;
   
   download = (Transfer *)data;
   
   download->status = 0;
   gtk_timeout_remove(download->timeout_tag);
   gtk_clist_set_text(GTK_CLIST(download_tab.clist), 
		      gtk_clist_find_row_from_data(GTK_CLIST(download_tab.clist), download),
		      4, "Retrying...");
   napster_send(conn_info.sock, NAPSTER_GET, "%s \"%s\"",
		download->user, download->file);
   
   return 0;
}

Transfer *find_valid_queue(int *row) {
   Transfer *download;
   int peruser, r = -1;
   GList *ptr;
   
   for(ptr=GTK_CLIST(download_tab.queue_clist)->row_list; ptr; ptr=ptr->next) {
      download = GTK_CLIST_ROW(ptr)->data;
      if (!download)
	break;
      
      r++;
      
      if (user_info.conf[MAX_PERUSER_DOWNLOADS_SWITCH]) {
	 peruser = peruser_downloads(download->user);
	 if (peruser >= user_info.conf[MAX_PERUSER_DOWNLOADS])
	   continue;
      }
      
      *row = r;
      return download;
   }
   
   return NULL;
}

void queue_check() {
   int downloads, queue, row = -1;
   Transfer *q, *dup;
   
   queue = GTK_CLIST(download_tab.queue_clist)->rows;
   if (!queue)
     return;
   
   downloads = GTK_CLIST(download_tab.clist)->rows;
   if (user_info.conf[MAX_DOWNLOADS_SWITCH] &&
       downloads >= user_info.conf[MAX_DOWNLOADS])
     return;
   
   q = find_valid_queue(&row);
   if (!q)
     return;
   
   dup = download_dup(q);
   handle_download(dup, DOWNLOAD_NORMAL);

   gtk_clist_remove(GTK_CLIST(download_tab.queue_clist), row);
}

void add_queue(Transfer *data) {
   char length_str[8], kbps_str[4];
   int seconds = 0;
   
   if (!data)
     return;
   
   if (data->kbps > 0)
     seconds = (data->size / ((data->kbps * 1000) / 8));
   
   g_snprintf(length_str, sizeof(length_str), "%i:%02i",
	      seconds / 60, seconds % 60);
   g_snprintf(kbps_str, sizeof(kbps_str), "%i", data->kbps);
   
   gnapster_clist_append(download_tab.queue_clist, download_tab.vbox, data,
			 data->trunc_file, napster_conn_as_str(data->connection),
			 data->user, simplify_size(data->size), length_str, 
			 kbps_str, NULL);
}

void cancel_movequeue_cb() {
   Transfer *temp, *dup;
   GtkWidget *clist = (GtkWidget *)download_tab.clist;
   GList *sel, *selptr = NULL;
   
   for(sel=GTK_CLIST(clist)->selection; sel; sel=sel->next) {
      temp = gtk_clist_get_row_data(GTK_CLIST(clist), INT(sel->data));
      if (!temp)
	continue;
      
      selptr = g_list_append(selptr, temp);
   }
   
   for(sel=selptr; sel; sel=sel->next) {
      temp = sel->data;
      if (!temp)
	continue;
      
      dup = download_dup(temp);
      download_close(temp, TRANSFER_CANCEL);
      
      dup->size = dup->total;
      
      d_free(dup->resume_path);
      dup->resume_path = NULL;
      
      add_queue(dup);
   }
}

void cancel_queue_cb() {
   Transfer *temp;
   GtkWidget *clist = (GtkWidget *)download_tab.queue_clist;
   GList *selection, *selptr = NULL;

   selection = GTK_CLIST(clist)->selection;
   if (!selection)
     return;

   while(selection) {
      temp = gtk_clist_get_row_data(GTK_CLIST(clist), INT(selection->data));
      if (temp)
	selptr = g_list_append(selptr, temp);

      selection = selection->next;
   }

   selection = selptr;
   while(selection) {
      temp = selection->data;
      gtk_clist_remove(GTK_CLIST(clist),
		       gtk_clist_find_row_from_data(GTK_CLIST(clist), temp));
      selection = selection->next;
   }
}

void force_queue_cb(GtkWidget *w, gpointer data) {
   download_true_cb(download_tab.queue_clist, data);
}
