/* ui.c - user interface creation and modification */

#include "xtext.h"

#include <netinet/in.h>

#include "napster.h"
#include "gnapster.h"
#include "ui.h"
#include "menus.h"
#include "callbacks.h"
#include "servers.h"
#include "resume.h"
#include "chan.h"
#include "text.h"
#include "utils.h"
#include "themes.h"

#include "debug.h"

#include <gdk/gdkx.h>
#include <gtk/gtkinvisible.h>

#define DEFAULT_WIDTH 	690
#define DEFAULT_HEIGHT	355

extern GnapsterMain gnapster_main;
extern SearchTab search_tab;
extern BrowseTab browse_tab;
extern DownloadTab download_tab;
extern UploadTab upload_tab;
extern ConsoleTab console_tab;
extern MOTDTab motd_tab;
extern PropDlg prop_dlg;
extern BuildDlg build_dlg;
extern BrowseDlg browse_dlg;
extern AddDlg add_dlg;
extern EditDlg edit_dlg;
extern ListChanDlg list_chan_dlg;
extern ResumeDlg resume_dlg;
extern RefreshDlg refresh_dlg;

extern ConnInfo conn_info;
extern UserInfo user_info;

extern GdkColor colors[];
extern GdkFont *xtext_font;

extern GList *dialogs;

extern int na_err;

#ifndef HAVE_GNOME
GtkItemFactory *search_popup_menu, *upload_popup_menu, *download_popup_menu,
               *browse_popup_menu, *queue_popup_menu, *chat_popup_menu;
#endif /* HAVE_GNOME */

void set_green_style(GtkWidget *w, int row, int col) {
   GtkCList *clist;
   GtkStyle *style;
   
   if (!w)
     return;
   
   clist = GTK_CLIST(w);
   
   style = gtk_clist_get_cell_style(clist, row, col);
   if (!style)
     style = gtk_style_copy(gtk_widget_get_style(w));
   
   if (!style)
     return;

   /* add configuration options for this later */
   style->fg[GTK_STATE_NORMAL].red = 0;
   style->fg[GTK_STATE_NORMAL].green = 0xd100;
   style->fg[GTK_STATE_NORMAL].blue = 0;
   
   gtk_clist_set_cell_style(clist, row, col, style);
}

void configure_event(GtkWidget *w, GdkEventConfigure *e, GtkWidget *text) {
   if (GTK_XTEXT(text)->transparent)
     gtk_widget_queue_draw(text);
}

int dialog_open(GtkWidget *window) {
   GList *ptr;
   GtkWidget *widget;
   
   for(ptr=dialogs; ptr; ptr=ptr->next) {
      widget = ptr->data;
      if (!widget)
	continue;
      
      if (window == widget)
	return 1;
   }
   
   return 0;
}

GtkWidget *j_line_new(GtkWidget *w, int i1, int i2) {
   GtkWidget *temp;
   
   temp = gtk_hbox_new(FALSE, 5);
   gtk_box_pack_start(GTK_BOX(w), temp, i1, i2, 0);
   
   return temp;
}

GtkWidget *j_combo_new(int type, ...) {
   GtkWidget *widget;
   char *s;
   GList *list = NULL;
   va_list args;
   
   widget = gtk_combo_new();
   
   va_start(args, type);
   while(1) {
      s = va_arg(args, char *);
      if (!s)
	break;
      list = g_list_append(list, s);
   }
   va_end(args);
   
   gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(widget)->entry), type);
   gtk_combo_set_popdown_strings(GTK_COMBO(widget), list);
   
   return widget;
}

#ifndef HAVE_GNOME
static char *translatelambda(char *s, void *data) {
   return dgettext(PACKAGE, s); 
}

#define CREATE_POPUP(data,name) \
   name = gtk_item_factory_new(GTK_TYPE_MENU, "<Menu>", NULL); \
   gtk_item_factory_set_translate_func(name, (GtkTranslateFunc)translatelambda, NULL, NULL); \
   gtk_item_factory_create_items(name, sizeof(data)/sizeof(data[0]), data, NULL);
#endif /* !HAVE_GNOME */

#if 0
GtkWidget *create_gnapster_tab(char *label, char *stock) {
   GtkWidget *vbox;
   GtkWidget *stock_widget;
   GtkWidget *label_widget;
   
   vbox = gtk_hbox_new(FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(vbox), 0);
   
   stock_widget = gnome_stock_pixmap_widget_at_size(gnapster_main.window, stock,
						    15, 15);
   label_widget = gtk_label_new(label);
   
   j_box_pack(vbox, stock_widget, 0, 0, 0);
   j_box_pack(vbox, label_widget, 0, 0, 0);
   
   gtk_widget_show(stock_widget);
   gtk_widget_show(label_widget);
   
   return vbox;
}
#endif

void create_gnapster_main() {
   GtkWidget *prop_notify;
   
#ifdef HAVE_GNOME
   gnapster_main.window = gnome_app_new("gnapster", "Gnapster");
#else
   GtkItemFactory *item_factory;
   GtkAccelGroup *accel_group;

   gint nmenu_items = sizeof (gnapster_menu) / sizeof (gnapster_menu[0]);

   gnapster_main.window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gnapster_main.vbox = gtk_vbox_new(FALSE, 0);
#endif /* HAVE_GNOME */
   
   gtk_widget_set_usize(GTK_WIDGET(gnapster_main.window), 
			DEFAULT_WIDTH, DEFAULT_HEIGHT);
   gtk_window_set_wmclass(GTK_WINDOW(gnapster_main.window),
			  "gnapster", "Gnapster");
   
#ifdef HAVE_GNOME
   gnome_app_create_menus(GNOME_APP(gnapster_main.window), gnapster_menu);
#else
   accel_group = gtk_accel_group_new();
   item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", 
				       accel_group);
   gtk_item_factory_create_items(item_factory, nmenu_items, gnapster_menu,
				 NULL);
   gtk_window_add_accel_group(GTK_WINDOW(gnapster_main.window), 
			      item_factory->accel_group);
   gnapster_main.menubar = gtk_item_factory_get_widget(item_factory, "<main>");
   j_box_pack(gnapster_main.vbox, gnapster_main.menubar, 0, 0, 0);
   
   CREATE_POPUP(search_popup_menu_data,search_popup_menu);
   CREATE_POPUP(browse_popup_menu_data,browse_popup_menu);
   CREATE_POPUP(upload_popup_menu_data,upload_popup_menu);
   CREATE_POPUP(queue_popup_menu_data,queue_popup_menu);
   CREATE_POPUP(chat_popup_menu_data,chat_popup_menu);
   CREATE_POPUP(download_popup_menu_data,download_popup_menu);   
#endif /* HAVE_GNOME */
   
   gtk_window_set_title(GTK_WINDOW(gnapster_main.window), "Gnapster");
   gtk_window_set_policy(GTK_WINDOW(gnapster_main.window), 1, 1, 1);
   
   gtk_signal_connect(GTK_OBJECT(gnapster_main.window), "delete-event",
		      GTK_SIGNAL_FUNC(gnapster_exit_cb), NULL);
   
   gnapster_main.notebook = gtk_notebook_new();
   gtk_signal_connect(GTK_OBJECT(gnapster_main.notebook), "switch_page",
		      GTK_SIGNAL_FUNC(switch_page_cb), NULL);
   
#ifdef HAVE_GNOME
   gnome_app_set_contents(GNOME_APP(gnapster_main.window), 
			  gnapster_main.notebook);
   
   gnapster_main.appbar = gnome_appbar_new(TRUE, TRUE, GNOME_PREFERENCES_NEVER);
   gnapster_main.progress = gnome_appbar_get_progress(GNOME_APPBAR(gnapster_main.appbar));
   gtk_progress_set_activity_mode(GTK_PROGRESS(gnapster_main.progress), TRUE);
   gtk_progress_set_value(GTK_PROGRESS(gnapster_main.progress), 0);
   gnome_app_set_statusbar(GNOME_APP(gnapster_main.window),
			   gnapster_main.appbar);
   
   gtk_container_border_width(GTK_CONTAINER(GNOME_APP(gnapster_main.window)->contents), 5);
#else
   gnapster_main.appbar = gtk_statusbar_new();
   gnapster_main.progress = GTK_PROGRESS(gtk_progress_bar_new ());
   gtk_progress_set_activity_mode(GTK_PROGRESS(gnapster_main.progress), TRUE);
   gtk_progress_set_value(GTK_PROGRESS(gnapster_main.progress), 0);
   gtk_box_pack_end(GTK_BOX(gnapster_main.appbar), GTK_WIDGET(gnapster_main.progress), 0, 0, 0);
   gtk_box_pack_start(GTK_BOX(gnapster_main.vbox), gnapster_main.notebook, 1, 1, 0);
   gtk_box_pack_end(GTK_BOX(gnapster_main.vbox), gnapster_main.appbar, 0, 0, 0);
   gtk_container_add(GTK_CONTAINER(gnapster_main.window), gnapster_main.vbox);
   gtk_widget_show_all(gnapster_main.window);
#endif /* HAVE_GNOME */
   
   /* from xchat ;P */
   prop_notify = gtk_invisible_new();
   gtk_widget_show(prop_notify);
   
   gdk_window_set_user_data(GDK_ROOT_PARENT(), prop_notify);
   
   XSelectInput(GDK_DISPLAY(), GDK_ROOT_WINDOW(), PropertyChangeMask);
   
   gtk_signal_connect(GTK_OBJECT(prop_notify), "property-notify-event",
		      GTK_SIGNAL_FUNC(property_notify), NULL);
}

void create_search_tab() {
   int w;   
   char *titles[] = { _("Filename"), _("Connection"), _("User"), _("Size"), _("Length"), _("KBps"), NULL };
   
   w = j_config_get_int("/gnapster/Geometry/w");
   
   search_tab.vbox = gtk_vbox_new(FALSE, 5);

   search_tab.hbox = j_line_new(search_tab.vbox, 0, 0);
   
   search_tab.label = gtk_label_new(_("Search: "));
   gtk_box_pack_start(GTK_BOX(search_tab.hbox), search_tab.label,
		      FALSE, FALSE, 0);
   
   search_tab.entry = j_entry_new("gnapster_search");
   search_tab.gtk_entry = j_entry_gtk_entry(search_tab.entry);
   gtk_signal_connect(GTK_OBJECT(search_tab.gtk_entry), "activate",
		      GTK_SIGNAL_FUNC(search_activate_cb), NULL);
   gtk_box_pack_start(GTK_BOX(search_tab.hbox), search_tab.entry,
		      TRUE, TRUE, 0);
   
   search_tab.not_label = gtk_label_new(_(" NOT "));
   j_box_pack(search_tab.hbox, search_tab.not_label, 0, 0, 0);
   gtk_widget_hide(search_tab.not_label);

   search_tab.not_entry = j_entry_new("gnapster_notsearch");
   search_tab.gtk_not_entry = j_entry_gtk_entry(search_tab.not_entry);
   gtk_signal_connect(GTK_OBJECT(search_tab.gtk_not_entry), "activate",
		      GTK_SIGNAL_FUNC(search_activate_cb), NULL);
   gtk_box_pack_start(GTK_BOX(search_tab.hbox), search_tab.not_entry,
		      TRUE, TRUE, 0);
   gtk_widget_hide(search_tab.not_entry);

   search_tab.search_btn = gtk_button_new_with_label(_("Search"));
   gtk_signal_connect(GTK_OBJECT(search_tab.search_btn), "clicked",
		      GTK_SIGNAL_FUNC(search_activate_cb), NULL);
   gtk_box_pack_start(GTK_BOX(search_tab.hbox), search_tab.search_btn,
		      FALSE, FALSE, 0);
   
   search_tab.cancel_btn = gtk_button_new_with_label(_("Cancel"));
   gtk_signal_connect(GTK_OBJECT(search_tab.cancel_btn), "clicked",
		      GTK_SIGNAL_FUNC(cancel_search_cb), NULL);
   j_box_pack(search_tab.hbox, search_tab.cancel_btn, 0, 0, 0);
   
   search_tab.hbox = j_line_new(search_tab.vbox, 0, 0);
   
   search_tab.line_speed_switch = gtk_check_button_new_with_label("Line Speed:");
   j_box_pack(search_tab.hbox, search_tab.line_speed_switch, 0, 0, 0);
   
   search_tab.line_speed_cmp = j_combo_new(0, "At least", "Equal to", "At best", NULL);
   gtk_widget_set_usize(search_tab.line_speed_cmp, 75, -1);
   gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(search_tab.line_speed_cmp)->entry),
		      listint_to_str(user_info.conf[LINE_SPEED_CMP], 0));
   gnapster_config_combo_widget(search_tab.line_speed_cmp,
				"line_speed_cmp");
   j_box_pack(search_tab.hbox, search_tab.line_speed_cmp, 0, 0, 0);

   search_tab.line_speed = j_combo_new(0, "Unknown", "14.4", "28.8", "33.6", 
				       "56k", "64k ISDN", "128k ISDN", "Cable",
				       "DSL", "T1", "T3+", NULL);
   gtk_widget_set_usize(search_tab.line_speed, 100, -1);
   gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(search_tab.line_speed)->entry),
		      listint_to_str(user_info.conf[LINE_SPEED], 1));
   gnapster_config_combo_widget(search_tab.line_speed,
				"line_speed");
   j_box_pack(search_tab.hbox, search_tab.line_speed, 0, 0, 0);   

   gnapster_config_toggle_widget(search_tab.line_speed_switch,
				 "line_speed_switch",
				 search_tab.line_speed_cmp,
				 search_tab.line_speed,
				 NULL);
   
//   search_tab.label = gtk_label_new("");
//   j_box_pack(search_tab.hbox, search_tab.label, 1, 1, 0);
   
   search_tab.bitrate_switch = gtk_check_button_new_with_label("Bitrate:");
   j_box_pack(search_tab.hbox, search_tab.bitrate_switch, 0, 0, 0);

   search_tab.bitrate_cmp = j_combo_new(0, "At least", "Equal to", "At best",
					NULL);
   gtk_widget_set_usize(search_tab.bitrate_cmp, 75, -1);
   gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(search_tab.bitrate_cmp)->entry),
		      (user_info.conf[BITRATE_CMP] == 0) ? "At least" : 
		      (user_info.conf[BITRATE_CMP] == 1) ? "Equal to" :
		      "At best");
   gnapster_config_combo_widget(search_tab.bitrate_cmp,
				"bitrate_cmp");
   j_box_pack(search_tab.hbox, search_tab.bitrate_cmp, 0, 0, 0);
   
   search_tab.bitrate = j_combo_new(0, "96", "112", "128", "160", "192", NULL);
   gtk_widget_set_usize(search_tab.bitrate, 50, -1);
     {
	char *array[] = { "96", "112", "128", "160", "192", NULL };
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(search_tab.bitrate)->entry),
			   array[user_info.conf[BITRATE]]);
     }
   gnapster_config_combo_widget(search_tab.bitrate,
				"bitrate");
   j_box_pack(search_tab.hbox, search_tab.bitrate, 0, 0, 0);
   
   gnapster_config_toggle_widget(search_tab.bitrate_switch,
				 "bitrate_switch",
				 search_tab.bitrate_cmp,
				 search_tab.bitrate,
				 NULL);

   search_tab.local_only = gtk_check_button_new_with_label("Search local server only");
   gnapster_config_toggle_widget(search_tab.local_only,
				 "search_local", NULL);
   j_box_pack(search_tab.hbox, search_tab.local_only, 0, 0, 0);

   search_tab.hbox = j_line_new(search_tab.vbox, 1, 1);
   
   search_tab.scroller = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(search_tab.scroller),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
   gtk_box_pack_start(GTK_BOX(search_tab.hbox), search_tab.scroller,
		      TRUE, TRUE, 0);
   
   search_tab.clist = gtk_clist_new_with_titles(6, titles);
   gtk_clist_set_compare_func(GTK_CLIST(search_tab.clist),
			      (GtkCListCompareFunc)compare_connection);
   gtk_clist_set_sort_column(GTK_CLIST(search_tab.clist), 1);
   gtk_clist_set_selection_mode(GTK_CLIST(search_tab.clist),
				GTK_SELECTION_EXTENDED);
   gtk_clist_set_column_width(GTK_CLIST(search_tab.clist), 0, 310 + (w - DEFAULT_WIDTH));
   gtk_clist_set_column_width(GTK_CLIST(search_tab.clist), 2, 75);
   gtk_clist_set_column_width(GTK_CLIST(search_tab.clist), 3, 40);
   gtk_container_add(GTK_CONTAINER(search_tab.scroller), search_tab.clist);
   gtk_signal_connect(GTK_OBJECT(search_tab.clist), "button_press_event",
		      GTK_SIGNAL_FUNC(menu_popup_cb), search_popup_menu);
   gtk_signal_connect(GTK_OBJECT(search_tab.clist), "click-column",
		      GTK_SIGNAL_FUNC(search_sort), NULL);

   gtk_widget_set_sensitive(search_tab.line_speed_cmp, user_info.conf[LINE_SPEED_SWITCH]);
   gtk_widget_set_sensitive(search_tab.line_speed, user_info.conf[LINE_SPEED_SWITCH]);
   gtk_widget_set_sensitive(search_tab.bitrate_cmp, user_info.conf[BITRATE_SWITCH]);
   gtk_widget_set_sensitive(search_tab.bitrate, user_info.conf[BITRATE_SWITCH]);

   gtk_container_set_border_width(GTK_CONTAINER(search_tab.vbox), 5);
   
   gtk_notebook_append_page(GTK_NOTEBOOK(gnapster_main.notebook), 
			    search_tab.vbox, gtk_label_new(_("Search")));
}

void create_browse_tab() {
   int w;
   char *titles[] = { _("Filename"), _("Size"), _("KBps"), NULL };

   w = j_config_get_int("/gnapster/Geometry/w");
   
   browse_tab.vbox = gtk_vbox_new(FALSE, 5);

   browse_tab.hbox = j_line_new(browse_tab.vbox, 0, 0);

   browse_tab.label = gtk_label_new("Username: ");
   j_box_pack(browse_tab.hbox, browse_tab.label, 0, 0, 0);
   
   browse_tab.entry = gtk_entry_new();
   gtk_signal_connect(GTK_OBJECT(browse_tab.entry), "activate",
		      GTK_SIGNAL_FUNC(browse_user_cb), NULL);
   j_box_pack(browse_tab.hbox, browse_tab.entry, 0, 0, 0);
   
   browse_tab.list_btn = gtk_button_new_with_label(_("Browse user"));
   gtk_signal_connect(GTK_OBJECT(browse_tab.list_btn), "clicked",
		      GTK_SIGNAL_FUNC(browse_user_cb), NULL);
   j_box_pack(browse_tab.hbox, browse_tab.list_btn, 0, 0, 0);
   
   browse_tab.clear_btn = gtk_button_new_with_label(_("Clear"));
   gtk_signal_connect(GTK_OBJECT(browse_tab.clear_btn), "clicked",
		      GTK_SIGNAL_FUNC(browse_clear), NULL);
   j_box_pack(browse_tab.hbox, browse_tab.clear_btn, 0, 0, 0);

   browse_tab.hbox = j_line_new(browse_tab.vbox, 1, 1);

   browse_tab.scroller = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(browse_tab.scroller),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
   j_box_pack(browse_tab.hbox, browse_tab.scroller, 1, 1, 0);
   
   browse_tab.ctree = gtk_ctree_new_with_titles(3, 0, titles);
   gtk_clist_set_column_width(GTK_CLIST(browse_tab.ctree), 0, 450 + (w - DEFAULT_WIDTH));
   gtk_clist_set_column_width(GTK_CLIST(browse_tab.ctree), 1, 90);
   gtk_clist_set_selection_mode(GTK_CLIST(browse_tab.ctree),
				GTK_SELECTION_EXTENDED);
   gtk_clist_set_sort_column(GTK_CLIST(browse_tab.ctree), 0);
   gtk_clist_set_auto_sort(GTK_CLIST(browse_tab.ctree), 1);
   gtk_container_add(GTK_CONTAINER(browse_tab.scroller), browse_tab.ctree);
   gtk_signal_connect(GTK_OBJECT(browse_tab.ctree), "button_press_event",
		      GTK_SIGNAL_FUNC(menu_popup_cb), browse_popup_menu);
   
   gtk_container_set_border_width(GTK_CONTAINER(browse_tab.vbox), 5);
   
   gtk_notebook_append_page(GTK_NOTEBOOK(gnapster_main.notebook),
			    browse_tab.vbox, gtk_label_new(_("Browse")));
}

GList *browse_breakdown_path(Search *browse, GList *elem) {
   char *ptr, *path, *orig = NULL;
   
   if (!browse || !browse->user || !browse->file)
     return NULL;
   
   path = d_strdup_printf(DEBUG "\\%s%s%s", browse->user, 
			  browse->file[0] == '\\' ? "" : "\\", browse->file);
   
   while((ptr = next_arg_full(path, &path, '\\'))) {
      if (!orig) {
	 orig = ptr;
	 
	 /* anything before the first \ is useless. */
	 ptr = next_arg_full(path, &path, '\\');
      }
      
      elem = g_list_append(elem, d_strdup(ptr));
   }
   
   NA_RESET();
   
   d_free(orig);
   
   return elem;
}

void browse_tab_find(GtkCTreeNode **last, GtkCTreeNode *node, GList *ptr, int *l) {
   char *buf;

   if (!node)
     node = GTK_CTREE_NODE(GTK_CLIST(browse_tab.ctree)->row_list);
   while(node) {
      buf = GTK_CELL_PIXTEXT(GTK_CTREE_ROW(node)->row.cell[0])->text;
      if (!strcmp(buf, (char *)g_list_nth_data(ptr, *l))) {
	 *last = node, (*l)++;
	 if (GTK_CTREE_ROW(node)->children)
	   browse_tab_find(last, GTK_CTREE_ROW(node)->children, ptr, l);
	 return;
      }
      node = GTK_CTREE_ROW(node)->sibling;
   }
}

void browse_tab_add(Search *browse) {
   GtkCTreeNode *node = NULL;
   GList *elem = NULL, *ptr;
   int i = 0, leaf = 0;
   char *text[3];
   
   elem = browse_breakdown_path(browse, elem);
   browse_tab_find(&node, NULL, elem, &i);
   
   ptr = g_list_nth(elem, i);
   while(ptr) {
      leaf = (ptr->next) ? 0 : 1;
      
      memset(text, 0, sizeof(text));
      
      text[0] = ptr->data;
      if (leaf) {
	 int seconds;
	 
	 seconds = 
	   (browse->kbps) ? (browse->size / ((browse->kbps * 1000) / 8)) : 0;
	 
	 text[1] = d_strdup_printf(DEBUG "%.02fM (%i:%02i)",
				   ((float)browse->size / 1024) / 1024,
				   seconds / 60, seconds % 60);
	 text[2] = d_strdup_printf(DEBUG "%i", browse->kbps);
      }
      
      node = gtk_ctree_insert_node(GTK_CTREE(browse_tab.ctree),
				   node, NULL, text, 0, NULL, NULL,
				   NULL, NULL, leaf, 0);
      if (leaf) {
	 char *pos;
	 
	 d_free(browse->user);
	 
	 if ((pos = strchr(browse->file, '\\')))
	   *pos = 0;
	 
	 /* on UNIX systems, the first char is a \, thus browse->file's
	  * first char will be 0. */
	 if (*(browse->file))
	   browse->file = d_realloc(browse->file, strlen(browse->file) + 1);
	 else /* d_free returns NULL always */
	   browse->file = d_free(browse->file);
	 
//	 d_free(browse->file);
	 
	 browse->user = browse->trunc_file = NULL;
	 
	 gtk_ctree_node_set_row_data_full(GTK_CTREE(browse_tab.ctree),
					  node, browse,
					  (GtkDestroyNotify)gnapster_destroy);
	 d_free(text[1]);
	 d_free(text[2]);
      }
      
      ptr = ptr->next;
   }

   ptr = elem;
   while(ptr) {
      d_free(ptr->data);
      ptr = ptr->next;
   }
   g_list_free(elem);
   
   gnapster_highlight(gnapster_main.notebook, browse_tab.vbox);
}

void create_download_tab() {
   int w;
   char *titles[] = { _("Filename"), _("Connection"), _("User"), _("KBps"), _("Info"), NULL };
   char *queue_titles[] = { _("Filename"), _("Connection"), _("User"), _("Size"), _("Length"), _("KBps"), NULL };
   
   w = j_config_get_int("/gnapster/Geometry/w");
   
   download_tab.vbox = gtk_vbox_new(FALSE, 5);
   
   
   download_tab.hbox = j_line_new(download_tab.vbox, 0, 0);


   download_tab.max_downloads_switch = gtk_check_button_new_with_label(_("Max simultaneous downloads:"));
   j_box_pack(download_tab.hbox, download_tab.max_downloads_switch, 0, 0, 0);

   
   download_tab.max_downloads = gtk_spin_button_new((GtkAdjustment *)gtk_adjustment_new(5.0, 0.0, 50.0, 1.0, 5.0, 0.0),
						    0, 0);
   gtk_widget_set_sensitive(download_tab.max_downloads, user_info.conf[MAX_DOWNLOADS_SWITCH]);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(download_tab.max_downloads),
			     user_info.conf[MAX_DOWNLOADS]);
   gnapster_config_combo_widget(download_tab.max_downloads,
				"max_downloads");
   j_box_pack(download_tab.hbox, download_tab.max_downloads, 0, 0, 0);
   
   gnapster_config_toggle_widget(download_tab.max_downloads_switch,
				 "max_downloads_switch",
				 download_tab.max_downloads,
				 NULL);
   
   
   download_tab.max_peruser_downloads_switch = gtk_check_button_new_with_label(_("Max per user downloads:"));
   j_box_pack(download_tab.hbox, download_tab.max_peruser_downloads_switch, 0, 0, 0);

   
   download_tab.max_peruser_downloads = gtk_spin_button_new((GtkAdjustment *)gtk_adjustment_new(1.0, 0.0, 50.0, 1.0, 5.0, 0.0),
							   0, 0);
   gtk_widget_set_sensitive(download_tab.max_peruser_downloads, user_info.conf[MAX_PERUSER_DOWNLOADS_SWITCH]);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(download_tab.max_peruser_downloads),
			     user_info.conf[MAX_PERUSER_DOWNLOADS]);
   gnapster_config_combo_widget(download_tab.max_peruser_downloads,
				"max_peruser_downloads");
   j_box_pack(download_tab.hbox, download_tab.max_peruser_downloads, 0, 0, 0);
   
   gnapster_config_toggle_widget(download_tab.max_peruser_downloads_switch,
				 "max_peruser_downloads_switch",
				 download_tab.max_peruser_downloads,
				 NULL);

   download_tab.autocancel = gtk_check_button_new_with_label("Auto-cancel remotely queued");
   gnapster_config_toggle_widget(download_tab.autocancel,
				 "autocancel",
				 NULL);
   j_box_pack(download_tab.hbox, download_tab.autocancel, 0, 0, 0);
   
   download_tab.hbox = j_line_new(download_tab.vbox, 1, 1);
   
   
   download_tab.scroller = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(download_tab.scroller),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
   gtk_box_pack_start(GTK_BOX(download_tab.hbox), download_tab.scroller,
		      TRUE, TRUE, 0);
   
   
   download_tab.clist = gtk_clist_new_with_titles(5, titles);
   gtk_clist_set_selection_mode(GTK_CLIST(download_tab.clist),
				GTK_SELECTION_EXTENDED);
   gtk_clist_set_column_width(GTK_CLIST(download_tab.clist), 0, 290 + (w - DEFAULT_WIDTH));
   gtk_clist_set_column_width(GTK_CLIST(download_tab.clist), 2, 75);
   gtk_clist_set_column_width(GTK_CLIST(download_tab.clist), 3, 35);
   gtk_container_add(GTK_CONTAINER(download_tab.scroller), download_tab.clist);
   gtk_signal_connect(GTK_OBJECT(download_tab.clist), "button_press_event",
		      GTK_SIGNAL_FUNC(menu_popup_cb), download_popup_menu);
   
   
   download_tab.hbox = j_line_new(download_tab.vbox, 1, 1);
   
   
   download_tab.scroller = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(download_tab.scroller),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
   gtk_box_pack_start(GTK_BOX(download_tab.hbox), download_tab.scroller,
		      TRUE, TRUE, 0);   
   
   
   download_tab.queue_clist = gtk_clist_new_with_titles(6, queue_titles);
   gtk_clist_set_selection_mode(GTK_CLIST(download_tab.queue_clist),
				GTK_SELECTION_EXTENDED);
   gtk_clist_set_column_width(GTK_CLIST(download_tab.queue_clist), 0, 310 + (w - DEFAULT_WIDTH));
   gtk_clist_set_column_width(GTK_CLIST(download_tab.queue_clist), 2, 75);
   gtk_clist_set_column_width(GTK_CLIST(download_tab.queue_clist), 3, 40);
   gtk_clist_set_reorderable(GTK_CLIST(download_tab.queue_clist), 1);
   gtk_container_add(GTK_CONTAINER(download_tab.scroller), download_tab.queue_clist);
   gtk_signal_connect(GTK_OBJECT(download_tab.queue_clist), "button_press_event",
		      GTK_SIGNAL_FUNC(menu_popup_cb), queue_popup_menu);

   
   gtk_container_set_border_width(GTK_CONTAINER(download_tab.vbox), 5);
   
   gtk_notebook_append_page(GTK_NOTEBOOK(gnapster_main.notebook),
			    download_tab.vbox, gtk_label_new("Download"));
}

void create_upload_tab() {
   int w;
   char *titles[] = { _("Filename"), _("Connection"), _("User"), _("Info"), NULL };
   
   w = j_config_get_int("/gnapster/Geometry/w");
   
   upload_tab.vbox = gtk_vbox_new(FALSE, 5);
   
   upload_tab.hbox = j_line_new(upload_tab.vbox, 0, 0);
   
   upload_tab.max_uploads_switch = gtk_check_button_new_with_label(_("Max uploads:"));
   j_box_pack(upload_tab.hbox, upload_tab.max_uploads_switch, 0, 0, 0);
   
   upload_tab.max_uploads = gtk_spin_button_new((GtkAdjustment *)gtk_adjustment_new(5.0, 0.0, 50.0, 1.0, 5.0, 0.0),
						0, 0);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(upload_tab.max_uploads),
			     user_info.conf[MAX_UPLOADS]);
   gnapster_config_combo_widget(upload_tab.max_uploads,
				"max_uploads");
   j_box_pack(upload_tab.hbox, upload_tab.max_uploads, 0, 0, 0);
   
   gnapster_config_toggle_widget(upload_tab.max_uploads_switch,
				 "max_uploads_switch",
				 upload_tab.max_uploads,
				 NULL);

   upload_tab.max_peruser_uploads_switch =
     gtk_check_button_new_with_label(_("Max per user uploads:"));
   j_box_pack(upload_tab.hbox, upload_tab.max_peruser_uploads_switch, 0, 0, 0);
   
   upload_tab.max_peruser_uploads =
     gtk_spin_button_new((GtkAdjustment *)gtk_adjustment_new(2.0, 0.0, 50.0, 1.0, 5.0, 0.0),
			 0, 0);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(upload_tab.max_peruser_uploads),
			     user_info.conf[MAX_PERUSER_UPLOADS]);
   gnapster_config_combo_widget(upload_tab.max_peruser_uploads,
				"max_peruser_uploads");
   j_box_pack(upload_tab.hbox, upload_tab.max_peruser_uploads, 0, 0, 0);
   
   gnapster_config_toggle_widget(upload_tab.max_peruser_uploads_switch,
				 "max_peruser_uploads_switch",
				 upload_tab.max_peruser_uploads,
				 NULL);
   
   upload_tab.label = gtk_label_new("");
   j_box_pack(upload_tab.hbox, upload_tab.label, 0, 0, 0);

   upload_tab.hbox = j_line_new(upload_tab.vbox, 1, 1);
   
   upload_tab.scroller = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(upload_tab.scroller),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
   gtk_box_pack_start(GTK_BOX(upload_tab.hbox), upload_tab.scroller,
		      TRUE, TRUE, 0);
   
   upload_tab.clist = gtk_clist_new_with_titles(4, titles);
   gtk_clist_set_column_width(GTK_CLIST(upload_tab.clist), 0, 235 + (w - DEFAULT_WIDTH));
   gtk_clist_set_column_width(GTK_CLIST(upload_tab.clist), 2, 75);
   gtk_clist_set_column_width(GTK_CLIST(upload_tab.clist), 3, 50);
   gtk_container_add(GTK_CONTAINER(upload_tab.scroller), upload_tab.clist);
   gtk_signal_connect(GTK_OBJECT(upload_tab.clist), "button_press_event",
		      GTK_SIGNAL_FUNC(menu_popup_cb), upload_popup_menu);
   
   upload_tab.hbox = j_line_new(upload_tab.vbox, 0, 0);
   
   gtk_widget_set_sensitive(upload_tab.max_uploads, user_info.conf[MAX_UPLOADS_SWITCH]);
   gtk_widget_set_sensitive(upload_tab.max_peruser_uploads, user_info.conf[MAX_PERUSER_UPLOADS_SWITCH]);
   
   gtk_container_set_border_width(GTK_CONTAINER(upload_tab.vbox), 5);
   
   gtk_notebook_append_page(GTK_NOTEBOOK(gnapster_main.notebook),
			    upload_tab.vbox, gtk_label_new(_("Upload")));
}

void create_console_tab() {
   console_tab.main_vbox = gtk_vbox_new(FALSE, 5);
   
   console_tab.notebook = gtk_notebook_new();
   gtk_signal_connect(GTK_OBJECT(console_tab.notebook), "switch_page",
		      GTK_SIGNAL_FUNC(switch_page_cb), NULL);
   j_box_pack(console_tab.main_vbox, console_tab.notebook, 1, 1, 1);

   console_tab.vbox = gtk_vbox_new(FALSE, 5);
   
   console_tab.hbox = j_line_new(console_tab.vbox, 1, 1);
   
//   console_tab.scroller = gtk_scrolled_window_new(NULL, NULL);
//   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(console_tab.scroller),
//				  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
//   gtk_box_pack_start(GTK_BOX(console_tab.hbox), console_tab.scroller,
//		      TRUE, TRUE, 0);
   
   /* NEW XTEXT METHOD */
   
   console_tab.text = gtk_xtext_new(0, 0);
   gtk_xtext_set_palette(GTK_XTEXT(console_tab.text), colors);
   gtk_xtext_set_font(GTK_XTEXT(console_tab.text), xtext_font, 0);
   gtk_xtext_set_background(GTK_XTEXT(console_tab.text),
			    NULL, user_info.conf[TINT], user_info.conf[TINT]);
   
   GTK_XTEXT(console_tab.text)->tint_red = user_info.conf[TINT_RED];
   GTK_XTEXT(console_tab.text)->tint_green = user_info.conf[TINT_GREEN];
   GTK_XTEXT(console_tab.text)->tint_blue = user_info.conf[TINT_BLUE];
   
   gtk_signal_connect(GTK_OBJECT(gnapster_main.window), "configure_event",
		      GTK_SIGNAL_FUNC(configure_event), console_tab.text);
   
   GTK_WIDGET_UNSET_FLAGS(console_tab.text, GTK_CAN_FOCUS);
   j_box_pack(console_tab.hbox, console_tab.text, 1, 1, 0);
   
   console_tab.scroller = gtk_vscrollbar_new(GTK_XTEXT(console_tab.text)->adj);
   j_box_pack(console_tab.hbox, console_tab.scroller, 0, 0, 0);
   
/*   console_tab.text = gtk_text_new(NULL, NULL);
   gtk_text_set_point(GTK_TEXT(console_tab.text), 0);
   gtk_text_set_editable(GTK_TEXT(console_tab.text), FALSE);
   gtk_container_add(GTK_CONTAINER(console_tab.scroller), console_tab.text);*/

   console_tab.entry = gtk_entry_new();
   gtk_signal_connect(GTK_OBJECT(console_tab.entry), "activate",
		      GTK_SIGNAL_FUNC(console_activate_cb), NULL);
   gtk_signal_connect(GTK_OBJECT(console_tab.entry), "key_press_event",
		      GTK_SIGNAL_FUNC(console_keypress_cb), NULL);
   gtk_box_pack_start(GTK_BOX(console_tab.main_vbox), console_tab.entry,
		      0, 0, 0);
   
   console_tab.channel_list = NULL;
   console_tab.links_map = NULL;
   
   gtk_container_set_border_width(GTK_CONTAINER(console_tab.vbox), 5);

   gtk_widget_show_all(console_tab.main_vbox);
   
   gtk_notebook_append_page(GTK_NOTEBOOK(console_tab.notebook),
			    console_tab.vbox, gtk_label_new("Messages"));
   
//   gtk_widget_realize(console_tab.text);
   
   gtk_notebook_append_page(GTK_NOTEBOOK(gnapster_main.notebook), 
			    console_tab.main_vbox, gtk_label_new("Console"));
   
   gtk_widget_realize(console_tab.text);
}

void unhook_configure(GtkWidget *w, void *data) {
   gtk_signal_disconnect_by_data(GTK_OBJECT(gnapster_main.window), 
				 data);
}

void append_console_tab(char *channel_name, int user) {
   char *titles[] = { _("Username"), _("Shares"), _("Connection"), NULL };
   ChannelTab channel;
   ChannelInfo *chan;

   if (!channel_name) 
     return;
   
   channel.vbox = gtk_vbox_new(FALSE, 5);
   
   channel.hbox = j_line_new(channel.vbox, 1, 1);
   
   channel.text = gtk_xtext_new(0, 0);
   
   gtk_xtext_set_palette(GTK_XTEXT(channel.text), colors);
   gtk_xtext_set_font(GTK_XTEXT(channel.text), xtext_font, 0);
   gtk_xtext_set_background(GTK_XTEXT(channel.text),
			    NULL, user_info.conf[TINT], user_info.conf[TINT]);
   
   GTK_XTEXT(channel.text)->tint_red = user_info.conf[TINT_RED];
   GTK_XTEXT(channel.text)->tint_green = user_info.conf[TINT_GREEN];
   GTK_XTEXT(channel.text)->tint_blue = user_info.conf[TINT_BLUE];

   gtk_signal_connect(GTK_OBJECT(channel.text), "destroy",
		      GTK_SIGNAL_FUNC(unhook_configure), channel.text);
   gtk_signal_connect(GTK_OBJECT(gnapster_main.window), "configure_event",
		      GTK_SIGNAL_FUNC(configure_event), channel.text);
   
   GTK_WIDGET_UNSET_FLAGS(channel.text, GTK_CAN_FOCUS);
   j_box_pack(channel.hbox, channel.text, 1, 1, 0);
   
   channel.scroller = gtk_vscrollbar_new(GTK_XTEXT(channel.text)->adj);
   j_box_pack(channel.hbox, channel.scroller, 0, 0, 0);
   
   if (!user) {
      channel.scroller2 = gtk_scrolled_window_new(NULL, NULL);
      gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(channel.scroller2),
				     GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
      j_box_pack(channel.hbox, channel.scroller2, 0, 0, 0);

      channel.clist = gtk_clist_new_with_titles(3, titles);
      gtk_clist_set_column_width(GTK_CLIST(channel.clist), 0, 90);
      gtk_clist_set_column_width(GTK_CLIST(channel.clist), 1, 30);
      gtk_clist_set_sort_column(GTK_CLIST(channel.clist), 0);
      gtk_clist_set_compare_func(GTK_CLIST(channel.clist), compare_chan_user);
      gtk_widget_set_usize(GTK_WIDGET(channel.clist), 200, -1);
      gtk_container_add(GTK_CONTAINER(channel.scroller2),
			channel.clist);
      gtk_signal_connect(GTK_OBJECT(channel.clist), "button_press_event",
			 GTK_SIGNAL_FUNC(menu_popup_cb), chat_popup_menu);
      gtk_signal_connect(GTK_OBJECT(channel.clist), "click-column",
			 GTK_SIGNAL_FUNC(channel_sort), NULL);

      channel.hide_list = gtk_button_new_with_label(">");
      j_box_pack(channel.hbox, channel.hide_list, 0, 0, 0);
      gtk_signal_connect(GTK_OBJECT(channel.hide_list), "clicked",
			 GTK_SIGNAL_FUNC(hide_list_cb), channel.scroller2);
   } else {
      channel.scroller2 = NULL;
      channel.clist = NULL;
   }
   
   gtk_container_set_border_width(GTK_CONTAINER(channel.vbox), 5);

   gtk_widget_show_all(channel.vbox);
   if (user_info.conf[HIDE_USERS])
     hide_list_cb(channel.hide_list, channel.scroller2);

   gtk_notebook_append_page(GTK_NOTEBOOK(console_tab.notebook),
			    channel.vbox, gtk_label_new(channel_name));
   
   gtk_widget_realize(channel.text);

   chan = d_calloc(sizeof(ChannelInfo));
   chan->channel_name = d_strdup(channel_name);
   chan->user = user;
   chan->text = channel.text;
   chan->clist = channel.clist;
   chan->vbox = channel.vbox;
   
   console_tab.channel_list = g_list_append(console_tab.channel_list,
					    chan);
}

int remove_console_tab(char *data, int user) {
   int page_num;
   GList *ptr;
   ChannelInfo *ch;
   char *chan;
   GtkCList *clist = NULL;
   
   if (!data)
     page_num = gtk_notebook_get_current_page(GTK_NOTEBOOK(console_tab.notebook));
   else
     page_num = (chan_lookup(data) - 1);
   
   if (page_num <= 0) 
     return 0;
  
   chan = chan_lookup_chan(page_num - 1);
   
   if (!chan && !user) 
     return 0;
   if (chan && user) 
     return 0;
   
   if (chan)
     clist = chan_lookup_clist(chan);
   if (clist)
     gtk_clist_clear(clist);
   
   ptr = g_list_nth(console_tab.channel_list, (page_num - 1));
   if (ptr) {
      console_tab.channel_list = g_list_remove_link(console_tab.channel_list, ptr);
      ch = ptr->data;
      if (!user)
	napster_send(conn_info.sock, NAPSTER_PART_CHAN, ch->channel_name);
      d_free(ch->channel_name);
      d_free(ch);
      gtk_notebook_remove_page(GTK_NOTEBOOK(console_tab.notebook),
			       page_num);
      return 1;
   }
   
   return 0;
}

void create_motd_tab() {
   motd_tab.vbox = gtk_vbox_new(FALSE, 5);
   
   motd_tab.hbox = j_line_new(motd_tab.vbox, 1, 1);
   
//   motd_tab.scroller = gtk_scrolled_window_new(NULL, NULL);
//   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(motd_tab.scroller),
//				  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
//   j_box_pack(motd_tab.hbox, motd_tab.scroller, 1, 1, 0);
   
   motd_tab.text = gtk_xtext_new(0, 0);
   gtk_xtext_set_palette(GTK_XTEXT(motd_tab.text), colors);
   gtk_xtext_set_font(GTK_XTEXT(motd_tab.text), xtext_font, 0);
   gtk_xtext_set_background(GTK_XTEXT(motd_tab.text),
			    NULL, 0, 0);
   GTK_WIDGET_UNSET_FLAGS(motd_tab.text, GTK_CAN_FOCUS);
   j_box_pack(motd_tab.hbox, motd_tab.text, 1, 1, 0);

   motd_tab.scroller = gtk_vscrollbar_new(GTK_XTEXT(motd_tab.text)->adj);
   j_box_pack(motd_tab.hbox, motd_tab.scroller, 0, 0, 0);
   
/*   motd_tab.text = gtk_text_new(NULL, NULL);
   gtk_text_set_point(GTK_TEXT(motd_tab.text), 0);
   gtk_text_set_editable(GTK_TEXT(motd_tab.text), 0);
   gtk_container_add(GTK_CONTAINER(motd_tab.scroller), motd_tab.text);*/
   
   gtk_container_set_border_width(GTK_CONTAINER(motd_tab.vbox), 5);

   gtk_widget_show_all(motd_tab.vbox);
   
   gtk_notebook_append_page(GTK_NOTEBOOK(gnapster_main.notebook), motd_tab.vbox,
			    gtk_label_new(_("Message Of The Day")));
   
   gtk_widget_realize(motd_tab.text);
}

void create_prop_dlg(void *changed_cb, void *button_cb, void *destroy_cb, void *clicked_cb, int *new_user_val) {
   char *themes_title[3] = { "Theme", "Author", NULL };   
   char *user, *pass, *download_dir, *upload_dir;
   int conn;
   GtkWidget *vbox;
   
   user = d_config_get_string("/gnapster/User/user");
   pass = d_config_get_string("/gnapster/User/pass");
   download_dir = d_config_get_string("/gnapster/User/dl_dir");
   upload_dir = d_config_get_string("/gnapster/User/ul_dir");
   conn = j_config_get_int("/gnapster/User/connection");
   
   prop_dlg.calling = 0;
   
   prop_dlg.window = j_property_box_new();
//   gtk_window_set_title(GTK_WINDOW(prop_dlg.window),
//			_("Gnapster Properties"));

   /* USER INFORMATION */
   
   prop_dlg.vbox = gtk_vbox_new(FALSE, 5);
   gtk_container_border_width(GTK_CONTAINER(prop_dlg.vbox), 5);
   
   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);
   
   prop_dlg.label = gtk_label_new(_("Username: "));
   gtk_box_pack_start(GTK_BOX(prop_dlg.hbox), prop_dlg.label,
		      FALSE, FALSE, 0);
   
   prop_dlg.user_entry = gtk_entry_new();
   if (user) 
     gtk_entry_set_text(GTK_ENTRY(prop_dlg.user_entry), user);
   gtk_signal_connect(GTK_OBJECT(prop_dlg.user_entry), "changed",
		      GTK_SIGNAL_FUNC(changed_cb), NULL);
   gtk_box_pack_start(GTK_BOX(prop_dlg.hbox), prop_dlg.user_entry,
		      FALSE, FALSE, 0);
   
   prop_dlg.new_user = gtk_check_button_new_with_label(_("New Account"));
   if (new_user_val) 
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prop_dlg.new_user),
				  *new_user_val);
   gtk_signal_connect(GTK_OBJECT(prop_dlg.new_user), "clicked",
		      GTK_SIGNAL_FUNC(clicked_cb), new_user_val);
   gtk_box_pack_start(GTK_BOX(prop_dlg.hbox), prop_dlg.new_user,
		      FALSE, FALSE, 0);

   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);
   
   prop_dlg.label = gtk_label_new(_("Password: "));
   gtk_box_pack_start(GTK_BOX(prop_dlg.hbox), prop_dlg.label,
		      FALSE, FALSE, 0);
   
   prop_dlg.pass_entry = gtk_entry_new();
   gtk_entry_set_visibility(GTK_ENTRY(prop_dlg.pass_entry), FALSE);
   if (pass) 
     gtk_entry_set_text(GTK_ENTRY(prop_dlg.pass_entry), pass);
   gtk_signal_connect(GTK_OBJECT(prop_dlg.pass_entry), "changed",
		      GTK_SIGNAL_FUNC(changed_cb), NULL);
   gtk_box_pack_start(GTK_BOX(prop_dlg.hbox), prop_dlg.pass_entry,
		      FALSE, FALSE, 0);

   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);
   
   prop_dlg.label = gtk_label_new(_("Data port: "));
   j_box_pack(prop_dlg.hbox, prop_dlg.label, 0, 0, 0);
   
   prop_dlg.nap_port = gtk_entry_new_with_max_length(5);
   if (conn_info.nap_port) {
      char *temp = d_strdup_printf(DEBUG "%i", conn_info.nap_port);
      gtk_entry_set_text(GTK_ENTRY(prop_dlg.nap_port), temp);
      d_free(temp);
   } else
     gtk_entry_set_text(GTK_ENTRY(prop_dlg.nap_port), "6699");
   gtk_widget_set_usize(prop_dlg.nap_port, 50, -1);
   gtk_signal_connect(GTK_OBJECT(prop_dlg.nap_port), "changed",
		      GTK_SIGNAL_FUNC(changed_cb), NULL);
   j_box_pack(prop_dlg.hbox, prop_dlg.nap_port, 0, 0, 0);
   
   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);

   prop_dlg.label = gtk_label_new(_("Connection: "));
   j_box_pack(prop_dlg.hbox, prop_dlg.label, 0, 0, 0);
   
   prop_dlg.connection = j_combo_new(0, "Unknown", "14.4", "28.8", "33.6",
				     "56k", "64k ISDN", "128k ISDN", "Cable",
				     "DSL", "T1", "T3+", NULL);
   gtk_widget_set_usize(prop_dlg.connection, 100, -1);
   gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(prop_dlg.connection)->entry),
		      napster_conn_as_str(conn));
   gtk_signal_connect(GTK_OBJECT(GTK_COMBO(prop_dlg.connection)->entry), "changed",
		      GTK_SIGNAL_FUNC(changed_cb), NULL);
   j_box_pack(prop_dlg.hbox, prop_dlg.connection, 0, 0, 0);
   
   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);
  
   prop_dlg.label = gtk_label_new(_("Download Directory: "));
   gtk_box_pack_start(GTK_BOX(prop_dlg.hbox), prop_dlg.label,
		      FALSE, FALSE, 0);
   
   prop_dlg.download_dir = j_file_entry_new(NULL, _("Browse Download Directory"));
   if (download_dir) 
     gtk_entry_set_text(GTK_ENTRY(j_file_entry_gtk_entry(prop_dlg.download_dir)), download_dir);
   gtk_signal_connect(GTK_OBJECT(j_file_entry_gtk_entry(prop_dlg.download_dir)),
		      "changed", GTK_SIGNAL_FUNC(changed_cb), NULL);
   gtk_box_pack_start(GTK_BOX(prop_dlg.hbox), prop_dlg.download_dir,
		      TRUE, TRUE, 0);

   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);
   
   prop_dlg.label = gtk_label_new(_("Upload Directory: "));
   j_box_pack(prop_dlg.hbox, prop_dlg.label, 0, 0, 0);

   prop_dlg.upload_dir = j_file_entry_new(NULL, _("Browse Upload Directory"));
   if (upload_dir) 
     gtk_entry_set_text(GTK_ENTRY(j_file_entry_gtk_entry(prop_dlg.upload_dir)), 
			upload_dir);
   gtk_signal_connect(GTK_OBJECT(j_file_entry_gtk_entry(prop_dlg.upload_dir)),
		      "changed", GTK_SIGNAL_FUNC(changed_cb), NULL);
   gtk_box_pack_start(GTK_BOX(prop_dlg.hbox), prop_dlg.upload_dir,
		      TRUE, TRUE, 0);

   prop_dlg.build_list = gtk_button_new_with_label(_("Build mp3 list"));
   gtk_signal_connect(GTK_OBJECT(prop_dlg.build_list), "clicked",
		      GTK_SIGNAL_FUNC(build_list_cb), NULL);
   j_box_pack(prop_dlg.hbox, prop_dlg.build_list, 0, 0, 0);

   prop_dlg.update_list = gtk_button_new_with_label(_("Update list"));
   gtk_signal_connect(GTK_OBJECT(prop_dlg.update_list), "clicked",
		      GTK_SIGNAL_FUNC(update_list_cb), NULL);
   j_box_pack(prop_dlg.hbox, prop_dlg.update_list, 0, 0, 0);

   j_property_box_append_page(prop_dlg.vbox,
			      gtk_label_new(_("User Information")));
   
   /* CONNECTION INFORMATION */
   
   prop_dlg.vbox = gtk_vbox_new(FALSE, 5);
   gtk_container_border_width(GTK_CONTAINER(prop_dlg.vbox), 0);
   
   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);

   prop_dlg.firewall = gtk_check_button_new_with_label(_("Firewalled without the ability to accept connections"));
   gtk_signal_connect(GTK_OBJECT(prop_dlg.firewall), "clicked",
		      GTK_SIGNAL_FUNC(changed_cb), NULL);
   j_box_pack(prop_dlg.hbox, prop_dlg.firewall, 0, 0, 0);
   
   prop_dlg.label = gtk_label_new("(required for SOCKS proxy)");
   j_box_pack(prop_dlg.hbox, prop_dlg.label, 0, 0, 0);
   
   vbox = gtk_vbox_new(FALSE, 5); //j_line_new(prop_dlg.vbox, 0, 0);
   gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
   
   prop_dlg.frame = gtk_frame_new("SOCKS (EXPERIMENTAL)");
   gtk_container_add(GTK_CONTAINER(prop_dlg.frame), vbox);
   j_box_pack(prop_dlg.vbox, prop_dlg.frame, 1, 1, 0);
   
   prop_dlg.hbox = j_line_new(vbox, 0, 0);
   
   prop_dlg.socks_firewall = gtk_check_button_new_with_label(_("SOCKS Proxy: "));
   gtk_signal_connect(GTK_OBJECT(prop_dlg.socks_firewall), "clicked",
		      GTK_SIGNAL_FUNC(changed_cb), NULL);
   j_box_pack(prop_dlg.hbox, prop_dlg.socks_firewall, 0, 0, 0);
   
   prop_dlg.hbox = j_line_new(vbox, 0, 0);
   
   prop_dlg.label = gtk_label_new(_("SOCKS Server IP: "));
   j_box_pack(prop_dlg.hbox, prop_dlg.label, 0, 0, 0);
   
   prop_dlg.socks_server = gtk_entry_new();
   gtk_signal_connect(GTK_OBJECT(prop_dlg.socks_server), "changed",
		      GTK_SIGNAL_FUNC(changed_cb), NULL);
   j_box_pack(prop_dlg.hbox, prop_dlg.socks_server, 0, 0, 0);
   
   prop_dlg.label = gtk_label_new(_(" Port: "));
   j_box_pack(prop_dlg.hbox, prop_dlg.label, 0, 0, 0);
   
   prop_dlg.socks_port = gtk_entry_new();
   gtk_signal_connect(GTK_OBJECT(prop_dlg.socks_port), "changed",
		      GTK_SIGNAL_FUNC(changed_cb), NULL);
   j_box_pack(prop_dlg.hbox, prop_dlg.socks_port, 0, 0, 0);
   
   prop_dlg.hbox = j_line_new(vbox, 0, 0);
   
   prop_dlg.socks4 = gtk_radio_button_new_with_label(NULL, _("SOCKS4"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prop_dlg.socks4),
				user_info.conf[SOCKS4]);
   gtk_signal_connect(GTK_OBJECT(prop_dlg.socks4), "clicked",
		      GTK_SIGNAL_FUNC(clicked_cb), &(user_info.conf[SOCKS4]));
   j_box_pack(prop_dlg.hbox, prop_dlg.socks4, 0, 0, 0);

   prop_dlg.hbox = j_line_new(vbox, 0, 0);
   
   prop_dlg.socks5 = gtk_radio_button_new_with_label(gtk_radio_button_group(GTK_RADIO_BUTTON(prop_dlg.socks4)),
						     _("SOCKS5"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prop_dlg.socks5),
				user_info.conf[SOCKS5]);
   
   gtk_signal_connect(GTK_OBJECT(prop_dlg.socks5), "clicked",
		      GTK_SIGNAL_FUNC(clicked_cb), &(user_info.conf[SOCKS5]));
   j_box_pack(prop_dlg.hbox, prop_dlg.socks5, 0, 0, 0);
   
   prop_dlg.hbox = j_line_new(vbox, 0, 0);

   prop_dlg.label = gtk_label_new("SOCKS5 Username: ");
   j_box_pack(prop_dlg.hbox, prop_dlg.label, 0, 0, 0);
   
   prop_dlg.socks5_username = gtk_entry_new();
   gtk_signal_connect(GTK_OBJECT(prop_dlg.socks5_username), "changed",
		      GTK_SIGNAL_FUNC(changed_cb), NULL);
   j_box_pack(prop_dlg.hbox, prop_dlg.socks5_username, 0, 0, 0);
   
   prop_dlg.hbox = j_line_new(vbox, 0, 0);
   
   prop_dlg.label = gtk_label_new("SOCKS5 Password: ");
   j_box_pack(prop_dlg.hbox, prop_dlg.label, 0, 0, 0);
   
   prop_dlg.socks5_password = gtk_entry_new();
   gtk_signal_connect(GTK_OBJECT(prop_dlg.socks5_password), "changed",
		      GTK_SIGNAL_FUNC(changed_cb), NULL);
   j_box_pack(prop_dlg.hbox, prop_dlg.socks5_password, 0, 0, 0);
      
   gnapster_config_toggle_widget(prop_dlg.firewall, "firewall", 
				 prop_dlg.socks_firewall, 
				 NULL);

   gnapster_config_toggle_widget(prop_dlg.socks_firewall, "socks_firewall",
				 prop_dlg.socks_server,
				 prop_dlg.socks_port,
				 prop_dlg.socks4,
				 prop_dlg.socks5,
				 prop_dlg.socks5_username,
				 prop_dlg.socks5_password,
				 NULL);

   j_property_box_append_page(prop_dlg.vbox,
			      gtk_label_new(_("Connection")));
   
   /* OPTIONS */

   prop_dlg.vbox = gtk_vbox_new(FALSE, 5);
   gtk_container_border_width(GTK_CONTAINER(prop_dlg.vbox), 5);
   
   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);
   
   prop_dlg.autoquery = gtk_check_button_new_with_label(_("Auto-query a user upon incoming message"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prop_dlg.autoquery),
				user_info.conf[AUTOQUERY]);
   gtk_signal_connect(GTK_OBJECT(prop_dlg.autoquery), "clicked",
		      GTK_SIGNAL_FUNC(clicked_cb), &(user_info.conf[AUTOQUERY]));
   j_box_pack(prop_dlg.hbox, prop_dlg.autoquery, 0, 0, 0);
   
   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);
   
   prop_dlg.name_fix = gtk_check_button_new_with_label(_("Convert spaces to underscores?"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prop_dlg.name_fix),
				user_info.conf[NAME_FIX]);
   gtk_signal_connect(GTK_OBJECT(prop_dlg.name_fix), "clicked",
		      GTK_SIGNAL_FUNC(clicked_cb), &(user_info.conf[NAME_FIX]));
   gtk_box_pack_start(GTK_BOX(prop_dlg.hbox), prop_dlg.name_fix,
		      FALSE, FALSE, 0);

   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);
   
   prop_dlg.hide_users = gtk_check_button_new_with_label(_("Auto-hide userlist on channel join"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prop_dlg.hide_users),
				user_info.conf[HIDE_USERS]);
   gtk_signal_connect(GTK_OBJECT(prop_dlg.hide_users), "clicked",
		      GTK_SIGNAL_FUNC(clicked_cb), &(user_info.conf[HIDE_USERS]));
   j_box_pack(prop_dlg.hbox, prop_dlg.hide_users, 0, 0, 0);
   
   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);
   
   prop_dlg.hide_joinpart = gtk_check_button_new_with_label(_("Hide join/part messages"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prop_dlg.hide_joinpart),
				user_info.conf[HIDE_JOINPART]);
   gtk_signal_connect(GTK_OBJECT(prop_dlg.hide_joinpart), "clicked",
		      GTK_SIGNAL_FUNC(clicked_cb), &(user_info.conf[HIDE_JOINPART]));
   j_box_pack(prop_dlg.hbox, prop_dlg.hide_joinpart, 0, 0, 0);
   
/*   prop_dlg.theme_colors = gtk_check_button_new_with_label(_("Use themes state colors for text widget\n(may cause problems with some themes)"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prop_dlg.theme_colors), 
				user_info.conf[THEME_COLORS]);
   gtk_signal_connect(GTK_OBJECT(prop_dlg.theme_colors), "clicked",
		      GTK_SIGNAL_FUNC(clicked_cb), &(user_info.conf[THEME_COLORS]));
   gtk_box_pack_start(GTK_BOX(prop_dlg.hbox), prop_dlg.theme_colors,
		      FALSE, FALSE, 0);*/
   
   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);
   
   prop_dlg.reject_ul = gtk_check_button_new_with_label(_("Reject uploads while downloads are active"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prop_dlg.reject_ul),
				user_info.conf[REJECT_UL]);
   gtk_signal_connect(GTK_OBJECT(prop_dlg.reject_ul), "clicked",
		      GTK_SIGNAL_FUNC(clicked_cb), &(user_info.conf[REJECT_UL]));
   j_box_pack(prop_dlg.hbox, prop_dlg.reject_ul, 0, 0, 0);
   
   j_property_box_append_page(prop_dlg.vbox,
			      gtk_label_new(_("Options")));
   
   /* LOGGING */
   
   prop_dlg.vbox = gtk_vbox_new(FALSE, 5);
   gtk_container_border_width(GTK_CONTAINER(prop_dlg.vbox), 5);
   
   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);
   
   prop_dlg.label = gtk_label_new("Note: Gnapster logs to ~/.gnapster/logs/");
   j_box_pack(prop_dlg.hbox, prop_dlg.label, 0, 0, 0);
   
   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);
   
   prop_dlg.logging = gtk_check_button_new_with_label(_("Log messages to disk"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prop_dlg.logging), 
				user_info.conf[LOGGING]);
   gtk_signal_connect(GTK_OBJECT(prop_dlg.logging), "clicked",
		      GTK_SIGNAL_FUNC(clicked_cb), &(user_info.conf[LOGGING]));
   j_box_pack(prop_dlg.hbox, prop_dlg.logging, 0, 0, 0);
      
   j_property_box_append_page(prop_dlg.vbox,
			      gtk_label_new(_("Logging")));
   
   /* XTEXT PREFS */
   
   prop_dlg.vbox = gtk_vbox_new(FALSE, 5);
   gtk_container_border_width(GTK_CONTAINER(prop_dlg.vbox), 5);
   
   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);
   
   prop_dlg.label = gtk_label_new("Font: ");
   j_box_pack(prop_dlg.hbox, prop_dlg.label, 0, 0, 0);
   
   prop_dlg.font_entry = gtk_entry_new();
   if (user_info.default_font)
     gtk_entry_set_text(GTK_ENTRY(prop_dlg.font_entry), user_info.default_font);
   gtk_signal_connect(GTK_OBJECT(prop_dlg.font_entry), "changed",
		      GTK_SIGNAL_FUNC(changed_cb), NULL);

   d_free(user_info.default_font);
   user_info.default_font = d_config_get_string("/gnapster/xtext/font");
   
   j_box_pack(prop_dlg.hbox, prop_dlg.font_entry, 1, 1, 0);
   
   prop_dlg.font_browse = gtk_button_new_with_label("Browse...");
   gtk_signal_connect(GTK_OBJECT(prop_dlg.font_browse), "clicked",
		      GTK_SIGNAL_FUNC(browse_font_cb), NULL);
   j_box_pack(prop_dlg.hbox, prop_dlg.font_browse, 0, 0, 0);
   
   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);
   
   prop_dlg.label = gtk_label_new("Note: changing font requires a client restart");
   j_box_pack(prop_dlg.hbox, prop_dlg.label, 0, 0, 0);
   
   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);
   
   prop_dlg.trans = gtk_check_button_new_with_label("Transparency");
   j_box_pack(prop_dlg.hbox, prop_dlg.trans, 0, 0, 0);
   
   prop_dlg.update_tint = gtk_button_new_with_label("Update XText");
   gtk_signal_connect(GTK_OBJECT(prop_dlg.update_tint), "clicked",
		      GTK_SIGNAL_FUNC(update_xtext), NULL);
   j_box_pack(prop_dlg.hbox, prop_dlg.update_tint, 0, 0, 0);
   
   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);
   
   prop_dlg.label = gtk_label_new("Red: ");
   j_box_pack(prop_dlg.hbox, prop_dlg.label, 0, 0, 0);
   
   prop_dlg.tint_red = 
     gtk_spin_button_new((GtkAdjustment *)gtk_adjustment_new(255.0, 0.0, 255.0, 1.0, 5.0, 0.0),
			 0, 0);
   gnapster_config_combo_widget(prop_dlg.tint_red,
				"tint_red");
   j_box_pack(prop_dlg.hbox, prop_dlg.tint_red, 0, 0, 0);
   
   prop_dlg.label = gtk_label_new("Green: ");
   j_box_pack(prop_dlg.hbox, prop_dlg.label, 0, 0, 0);
   
   prop_dlg.tint_green =
     gtk_spin_button_new((GtkAdjustment *)gtk_adjustment_new(255.0, 0.0, 255.0, 1.0, 5.0, 0.0),
			 0, 0);
   gnapster_config_combo_widget(prop_dlg.tint_green,
				"tint_green");
   j_box_pack(prop_dlg.hbox, prop_dlg.tint_green, 0, 0, 0);
   
   prop_dlg.label = gtk_label_new("Blue: ");
   j_box_pack(prop_dlg.hbox, prop_dlg.label, 0, 0, 0);
   
   prop_dlg.tint_blue = 
     gtk_spin_button_new((GtkAdjustment *)gtk_adjustment_new(255.0, 0.0, 255.0, 1.0, 5.0, 0.0),
			 0, 0);
   gnapster_config_combo_widget(prop_dlg.tint_blue,
				"tint_blue");
   j_box_pack(prop_dlg.hbox, prop_dlg.tint_blue, 0, 0, 0);
   
   gnapster_config_toggle_widget(prop_dlg.trans, "tint",
				 prop_dlg.tint_red, prop_dlg.tint_green,
				 prop_dlg.tint_blue, NULL);
   
   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);
      
   prop_dlg.label = gtk_label_new("Current theme: ");
   j_box_pack(prop_dlg.hbox, prop_dlg.label, 0, 0, 0);
   
   prop_dlg.curr_theme = gtk_entry_new();
   gtk_entry_set_editable(GTK_ENTRY(prop_dlg.curr_theme), 0);
   gtk_entry_set_text(GTK_ENTRY(prop_dlg.curr_theme), user_info.theme);
   gtk_signal_connect(GTK_OBJECT(prop_dlg.curr_theme), "changed",
		      GTK_SIGNAL_FUNC(changed_cb), NULL);
   j_box_pack(prop_dlg.hbox, prop_dlg.curr_theme, 1, 1, 0);

   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 1, 1);
   
   prop_dlg.scroller = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(prop_dlg.scroller),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
   j_box_pack(prop_dlg.hbox, prop_dlg.scroller, 1, 1, 0);
   
   prop_dlg.themes = gtk_clist_new_with_titles(2, themes_title);
   gtk_clist_set_column_width(GTK_CLIST(prop_dlg.themes), 0, 150);
   populate_themes_clist();
   gtk_signal_connect(GTK_OBJECT(prop_dlg.themes), "select-row",
		      GTK_SIGNAL_FUNC(themes_select_row), NULL);
   gtk_container_add(GTK_CONTAINER(prop_dlg.scroller), prop_dlg.themes);
   
   j_property_box_append_page(prop_dlg.vbox,
			      gtk_label_new(_("Display")));
   /* AUTO-JOIN */
   
   prop_dlg.vbox = gtk_vbox_new(FALSE, 5);
   gtk_container_border_width(GTK_CONTAINER(prop_dlg.vbox), 5);
   
   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);
   
   prop_dlg.aj_entry = gtk_entry_new();
   gtk_signal_connect(GTK_OBJECT(prop_dlg.aj_entry), "activate",
		      GTK_SIGNAL_FUNC(autojoin_add), NULL);
   j_box_pack(prop_dlg.hbox, prop_dlg.aj_entry, 1, 1, 0);
   
   prop_dlg.aj_add = gtk_button_new_with_label("Add");
   gtk_signal_connect(GTK_OBJECT(prop_dlg.aj_add), "clicked",
		      GTK_SIGNAL_FUNC(autojoin_add), NULL);
   j_box_pack(prop_dlg.hbox, prop_dlg.aj_add, 0, 0, 0);
   
   prop_dlg.aj_remove = gtk_button_new_with_label("Remove");
   gtk_signal_connect(GTK_OBJECT(prop_dlg.aj_remove), "clicked",
		      GTK_SIGNAL_FUNC(autojoin_remove), NULL);
   j_box_pack(prop_dlg.hbox, prop_dlg.aj_remove, 0, 0, 0);
   
   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 1, 1);
   
   prop_dlg.aj_clist = gtk_clist_new(1);
   populate_autojoin_clist();
   j_box_pack(prop_dlg.hbox, prop_dlg.aj_clist, 1, 1, 0);
   
   j_property_box_append_page(prop_dlg.vbox,
			      gtk_label_new(_("Auto-Join")));

   d_free(user);
   d_free(pass);
   d_free(download_dir);
   d_free(upload_dir);
   
#ifndef HAVE_GNOME
   j_dialog_show(prop_dlg.window, NULL);
#else
   /* simulate a j_dialog_new */
   dialogs = g_list_append(dialogs, GTK_WIDGET(prop_dlg.window));
   
   gtk_signal_connect(GTK_OBJECT(prop_dlg.window), "apply",
		      GTK_SIGNAL_FUNC(preferences_button_cb), "apply");
   gtk_signal_connect(GTK_OBJECT(prop_dlg.window), "destroy",
		      GTK_SIGNAL_FUNC(preferences_destroy_cb), "destroy");   
#endif /* !HAVE_GNOME */
}

void create_build_dlg() {
   build_dlg.window = j_dialog_new(_("Building MP3 list..."),
				   "Button_Cancel",
				   NULL);
   
   gtk_widget_set_usize(build_dlg.window, 250, -1);
   
   build_dlg.vbox = j_dialog_vbox(build_dlg.window);
  
   build_dlg.hbox = j_line_new(build_dlg.vbox, 0, 0);
   
   build_dlg.label = gtk_label_new("Building list...");
   j_box_pack(build_dlg.hbox, build_dlg.label, 0, 0, 0);
   
   build_dlg.hbox = j_line_new(build_dlg.vbox, 0, 0);
   
   build_dlg.label = gtk_label_new("");
   j_box_pack(build_dlg.hbox, build_dlg.label, 0, 0, 0);
   
   j_dialog_button_connect(build_dlg.window, 0,
			   build_dlg_cancel_cb, NULL);
   
   j_dialog_show(build_dlg.window, build_dlg.vbox);
}

void create_browse_dlg() {
   char *titles[] = { "Server title", "Address", NULL };
   
   
   browse_dlg.window = j_dialog_new(_("Browse servers"),
				    "Connect",
				    "Button_Cancel", 
				    NULL);

   gtk_widget_set_usize(browse_dlg.window, 450, 300);

   browse_dlg.vbox = j_dialog_vbox(browse_dlg.window);

   browse_dlg.hbox = j_line_new(browse_dlg.vbox, 0, 0);
   
   browse_dlg.add = gtk_button_new_with_label("Add server");
   gtk_signal_connect(GTK_OBJECT(browse_dlg.add), "clicked",
		      GTK_SIGNAL_FUNC(browse_dlg_add), NULL);
   j_box_pack(browse_dlg.hbox, browse_dlg.add, 0, 0, 0);
   
   browse_dlg.edit = gtk_button_new_with_label("Edit server");
   gtk_signal_connect(GTK_OBJECT(browse_dlg.edit), "clicked",
		      GTK_SIGNAL_FUNC(browse_dlg_edit), NULL);
   j_box_pack(browse_dlg.hbox, browse_dlg.edit, 0, 0, 0);
   
   browse_dlg.delete = gtk_button_new_with_label("Delete server");
   gtk_signal_connect(GTK_OBJECT(browse_dlg.delete), "clicked",
		      GTK_SIGNAL_FUNC(browse_dlg_delete), NULL);
   j_box_pack(browse_dlg.hbox, browse_dlg.delete, 0, 0, 0);
   
   browse_dlg.refresh = gtk_button_new_with_label("Refresh list");
   gtk_signal_connect(GTK_OBJECT(browse_dlg.refresh), "clicked",
		      GTK_SIGNAL_FUNC(browse_dlg_refresh), NULL);
   j_box_pack(browse_dlg.hbox, browse_dlg.refresh, 0, 0, 0);
   
   browse_dlg.trefresh = gtk_check_button_new_with_label(_("Remove server list before refresh"));
   gnapster_config_toggle_widget(browse_dlg.trefresh,
				 "list_refresh", NULL);
   j_box_pack(browse_dlg.hbox, browse_dlg.trefresh, 0, 0, 0);

   browse_dlg.hbox = j_line_new(browse_dlg.vbox, 0, 0);
   
   browse_dlg.label = gtk_label_new("Show Network: ");
   j_box_pack(browse_dlg.hbox, browse_dlg.label, 0, 0, 0);
   
   browse_dlg.show_network = gtk_combo_new();
   j_box_pack(browse_dlg.hbox, browse_dlg.show_network, 1, 1, 0);
   
   browse_dlg.hbox = j_line_new(browse_dlg.vbox, 1, 1);

   browse_dlg.scroller = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(browse_dlg.scroller),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
   j_box_pack(browse_dlg.hbox, browse_dlg.scroller, 1, 1, 0);
   
   browse_dlg.ctree = gtk_ctree_new_with_titles(2, 0, titles);
   gtk_clist_set_column_width(GTK_CLIST(browse_dlg.ctree), 0, 275);
   gtk_clist_set_column_width(GTK_CLIST(browse_dlg.ctree), 1, 75);
   gtk_signal_connect(GTK_OBJECT(browse_dlg.ctree), "button_press_event",
		      GTK_SIGNAL_FUNC(browse_dlg_button_cb), NULL);
   browse_dlg.show_network_id = -1;
   
   browse_dlg.ctree_mem = gtk_ctree_new(2, 0);   
   populate_browse_dlg_clist();
   network_changed(NULL, NULL);
   
   gtk_container_add(GTK_CONTAINER(browse_dlg.scroller), browse_dlg.ctree);

   gtk_signal_connect(GTK_OBJECT(GTK_COMBO(browse_dlg.show_network)->entry), "changed",
		      GTK_SIGNAL_FUNC(network_changed), NULL);
   
   j_dialog_button_connect(browse_dlg.window, 0,
			   browse_dlg_connect_cb, NULL);
   
   j_dialog_button_connect(browse_dlg.window, 1,
			   browse_dlg_cancel_cb, NULL);
   
   gtk_signal_connect(GTK_OBJECT(browse_dlg.window), "delete_event",
		      browse_dlg_cancel_cb, NULL);

   j_dialog_show(browse_dlg.window, browse_dlg.vbox);
}

void create_refresh_dlg() {
   refresh_dlg.window = j_dialog_new(_("Refresh server list"),
				     "Button_Cancel",
				     NULL);
   
   refresh_dlg.vbox = j_dialog_vbox(refresh_dlg.window);
   
   refresh_dlg.hbox = j_line_new(refresh_dlg.vbox, 0, 0);
   
   refresh_dlg.label = gtk_label_new("Refreshing server list from\nhttp://www.napigator.com/servers.php?version=107...");
   j_box_pack(refresh_dlg.hbox, refresh_dlg.label, 0, 0, 0);
   
   j_dialog_button_connect(refresh_dlg.window, 0,
			   refresh_dlg_cb, NULL);
   
   j_dialog_show(refresh_dlg.window, refresh_dlg.vbox);
}

void create_add_dlg() {
   add_dlg.window = j_dialog_new(_("Add server"),
				 "Button_Ok",
				 "Button_Cancel",
				 NULL);

   add_dlg.vbox = j_dialog_vbox(add_dlg.window);
   
   add_dlg.hbox = j_line_new(add_dlg.vbox, 0, 0);

   add_dlg.label = gtk_label_new("IP: ");
   j_box_pack(add_dlg.hbox, add_dlg.label, 0, 0, 0);
   
   add_dlg.server = gtk_entry_new();
   j_box_pack(add_dlg.hbox, add_dlg.server, 1, 1, 0);   

   add_dlg.hbox = j_line_new(add_dlg.vbox, 0, 0);

   add_dlg.label = gtk_label_new("Port: ");
   j_box_pack(add_dlg.hbox, add_dlg.label, 0, 0, 0);
   
   add_dlg.port = gtk_entry_new();
   gtk_widget_set_usize(add_dlg.port, 40, -1);
   j_box_pack(add_dlg.hbox, add_dlg.port, 1, 1, 0);
   
   add_dlg.hbox = j_line_new(add_dlg.vbox, 0, 0);

   add_dlg.label = gtk_label_new("Description: ");
   j_box_pack(add_dlg.hbox, add_dlg.label, 0, 0, 0);
   
   add_dlg.desc = gtk_entry_new();
   j_box_pack(add_dlg.hbox, add_dlg.desc, 1, 1, 0);
   
   add_dlg.hbox = j_line_new(add_dlg.vbox, 0, 0);
   
   add_dlg.label = gtk_label_new("Network: ");
   j_box_pack(add_dlg.hbox, add_dlg.label, 0, 0, 0);
   
   add_dlg.network = gtk_entry_new();
   j_box_pack(add_dlg.hbox, add_dlg.network, 1, 1, 0);
   
   j_dialog_button_connect(add_dlg.window, 0,
			   add_dlg_cb, "ok");
   j_dialog_button_connect(add_dlg.window, 1,
			   add_dlg_cb, "cancel");

   j_dialog_show(add_dlg.window, add_dlg.vbox);
}

void create_edit_dlg(Server *nsrv, char *desc) {
   char *port;
   
   if (!nsrv || !desc) 
     return;
   
   edit_dlg.window = j_dialog_new(_("Edit server"),
				  "Button_Ok",
				  "Button_Cancel",
				  NULL);

   edit_dlg.vbox = j_dialog_vbox(edit_dlg.window);
   
   edit_dlg.hbox = j_line_new(edit_dlg.vbox, 0, 0);

   edit_dlg.label = gtk_label_new("IP: ");
   j_box_pack(edit_dlg.hbox, edit_dlg.label, 0, 0, 0);
   
   edit_dlg.server = gtk_entry_new();
   gtk_entry_set_text(GTK_ENTRY(edit_dlg.server), nsrv->ip);
   j_box_pack(edit_dlg.hbox, edit_dlg.server, 1, 1, 0);   

   edit_dlg.hbox = j_line_new(edit_dlg.vbox, 0, 0);

   edit_dlg.label = gtk_label_new("Port: ");
   j_box_pack(edit_dlg.hbox, edit_dlg.label, 0, 0, 0);
   
   edit_dlg.port = gtk_entry_new();
   port = d_strdup_printf(DEBUG "%i", nsrv->port);
   gtk_entry_set_text(GTK_ENTRY(edit_dlg.port), port);
   d_free(port);
   gtk_widget_set_usize(edit_dlg.port, 40, -1);
   j_box_pack(edit_dlg.hbox, edit_dlg.port, 1, 1, 0);
   
   edit_dlg.hbox = j_line_new(edit_dlg.vbox, 0, 0);

   edit_dlg.label = gtk_label_new("Description: ");
   j_box_pack(edit_dlg.hbox, edit_dlg.label, 0, 0, 0);
   
   edit_dlg.desc = gtk_entry_new();
   gtk_entry_set_text(GTK_ENTRY(edit_dlg.desc), desc);
   j_box_pack(edit_dlg.hbox, edit_dlg.desc, 1, 1, 0);

   edit_dlg.hbox = j_line_new(edit_dlg.vbox, 0, 0);
   
   edit_dlg.label = gtk_label_new("Network: ");
   j_box_pack(edit_dlg.hbox, edit_dlg.label, 0, 0, 0);
   
   edit_dlg.network = gtk_entry_new();
   gtk_entry_set_text(GTK_ENTRY(edit_dlg.network), nsrv->network);
   j_box_pack(edit_dlg.hbox, edit_dlg.network, 1, 1, 0);
   
   j_dialog_button_connect(edit_dlg.window, 0,
			   edit_dlg_cb, nsrv);
   j_dialog_button_connect(edit_dlg.window, 1,
			   edit_dlg_cb, NULL);
   
   j_dialog_show(edit_dlg.window, edit_dlg.vbox);
}

void create_list_chan_dlg() {
   char *titles[] = { "Channel", "Users", "Topic", NULL };
   
   list_chan_dlg.window = j_dialog_new(_("Channel Listing"),
				       "Join",
				       "Button_Ok",
				       NULL);
   
   gtk_widget_set_usize(list_chan_dlg.window, 400 + 150, 300 + 80);
   
   list_chan_dlg.vbox = j_dialog_vbox(list_chan_dlg.window);
   
   list_chan_dlg.hbox = j_line_new(list_chan_dlg.vbox, 0, 0);
   
   list_chan_dlg.label = gtk_label_new(_("Requesting channel listing..."));
   j_box_pack(list_chan_dlg.hbox, list_chan_dlg.label, 0, 0, 0);
   
   list_chan_dlg.hbox = j_line_new(list_chan_dlg.vbox, 1, 1);

   list_chan_dlg.scroller = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(list_chan_dlg.scroller),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
   j_box_pack(list_chan_dlg.hbox, list_chan_dlg.scroller, 1, 1, 0);

   list_chan_dlg.clist = gtk_clist_new_with_titles(3, titles);
   gtk_clist_set_column_width(GTK_CLIST(list_chan_dlg.clist), 0, 100);
   gtk_clist_set_column_width(GTK_CLIST(list_chan_dlg.clist), 2, 50);
   gtk_clist_set_column_width(GTK_CLIST(list_chan_dlg.clist), 3, 150);
   gtk_signal_connect(GTK_OBJECT(list_chan_dlg.clist), "button_press_event",
		      GTK_SIGNAL_FUNC(list_chan_dlg_clist_cb), NULL);
   gtk_container_add(GTK_CONTAINER(list_chan_dlg.scroller), list_chan_dlg.clist);
   
   j_dialog_button_connect(list_chan_dlg.window, 0,
			   list_chan_dlg_cb, "join");
   j_dialog_button_connect(list_chan_dlg.window, 1,
			   list_chan_dlg_cb, "ok");
   
   j_dialog_show(list_chan_dlg.window, list_chan_dlg.vbox);
}

void create_resume_dlg() {
   char *titles[] = { "Filename", "Remote size", NULL };
   
   resume_dlg.window = j_dialog_new(_("Resume Download"),
				    "Resume",
				    "Remove",
				    "Button_Cancel",
				    NULL);
   
   gtk_widget_set_usize(resume_dlg.window, 400, 300);
   
   resume_dlg.vbox = j_dialog_vbox(resume_dlg.window);

   resume_dlg.hbox = j_line_new(resume_dlg.vbox, 0, 0);
   
   resume_dlg.label = gtk_label_new(_(""));
   j_box_pack(resume_dlg.hbox, resume_dlg.label, 0, 0, 0);
   
   resume_dlg.hbox = j_line_new(resume_dlg.vbox, 1, 1);

   resume_dlg.scroller = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(resume_dlg.scroller),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
   j_box_pack(resume_dlg.hbox, resume_dlg.scroller, 1, 1, 0);

   resume_dlg.clist = gtk_clist_new_with_titles(2, titles);
   gtk_clist_set_column_width(GTK_CLIST(resume_dlg.clist), 0, 275);
   gtk_clist_set_column_width(GTK_CLIST(resume_dlg.clist), 2, 50);
   gtk_signal_connect(GTK_OBJECT(resume_dlg.clist), "button_press_event",
		      GTK_SIGNAL_FUNC(resume_dlg_clist_cb), NULL);
   gtk_container_add(GTK_CONTAINER(resume_dlg.scroller), resume_dlg.clist);

   gtk_signal_connect(GTK_OBJECT(resume_dlg.window), "delete_event",
		      resume_dlg_cb, "ok");

   j_dialog_button_connect(resume_dlg.window, 0,
			   resume_dlg_cb, "resume");
   j_dialog_button_connect(resume_dlg.window, 1,
			   resume_dlg_cb, "remove");
   j_dialog_button_connect(resume_dlg.window, 2,
			   resume_dlg_cb, "ok");
   
   j_dialog_show(resume_dlg.window, resume_dlg.vbox);
}
